/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.common;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * ConfigManager.java
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 25, 2008 10:12:12 AM
 */
public class ConfigManager {
	
	/* ******************* */
	/* * Path properties * */
	/* ******************* */

	// properties for System
	/** The start repository of the application */
	public static final String LOCAL_DIRECTORY = System.getProperty("user.dir");

	/** The file separator of the system */
	public static final String FILE_SEPARATOR = System.getProperty("file.separator");
	
	/** The repository where will be stored the configuration file */
	public static final String CONFIG_DIR = LOCAL_DIRECTORY + FILE_SEPARATOR
			+ "config";

	/**
	 * The separator used to separate the name of the process to the name of the
	 * toProcess tag for data binding
	 */
	public static final String PROCESS_SEPARATOR = "@";


	// name of the different folders

	/** The path of the folder where the ontology are stored */
	private String ONTOLOGY_FOLDER = LOCAL_DIRECTORY + FILE_SEPARATOR
			+ "ontologies";

	/** The path of the folder where the generated bpel files will be stored */
	private String BPEL_FOLDER = LOCAL_DIRECTORY + FILE_SEPARATOR
			+ "bpel_generatedFiles";

	/** The path of the folder where the generated repository will be stored */
	private String REPOSITORY_FOLDER = LOCAL_DIRECTORY + FILE_SEPARATOR
			+ "repository";

	/** The path of the folder where the data of the repository will be stored */
	private String REPOSITORY_DATA_FOLDER = this.getRepositoryFolder()
			+ FILE_SEPARATOR + "data" + FILE_SEPARATOR;

	/** The path of the folder where the index of the repository will be stored */
	private String REPOSITORY_INDEX_FOLDER = this.getRepositoryFolder()
			+ FILE_SEPARATOR + "index" + FILE_SEPARATOR;

	/** The name of the file containing the index of services */
	public String REPOSITORY_INDEX_SERVICE = this.getRepositoryIndexFolder()
			+ "index_service";

	/** The name of the file containing the index of WSDL files */
	public String REPOSITORY_INDEX_WSDL = this.getRepositoryIndexFolder()
			+ "index_wsdl";

	/**
	 * The name of the file containing the index of services indexed with the
	 * filename
	 */
	public String REPOSITORY_INDEX_SERVICE_REVERT = this.getRepositoryIndexFolder()
			+ "index_service_revert";

	/**
	 * The name of the file containing the index of WSDL files indexed on the
	 * filename
	 */
	public String REPOSITORY_INDEX_WSDL_REVERT = this.getRepositoryIndexFolder()
			+ "index_index_wsld_revert";

	/**
	 * The name of the file containing a dump of the map between the service and
	 * the WSDL file
	 */
	public String REPOSITORY_INDEX_SERVICE_WSDL = this.getRepositoryIndexFolder()
			+ "index_service_wsdl";

	/**
	 * The name of the file containing a dump of the map between the capability
	 * and the owl file inside it appears
	 */
	public String REPOSITORY_INDEX_CAPABILITIES_FILE = this.getRepositoryIndexFolder()
			+ "index_capabilities_files";

	/**
	 * The name of the file containing a dump of the map containing Service
	 * Object
	 */
	public String REPOSITORY_SERVICE_OBJECT = this.getRepositoryIndexFolder()
			+ "index_services_object";

	/**
	 * The path of the folder where the source code files for new operation type
	 * are stored
	 */
	public String OPERATION_TYPE_FOLDER = LOCAL_DIRECTORY + FILE_SEPARATOR
			+ "OperationType";

	/** The path of the folder where the library are stored */
	public String LIB_FOLDER = LOCAL_DIRECTORY + FILE_SEPARATOR + "lib";

	/** The path of the folder where temp files will be stored */
	private String TEMP_FOLDER = LOCAL_DIRECTORY + FILE_SEPARATOR + "tmp";

	

	/** The name of the file containing configuration properties */
	private static final String CONFIG_FILE_NAME = "config.xml";

		
	/* **************************** */
	/* * Singloton Design Pattern * */
	/* **************************** */

	/** The global instance of the object */
	private static ConfigManager currentManager = null;

	/**
	 * Default Constructor of the Manager
	 * 
	 */
	private ConfigManager() {
		currentManager = this;

		// initialize fields thanks to the configuration file
		this.initialize();
	}
	
	/**
	 * This method permit to get a reference of the singloton object containing
	 * all propertt needed for all process
	 * 
	 * @return the singloton object PropertyManager
	 */
	public static ConfigManager getInstance() {
		if (currentManager == null) {
			return new ConfigManager();
		} else {
			return currentManager;
		}
	}
	

	/* ************************** */
	/* * Bpel Server properties * */
	/* ************************** */

	/** The address of the bpel server */
	private String BPEL_SERVER = "localhost:9080";

	/**
	 * The address address of the deployement service for bpel script on the
	 * server
	 */
	private String BPEL_DEPLOYEMENT_SERVICE = "/active-bpel/services/DeployBPRService";

	/** The name of the operation of deployement of a bpel script */
	public String BPEL_DEPLOYEMENT_OPERATIONAME = "deployBpr";

	/** The address of the bpel server */
	private String underlyingWFEngineURL = "http://" + BPEL_SERVER;

	/** The address where the generated service will be deployed */
	private String generated_bpel_services_base_address = "http://"
			+ BPEL_SERVER + "/active-bpel/services";

	/* Debug Mode */

	/** Indicate if we work in a debug mode or not */
	private boolean debugMode = true;

	/**
	 * Indicate the debug level ( 0 - debug, 1 - info, 2 - warning, 3 - error, 4 -
	 * fatal ) that will be displayed
	 */
	private int debugLevel = 0;

	/* Service Discovery */

	/** Indicate which class has to be used has ServiceDiscovery */
	private String serviceDiscoveryClassName = "fr.inria.icocoa.semanticcomposition.servicediscovery.SD";
	
	/* **************************** */
	/* * Path Calculation Methods * */
	/* **************************** */

	/**
	 * Get the repository path. If it doesn't exist, the folder is created
	 * 
	 * @return the path of the folder containing all ontology
	 */
	public String getOntologyFolder() {
		File f = new File(ONTOLOGY_FOLDER);
		if (!f.exists()) {
			f.mkdir();
		}
		return ONTOLOGY_FOLDER;
	}

	/**
	 * Get the repository path. If it doesn't exist, the folder is created
	 * 
	 * @return the path of the folder where the bpel-scripts will be generated
	 */
	public String getBpelFolder() {
		File f = new File(BPEL_FOLDER);
		if (!f.exists()) {
			f.mkdir();
		}
		return BPEL_FOLDER;
	}

	/**
	 * Get the repository path. If it doesn't exist, the folder is created
	 * 
	 * @return the path of the folder where the repository will be generated
	 */
	public String getRepositoryFolder() {
		File f = new File(REPOSITORY_FOLDER);
		if (!f.exists()) {
			f.mkdir();
		}
		return REPOSITORY_FOLDER;
	}

	/**
	 * Get the repository path. If it doesn't exist, the folder is created
	 * 
	 * @return the path of the folder where the repository index will be
	 *         generated
	 */
	public String getRepositoryIndexFolder() {
		File f = new File(REPOSITORY_INDEX_FOLDER);
		if (!f.exists()) {
			f.mkdir();
		}
		return REPOSITORY_INDEX_FOLDER;
	}

	/**
	 * Get the repository path. If it doesn't exist, the folder is created
	 * 
	 * @return the path of the folder where the repository data will be stored
	 */
	public String getRepositoryDataFolder() {
		File f = new File(REPOSITORY_DATA_FOLDER);
		if (!f.exists()) {
			f.mkdir();
		}
		return REPOSITORY_DATA_FOLDER;
	}

	/**
	 * Get the repository path. If it doesn't exist, the folder is created
	 * 
	 * @return the path of the folder where the temporary files will be add
	 */
	public String getTempFolder() {
		File f = new File(TEMP_FOLDER);
		if (!f.exists()) {
			f.mkdir();
		}
		return TEMP_FOLDER;
	}

	/**
	 * Get the adress of the Deployment service of the BPEL Server
	 * 
	 * @return the adress of the Deployment service of the BPEL Server (URL)
	 */
	public String getBpelDeploymentServiceAddress() {
		return "http://" + this.BPEL_SERVER + this.BPEL_DEPLOYEMENT_SERVICE;
	}

	/* ****************** */
	/* * List of getter * */
	/* ****************** */

	/**
	 * @return the debugLevel
	 */
	public int getDebugLevel() {
		return debugLevel;
	}

	/**
	 * @param debugLevel
	 *            the debugLevel to set
	 */
	public void setDebugLevel(int debugLevel) {
		this.debugLevel = debugLevel;
	}

	/**
	 * @return the debugMode
	 */
	public boolean isDebugMode() {
		return debugMode;
	}

	/**
	 * @param debugMode
	 *            the debugMode to set
	 */
	public void setDebugMode(boolean debugMode) {
		this.debugMode = debugMode;
	}

	/**
	 * @return the serviceDiscoveryClassName
	 */
	public String getServiceDiscoveryClassName() {
		return serviceDiscoveryClassName;
	}

	/**
	 * @param serviceDiscoveryClassName
	 *            the serviceDiscoveryClassName to set
	 */
	public void setServiceDiscoveryClassName(String serviceDiscoveryClassName) {
		this.serviceDiscoveryClassName = serviceDiscoveryClassName;
	}

	/**
	 * @return the underlyingWFEngineURL
	 */
	public String getUnderlyingWFEngineURL() {
		return underlyingWFEngineURL;
	}

	/**
	 * @param underlyingWFEngineURL
	 *            the underlyingWFEngineURL to set
	 */
	public void setUnderlyingWFEngineURL(String underlyingWFEngineURL) {
		this.underlyingWFEngineURL = underlyingWFEngineURL;
	}

	/**
	 * @return the generated_bpel_services_base_address
	 */
	public String getGenerated_bpel_services_base_address() {
		return generated_bpel_services_base_address;
	}

	/**
	 * @param generated_bpel_services_base_address
	 *            the generated_bpel_services_base_address to set
	 */
	public void setGenerated_bpel_services_base_address(
			String generated_bpel_services_base_address) {
		this.generated_bpel_services_base_address = generated_bpel_services_base_address;
	}
	
	/* ****************** */
	/* * Initialisation * */
	/* ****************** */

	/* List of searched Tag */

	/** Tag in the config file that give the address of the bpel server */
	private static final String BPEL_SERVER_ADRESS_TAG = "bpel_server_address";

	/**
	 * Tag in the config file that give the address address of the deployement
	 * service for bpel script on the server
	 */
	private static final String BPEL_DEPLOYEMENT_SERVICE_TAG = "bpel_deployement_service";

	/**
	 * Tag in the config file that name of the operation of deployement of a
	 * bpel script
	 */
	private static final String BPEL_DEPLOYEMENT_OPERATION_TAG = "bpel_deployement_operation";

	/**
	 * Tag in the config file that give the path of the folder where the
	 * ontology are stored
	 */
	private static final String ONTOLOGY_FOLDER_PATH_TAG = "ontology_folder_path";

	/**
	 * Tag in the config file that give the path of the folder where the
	 * generated bpel files will be stored
	 */
	private static final String BPEL_FOLDER_PATH_TAG = "bpel_folder_path";

	/**
	 * Tag in the config file that give the path of the folder where the
	 * generated repository will be stored
	 */
	private static final String REPOSITORY_FOLDER_PATH_TAG = "repository_folder_path";

	/**
	 * Tag in the config file that give the path of the folder where the library
	 * are stored
	 */
	private static final String LIB_FOLDER_PATH_TAG = "lib_folder_path";

	/**
	 * Tag in the config file that give the path of the folder where temp files
	 * will be stored
	 */
	private static final String TMP_FOLDER_PATH_TAG = "tmp_folder_path";

	/**
	 * Tag in the config file that give the path of the folder where the source
	 * code files for new operation type are stored
	 */
	private static final String OPERATIONTYPE_FOLDER_PATH_TAG = "operation_folder_path";

	/**
	 * Tag in the config file that give The path of the configuration path for
	 * log4j
	 */
	private static String LOG4J_FILE_PATH_TAG = "log4j_configuration_file_path";
	
	/**
	 * Tag in the config file that give value to add in the map of system properties
	 */
	private static String SYSTEM_PROPERTY_TAG = "system_property";
	
	/**
	 * The map that contains the list of system properties added
	 */
	private Map<String,String> property_added = new HashMap<String,String>();

	/**
	 * Initialisation of the object PropertyManager by reading the configuration
	 * File
	 */
	private void initialize() {
		File configFile = new File(CONFIG_DIR + FILE_SEPARATOR
				+ CONFIG_FILE_NAME);

		XMLReader myReader;
		try {
			myReader = XMLReaderFactory.createXMLReader();

			// creation of the SAX handler
			DefaultHandler handler = new DefaultHandler() {

				/** The current tag */
				private String currentTag = null;
				

				public void startElement(String uri, String localName,
						String qName, Attributes attributes)
						throws SAXException {
					this.currentTag = localName;
					if(currentTag.equals(SYSTEM_PROPERTY_TAG)){
						String key = null;
						String value = null;
						for(int i=0; i<attributes.getLength();i++){
							//DebugPrinter.print(attributes.getQName(i)+" = "+attributes.getValue(i),DebugPrinter.INFO);
							if(attributes.getQName(i).equals("key"))
								key = attributes.getValue(i);
							if(attributes.getQName(i).equals("value"))
								value = attributes.getValue(i);
						}
						if(key != null && value != null){
							System.setProperty(key, value);
							property_added.put(key, value);
						}
					}
				}

				public void characters(char[] ch, int start, int length) {
					String s = (new String(ch, start, length)).trim();
					if (!s.equals("")) {
						if (currentTag.equals(BPEL_SERVER_ADRESS_TAG)) {
							BPEL_SERVER = s;
							underlyingWFEngineURL = "http://" + BPEL_SERVER;
							generated_bpel_services_base_address = "http://"
									+ BPEL_SERVER + "/active-bpel/services";
						} else if (currentTag.equals(BPEL_DEPLOYEMENT_SERVICE_TAG)) {
							BPEL_DEPLOYEMENT_SERVICE = s;
						} else if (currentTag.equals(BPEL_DEPLOYEMENT_OPERATION_TAG)) {
							BPEL_DEPLOYEMENT_OPERATIONAME = s;
						} else if (currentTag.equals(ONTOLOGY_FOLDER_PATH_TAG)) {
							ONTOLOGY_FOLDER = checkStarterAndChangeIt(s);
						} else if (currentTag.equals(BPEL_FOLDER_PATH_TAG)) {
							BPEL_FOLDER = checkStarterAndChangeIt(s);
						} else if (currentTag.equals(REPOSITORY_FOLDER_PATH_TAG)) {
							REPOSITORY_FOLDER = checkStarterAndChangeIt(s);
							REPOSITORY_DATA_FOLDER = getRepositoryFolder()
									+ FILE_SEPARATOR + "data" + FILE_SEPARATOR;
							REPOSITORY_INDEX_FOLDER = getRepositoryFolder()
									+ FILE_SEPARATOR + "index" + FILE_SEPARATOR;
							REPOSITORY_INDEX_SERVICE = getRepositoryIndexFolder()
									+ "index_service";
							REPOSITORY_INDEX_WSDL = getRepositoryIndexFolder()
									+ "index_wsdl";
							REPOSITORY_INDEX_SERVICE_WSDL = getRepositoryIndexFolder()
									+ "index_service_wsdl";
							REPOSITORY_INDEX_CAPABILITIES_FILE = getRepositoryIndexFolder()
									+ "index_capabilities_files";
							REPOSITORY_INDEX_WSDL_REVERT = getRepositoryIndexFolder()
									+ "index_index_wsld_revert";
							REPOSITORY_INDEX_SERVICE_REVERT = getRepositoryIndexFolder()
									+ "index_service_revert";
							REPOSITORY_SERVICE_OBJECT = getRepositoryIndexFolder()
									+ "index_services_object";

						} else if (currentTag.equals(LIB_FOLDER_PATH_TAG)) {
							LIB_FOLDER = checkStarterAndChangeIt(s);
						} else if (currentTag.equals(TMP_FOLDER_PATH_TAG)) {
							TEMP_FOLDER = checkStarterAndChangeIt(s);
						} else if (currentTag.equals(OPERATIONTYPE_FOLDER_PATH_TAG)) {
							OPERATION_TYPE_FOLDER = checkStarterAndChangeIt(s);
						} else if (currentTag.equals(LOG4J_FILE_PATH_TAG)) {
							DebugPrinter.loadLog4JPropertyFile(checkStarterAndChangeIt(s));
						} 

					}
				}

			};

			// put the handler in the sax machine
			myReader.setContentHandler(handler);
			myReader.setErrorHandler(handler);

			// parse the file
			myReader.parse(new InputSource(new FileReader(configFile)));

		} catch (SAXException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 4);
			System.exit(1);
		} catch (FileNotFoundException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 4);
			System.exit(1);
		} catch (IOException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 4);
			System.exit(1);
		}

		// DebugPrinter.print(PropertyManager.getInstance().myToString(),1);		
		DebugPrinter.print("Configuration file loaded", DebugPrinter.INFO);
		//DebugPrinter.print(property_added,"properties_added",DebugPrinter.INFO);
		//SemanticTypeFactory.clearInstance();
	}

	/**
	 * Check if the string start with a dot. in this case, change it with
	 * LOCAL_DIRECTORY
	 * 
	 * @param s :
	 *            the string to check
	 * @return the modified string is necessary, s otherwise
	 */
	private static String checkStarterAndChangeIt(String s) {
		if (s.startsWith(".")) {
			String result = s.substring(2);
			return LOCAL_DIRECTORY
					+ FILE_SEPARATOR
					+ result.replace('/', FILE_SEPARATOR.toCharArray()[0]).replace(
						'\\', FILE_SEPARATOR.toCharArray()[0]);
		} else {
			return s;
		}
	}

}
