/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.common;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Appender;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * DebugPrinter.java This class is composed with only methods which permit to
 * display DEBUG trace.<br>
 * 
 * You can configure the path of your file that contains the the Log4J
 * properties by using the environment variable "log4j.configfile".<br>
 * To set it, please run jvm with option <i>-Dlog4j.configfile="YourPath"</i>.
 * If the path is false, then default properties will be set for log4j logger !
 * The default value is "./log4j.prop" <br>
 * If no file is found, a default Appender will be set (System.err stream).
 * 
 * By the environment property "log4j.debuglevel". This value will fix the level
 * of message displayed (-1 - no message, 0 - debug, 1 - info, 2 - warning, 3 -
 * error, 4 - fatal ). It means that only the messages with level > debug level
 * will be displayed. The default value is "0"
 * 
 * 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jan 9, 2007 8:58:49 AM
 */
public class DebugPrinter {

	/**
	 * Static value that code the DEBUG level of the logger
	 */
	public static final int DEBUG = 0;

	/**
	 * Static value that code the INFO level of the logger
	 */
	public static final int INFO = 1;

	/**
	 * Static value that code the WARN level of the logger
	 */
	public static final int WARN = 2;

	/**
	 * Static value that code the ERROR level of the logger
	 */
	public static final int ERROR = 3;

	/**
	 * Static value that code the FATAL level of the logger
	 */
	public static final int FATAL = 4;

	/** Log Service (Log4J) reference */
	static Logger logger = Logger.getLogger(DebugPrinter.class);

	/**
	 * Set this variable to change debug mode.<br>
	 * If <i>true</i> then all messages will be displayed, else none of them.
	 */
	private static boolean isDebugMode;

	/**
	 * Set the debug level (-1 - no message, 0 - debug, 1 - info, 2 - warning, 3 -
	 * error, 4 - fatal ) limit. It means that only the messages with level >
	 * debug level will be displayed
	 */
	private static int debugLevel = 0;

	/** Default value of log4j Property file */
	private static final String DEFAULT_LOG4J_PROPERTY_FILE = "."
			+ System.getProperty("file.separator") + "log4j.prop";

	/** Default value of debugLevel */
	private static final int DEFAULT_DEBUGLEVEL = 0;

	/** THe default Appender */
	private static Appender default_appender = new ConsoleAppender(
			new SimpleLayout(), ConsoleAppender.SYSTEM_ERR);

	static {

		/*
		 * Set Property of Log4J parser
		 */

		// load Log4J configuration file
		String propertyFile = System.getProperty("log4j.configfile");

		// if no value is set in environment use default value
		if (propertyFile == null)
			propertyFile = DEFAULT_LOG4J_PROPERTY_FILE;

		// load the propertyFile
		loadLog4JPropertyFile(propertyFile);

		/*
		 * Set Debug Level
		 */
		try {
			setDebugLevel(Integer.parseInt(System.getProperty("log4j.debuglevel")));
		} catch (NumberFormatException e) {
			setDebugLevel(DEFAULT_DEBUGLEVEL);
		}
	}

	/**
	 * This method diplay the given message, with prefix the caller name
	 * 
	 * @param message :
	 *            the message to display
	 * @param level :
	 *            the level of the message ( 0 - debug, 1 - info, 2 - warning, 3 -
	 *            error, 4 - fatal )
	 */
	public static void print(String message, int level) {

		if (isDebugMode) {
			String result = "\n";
			StackTraceElement[] stack = (new Throwable()).getStackTrace();

			try {
				result += stack[1].toString() + " >> " + message + "\n";

			} catch (IndexOutOfBoundsException r) {
				result += "Somewhere >> " + message + "\n";
			}
			logMessage(result, level);
		}

	}

	/**
	 * This method diplay all element of the list, with prefix the caller name
	 * 
	 * @param l
	 *            the list to display
	 * @param message :
	 *            the message to display
	 * @param level :
	 *            the level of the message ( 0 - debug, 1 - info, 2 - warning, 3 -
	 *            error, 4 - fatal )
	 */
	public static <K> void print(List<K> l, String message, int level) {
		if (isDebugMode) {
			String result = "\n";
			StackTraceElement[] stack = (new Throwable()).getStackTrace();
			try {
				result += stack[2].toString()
						+ " >>  "
						+ message
						+ "\n\t***********************************************\n";

			} catch (IndexOutOfBoundsException r) {
				result += "Somewhere >>  "
						+ message
						+ "\n\t***********************************************\n";
			} finally {
				if (l != null) {
					if (l.size() != 0) {
						Iterator<K> it = l.iterator();
						int i = 0;
						while (it.hasNext()) {
							K k = it.next();
							if (k != null) {
								result += "\t" + i++ + " : " + k.toString()
										+ "\n";
							} else {
								result += "\t" + i++ + " : null\n";
							}
						}
					} else {
						result += "\tempty\n";
					}
				} else {
					result += "\tnull\n";
				}

				result += "\t***********************************************\n";
			}
			logMessage(result, level);
		}
	}

	/**
	 * This method diplay all element of a map, with prefix the caller name
	 * 
	 * @param m
	 *            the map to display
	 * @param message
	 *            the message to display
	 * @param level
	 *            the level of the message ( 0 - debug, 1 - info, 2 - warning, 3 -
	 *            error, 4 - fatal )
	 */
	public static <T, K> void print(Map<T, K> m, String message, int level) {
		if (isDebugMode) {
			String result = "\n";
			StackTraceElement[] stack = (new Throwable()).getStackTrace();
			try {
				result += stack[2].toString()
						+ " >> "
						+ message
						+ "\n\t***********************************************\n";

			} catch (IndexOutOfBoundsException r) {
				result += "Somewhere >>  "
						+ message
						+ "\n\t***********************************************\n";
			} finally {
				if (m != null) {
					if (m.size() != 0) {
						List<T> l = new ArrayList<T>(m.keySet());
						Iterator<T> it = l.iterator();

						while (it.hasNext()) {
							T key = it.next();
							result += "\t" + key.toString() + " : "
									+ m.get(key).toString() + "\n";
						}
					} else {
						result += "\tempty\n";
					}
				} else {
					result += "\tnull\n";
				}
				result += "\t***********************************************\n";
			}
			logMessage(result, level);
		}
	}

	/**
	 * This method permit to display an exception that will be rethrown
	 * 
	 * @param stackTrace
	 *            the initial stack trace
	 * @param message
	 *            message to diplay
	 * @param level
	 *            the level of the message ( 0 - debug, 1 - info, 2 - warning, 3 -
	 *            error, 4 - fatal )
	 */
	public static void print(StackTraceElement[] stackTrace, String message,
			int level) {
		String result = "Exception = " + message + "\n";
		for (int i = 0; i < stackTrace.length; i++) {
			result += "\tat " + stackTrace[i] + "\n";
		}
		logMessage(result, level);
	}

	/**
	 * This method permit to display an exception that will be rethrown
	 * 
	 * @param e
	 *            the exception to display
	 * @param level
	 *            the level of the message ( 0 - debug, 1 - info, 2 - warning, 3 -
	 *            error, 4 - fatal )
	 */
	public static void print(Exception e, int level) {
		String result = e.getClass().getName() + " = "
				+ e.getLocalizedMessage() + "\n";
		for (int i = 0; i < e.getStackTrace().length; i++) {
			result += "\tat " + e.getStackTrace()[i] + "\n";
		}
		logMessage(result, level);
	}

	/**
	 * This method diplay all element of a map, with prefix the caller name
	 * 
	 * @param m
	 *            the map to display
	 * @param message
	 *            the message to display
	 * @param level
	 *            the level of the message ( 0 - debug, 1 - info, 2 - warning, 3 -
	 *            error, 4 - fatal )
	 */
	public static <T, K> void print(HashMap<T, K> m, String message, int level) {
		if (isDebugMode) {
			String result = "\n";
			StackTraceElement[] stack = (new Throwable()).getStackTrace();
			try {
				result += stack[2].toString()
						+ " >> "
						+ message
						+ "\n\t***********************************************\n";

			} catch (IndexOutOfBoundsException r) {
				result += "Somewhere >>  "
						+ message
						+ "\n\t***********************************************\n";
			} finally {
				if (m != null) {
					if (m.size() != 0) {
						List<T> l = new ArrayList<T>(m.keySet());
						Iterator<T> it = l.iterator();

						while (it.hasNext()) {
							T key = it.next();
							result += "\t" + key.toString() + " : "
									+ m.get(key).toString() + "\n";
						}
					} else {
						result += "\tempty\n";
					}
				} else {
					result += "\tnull\n";
				}
				result += "\t***********************************************\n";
			}
			logMessage(result, level);
		}

	}

	/**
	 * This message use the Log4J logger to display a message. This message
	 * 
	 * @param message
	 *            message to diplay
	 * @param level
	 *            the level of the message ( 0 - debug, 1 - info, 2 - warning, 3 -
	 *            error, 4 - fatal )
	 */
	private static void logMessage(String message, int level) {
		if (isDebugMode) {
			switch (level) {
			case 0:
				if (0 >= debugLevel) {
					logger.debug(message);
				}
				break;
			case 1:
				if (1 >= debugLevel) {
					logger.info(message);
				}
				break;
			case 2:
				if (2 >= debugLevel) {
					logger.warn(message);
				}
				break;
			case 3:
				if (3 >= debugLevel) {
					logger.error(message);
				}
				break;
			case 4:
				if (4 >= debugLevel) {
					logger.fatal(message);
				}
				break;
			default:
				break;
			}
		}

	}

	/**
	 * This method permit to modify the debugLevel
	 * 
	 * @param newLevel
	 *            the new value of debugLevel ( 0 - debug, 1 - info, 2 -
	 *            warning, 3 - error, 4 - fatal )
	 */
	public static void setDebugLevel(int newLevel) {
		// set the new debugLevel
		debugLevel = newLevel;

		// set isDebugMode depending debugLevel
		if (newLevel <= -1) {
			isDebugMode = false;
		} else {
			isDebugMode = true;

			// debugLevel is maximise by 4, so correct higher value.
			if (newLevel > 4) {
				debugLevel = 4;
			}
		}
	}

	/**
	 * get the current value of DebugLevel
	 * 
	 * @return the value of the debugLevel (between -1 and 4)
	 */
	public static int getDebugLevel() {
		return debugLevel;
	}

	/**
	 * Load the given log4J property File. This file could be a classical
	 * property file or an xml one. In the second case, the filename must finish
	 * by ".xml"
	 * 
	 * @param filePath
	 *            the path of the log4j Property file.
	 */
	public static void loadLog4JPropertyFile(String filePath) {

		System.err.println("Enter DebugPrinter::loadLog4JPropertyFile ---> filePath = "
				+ filePath);

		logger.removeAppender(default_appender);
		// logger.removeAllAppenders();
		if ((new File(filePath)).exists()) {
			// if (filePath.endsWith(".xml")) {
			// DOMConfigurator.configure(filePath);
			// } else {
			PropertyConfigurator.configure(filePath);
			// }
		} else {
			System.err.println("File " + filePath
					+ " not exists => Default config");
			// logger.addAppender(default_appender);
		}
	}

	/**
	 * @return the isDebugMode
	 */
	public static boolean isDebugMode() {
		return isDebugMode;
	}

	/**
	 * @param isDebugMode
	 *            the isDebugMode to set
	 */
	public static void setDebugMode(boolean isDebugMode) {
		DebugPrinter.isDebugMode = isDebugMode;
	}

}