/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.common;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * This class contains some tools to display and save some time measurement. <br>
 * 
 * Set System property "measurementtool.savetrace" to "true" to store all information abnd display its at the end of the execution. If This property is not set, the default value is false<br>
 * 
 * To set it, please run jvm with option <i>-Dmeasurementtool.savetrace="true"</i>. 
 * 
 * 
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 23, 2008 10:35:33 AM
 */
public class MeasurementTool {

	/**
	 * The name of the system property to check
	 */
	private static final String PROPERTYNAME = "measurementtool.savetrace";

	/**
	 * Class use to store the couple of information "message" - "time" 
	 */
	private class LogTimeElement {

		/**
		 * The message associated to the time measure
		 */
		private String message = null;

		/**
		 * The time measure
		 */
		private long time = 0;

		/**
		 * @param message
		 * @param time
		 */
		public LogTimeElement(String message, long time) {
			this.message = message;
			this.time = time;
		}

		/**
		 * @return the message
		 */
		public String getMessage() {
			return message;
		}

		/**
		 * @return the time
		 */
		public long getTime() {
			return time;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#hashCode()
		 */
		@Override
		public int hashCode() {
			final int PRIME = 31;
			int result = 1;
			result = PRIME * result
					+ ((message == null) ? 0 : message.hashCode());
			result = PRIME * result + (int) (time ^ (time >>> 32));
			return result;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#equals(java.lang.Object)
		 */
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			final LogTimeElement other = (LogTimeElement) obj;
			if (message == null) {
				if (other.message != null)
					return false;
			} else if (!message.equals(other.message))
				return false;
			if (time != other.time)
				return false;
			return true;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			String result = message;

			long second = time / 1000;
			long hour = second / 3600;
			long minuts = (second % 3600) / 60;
			long sec = (second % 3600) % 60;

			result += time + " ms (" + hour + "h " + minuts + "mn " + sec
					+ "s)";

			return result;
		}

	}

	/**
	 * The list of time measure already done
	 */
	private List<LogTimeElement> logs;

	/**
	 * The current instance of {@link MeasurementTool}
	 */
	private static MeasurementTool instance;

	/**
	 * indicate if the time mesure needs to be store
	 */
	private boolean logEnabled = false;

	/**
	 * Default constructor
	 */
	public MeasurementTool() {
		String propertyValue = System.getProperty(PROPERTYNAME);

		if (propertyValue != null) {
			try {
				this.logEnabled = Boolean.parseBoolean(propertyValue);

				if (this.logEnabled)
					this.logs = new ArrayList<LogTimeElement>();
			} catch (Exception e) {
				DebugPrinter.print(e, DebugPrinter.WARN);
			}

		}
	}

	/**
	 * Get the current instance of MeasurementTool
	 * @return the current instance
	 */
	private static MeasurementTool getInstance() {
		if (instance == null)
			instance = new MeasurementTool();

		return instance;
	}

	/**
	 * Add a TimeMeasureElement in the list of logs
	 * @param message the message to store
	 * @param time the time measure to log
	 */
	private void addTimeMeasureElement(String message, long time) {
		this.logs.add(new LogTimeElement(message, time));
	}

	/**
	 * @return the logEnabled
	 */
	public boolean isLogEnabled() {
		return logEnabled;
	}

	private final String filePath = "out_measurement.txt";

	/**
	 * Write the content of the list logs on the current logger 
	 *
	 */
	private void displayLogs() {
		File f = new File(filePath);
		try {

			if (logEnabled) {
				if (!f.exists())
					f.createNewFile();

				BufferedWriter writer = new BufferedWriter(new FileWriter(f, true));
				writer.write("-------------------------------------------------------------------");
				writer.newLine();
				for(LogTimeElement element : this.logs){
					writer.write(element.toString());
					writer.newLine();
				}
				writer.write("-------------------------------------------------------------------");
				writer.newLine();
				writer.flush();
				writer.close();

				DebugPrinter.print(this.logs, "Measurement logs",
						DebugPrinter.INFO);

			}

		} catch (IOException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
		}
	}

	@Override
	protected void finalize() throws Throwable {
		displayLogs();
		super.finalize();
	}

	/**
	 * Display the time
	 * 
	 * @param message
	 *            the message to display befor the time to display
	 * @param time
	 *            the time to display (in ms)
	 */
	public static void displayTime(String message, long time) {
		String result = message + " ";

		long second = time / 1000;
		long hour = second / 3600;
		long minuts = (second % 3600) / 60;
		long sec = (second % 3600) % 60;

		result += hour + "h " + minuts + "mn " + sec + "s" + "  (= " + time
				+ " ms)";

		DebugPrinter.print(result, DebugPrinter.INFO);

		MeasurementTool tool = MeasurementTool.getInstance();

		/*
		 * Log, if required
		 */
		if (tool.isLogEnabled()) {
			tool.addTimeMeasureElement(message, time);
		}
	}

	/**
	 * Write the content of the list logs on the current logger 
	 *
	 */
	public static void displayAllStoredMeasurement() {
		MeasurementTool tool = MeasurementTool.getInstance();
		if (tool.isLogEnabled()) {
			tool.displayLogs();
		}
	}
}
