/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.common;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.Map.Entry;

import fr.inria.icocoa.exception.ParsingException;

/**
 * This class contains many method that will be used to convert some data type
 * in an other type
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jan 29, 2007 3:23:44 PM
 */
public class Tools {

	/**
	 * The character used as file separator. This character depends of the hosting system.
	 */
	private static final String FILE_SEPARATOR = System.getProperty("file.separator");

	/**
	 * Read the contains of a file
	 * 
	 * @param filename
	 *            the path of the file we want to read
	 * @return the content of the readed file if no problem occurs, <i>null</i>
	 *         otherwise.
	 */
	public static String readFromFile(String filename) {
		String content = "";
		FileInputStream f = null;
		try {
			f = new FileInputStream(filename);

			int len = f.available();

			for (int i = 1; i <= len; i++) {
				content = content + (char) f.read();
			}
		} catch (FileNotFoundException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
			return null;
		} catch (IOException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
			return null;
		} finally {
			try {
				f.close();
			} catch (IOException e) {
				DebugPrinter.print(e.getStackTrace(), e.getMessage(),
					DebugPrinter.WARN);
			}
		}

		return content;
	}

	/**
	 * Read the content of a File and return it
	 * @param url the url of the file to read
	 * @return the content of the given file
	 * @throws IOException this exception is thrown if an error occurs during the reading process
	 */
	public static String readFromUrl(URL url) throws IOException {
		BufferedReader in = new BufferedReader(new InputStreamReader(
				url.openStream()));

		String line;
		StringBuilder result = new StringBuilder();
		while ((line = in.readLine()) != null) {
			result.append(line);
			result.append('\n');
		}

		in.close();

		return result.toString();
	}

	/**
	 * Create a temp file with the specified content
	 * 
	 * @param content
	 *            the content to put in the temp file.
	 * @return the URI of the generated file
	 * @throws IOException
	 *             thrown in case of problem with IO
	 * @throws ParsingException :
	 *             thrown in case of problem when trying to obtain a valid URI
	 */
	public static String createTempOwlFile(String content) throws IOException,
			ParsingException {
		UUID owl_uuid = UUID.randomUUID();
		File f = File.createTempFile(owl_uuid.toString(), null);
		f.deleteOnExit();
		BufferedWriter out = new BufferedWriter(new FileWriter(f));
		out.write(content);
		out.flush();
		out.close();

		/*
		 * DebugPrinter.print("Service description write int temp file " +
		 * f.getAbsolutePath(), DebugPrinter.INFO);
		 */

		return Tools.getURIStringFromFilePath(f.getAbsolutePath());
	}

	/* ************************************* */
	/* * Method to calculate URI from Path * */
	/* ************************************* */

	/**
	 * This method take a Path and transform it into an URI, then check it to
	 * valide it if the OS is Windows
	 * 
	 * @param filePath
	 *            the path of the file we want to obtain the URI
	 * @return a String coding the URI.
	 * @throws URISyntaxException
	 *             if the resulting URI is not correct
	 */
	public static String getURIStringFromFilePath(String filePath) {
		return getURIFromFilePath(filePath).toString();
	}

	/**
	 * This method take a Path and transform it into an URI
	 * 
	 * @param filePath
	 *            the path of the file we want to obtain the URI
	 * @return the URI of the File, null if an error occurs
	 */
	public static URI getURIFromFilePath(String filePath) {
		try {
			// we use different way to produce an URI, depends if we are under
			// linux
			// or windows
			if (Tools.FILE_SEPARATOR.equals("\\")) {
				return new URI("file:/"
						+ filePath.replace('\\', '/').replace(" ", "%20"));
			} else {
				return new URI("file", filePath, null);
			}
		} catch (URISyntaxException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
			return null;
		}
	}

	/**
	 * This method convert an uri to an URL
	 * 
	 * @param uri
	 *            the uri we want to convert
	 * @return the corresponding URL, null if an error occurs
	 */
	public static URL getURLfromURI(String uri) {
		try {
			// we use different way to produce an URI, depends if we are under
			// linux or windows
			if (Tools.FILE_SEPARATOR.equals("\\")) {
				return new URL(uri.replace('\\', '/').replace(" ", "%20"));
			} else {
				return new URL(uri);
			}
		} catch (MalformedURLException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
			return null;
		}
	}

	/**
	 * THis method convert an uri to an URL
	 * 
	 * @param uri :
	 *            the uri we want to convert
	 * @return the corresponding URL, "null" if an error occurs
	 */
	public static String getURLStringFromURI(String uri) {
		URL url = getURLfromURI(uri);
		if (url != null) {
			return url.toString();
		} else {
			return "null";
		}
	}

	/**
	 * Split the given string from char '#' and return the second part of the
	 * split
	 * 
	 * @param s
	 *            the string to split
	 * @return the second part of the split string, s if s does not contain '#'
	 */
	public static String getSecondPart(String s) {
		try {
			return s.split("#")[1];
		} catch (IndexOutOfBoundsException e) {
			// DebugPrinter.print(e.getStackTrace(), e.getMessage() + "(" + s +
			// " unsplitable)", 2);
			return s;
		}
	}

	/**
	 * Split the given string from char '#' and return the first part of the
	 * split
	 * 
	 * @param s
	 *            the string to split
	 * @return the second part of the splited string, s if s does not contain
	 *         '#'
	 */
	public static String getFirstPart(String s) {
		try {
			return s.split("#")[0];
		} catch (IndexOutOfBoundsException e) {
			// DebugPrinter.print(e.getStackTrace(), e.getMessage() + "(" + s +
			// " unsplitable)", 2);
			return s;
		}
	}

	/**
	 * This method test if the two string are equals even if one of the
	 * parameter is null
	 * 
	 * @param s1
	 *            the first string to compare
	 * @param s2
	 *            the second string to compare
	 * @return true if the two string are equals
	 */
	public static boolean compareString(String s1, String s2) {
		if (s1 == null || s2 == null) {
			return s1 == s2;
		} else {
			return s1.equals(s2);
		}
	}

	/**
	 * This method to compare two lists of object whatever the order.<br>
	 * <b>Warning</b>: T and K need to be exactly the same (not subclasses)
	 * 
	 * @param <T>
	 *            the type of object contained in list1
	 * @param <K>
	 *            the type of object contained in list2
	 * @param list1
	 *            the first list to compare
	 * @param list2
	 *            the second list to compare
	 * @return <i>true</i>, if they are the same list, <i>false</i> otherwise
	 */
	@SuppressWarnings("unchecked")
	public static <T, K> boolean compareList(List<T> list1, List<K> list2) {
		/* if both list are null or empty, we suppose that they are equivalent */
		if (list1 == null) {
			return list2 == null;
		}
		if (list2 == null) {
			return list1 == null;
		}
		if (list1.isEmpty()) {
			return list2.isEmpty();
		}
		if (list2.isEmpty()) {
			return list1.isEmpty();
		}

		Object o1 = list1.get(0);
		Object o2 = list2.get(0);
		Class c1 = o1.getClass();
		Class c2 = o2.getClass();

		// test the type of the contained elements
		if (c1.equals(c2)) {

			// test on the number of element
			if (list1.size() == list2.size()) {
				List<Object> master = new LinkedList<Object>(list1);
				List<Object> secondary = new LinkedList<Object>(list2);

				// save current size of master
				int master_size_init = master.size();

				// Master now only contains records that exists in both lists.
				master.retainAll(secondary);

				// if master contains as many element as before, the two list
				// are the same
				return master.size() == master_size_init;
			} else {
				return false;
			}

		} else {
			return false;
		}

	}

	/**
	 * This method test the egality two list of object whatever the order.<br>
	 * <b>Warnings</b>:
	 * <ul>
	 * <li>(T and L) and (K and J) need to be exactly the same (not subclasses)</li>
	 * <li>This method considere that if the list of keys and the list of
	 * objects of the two maps are equals then the two maps are equivalent. It
	 * means that if the same keys are associated with the same ojects, the two
	 * maps will be consider as equivalent <b>whatever the associations</b>
	 * </li>
	 * </ul>
	 * 
	 * @param <T>
	 *            the type of key object contained in map1
	 * @param <K>
	 *            the type of object contained in map1
	 * @param <L>
	 *            the type of key object contained in map2
	 * @param <J>
	 *            the type of object contained in map2
	 * @param map1
	 *            the first map to compare
	 * @param map2
	 *            the second map to compare
	 * @return <i>true</i>, if they are equivalent map, <i>false</i> otherwise
	 */
	@SuppressWarnings("unchecked")
	public static <T, K, L, J> boolean compareMap(Map<T, K> map1, Map<L, J> map2) {
		/* if both list are null or empty, we suppose that they are equivalent */
		if (map1 == null) {
			return map2 == null;
		}
		if (map2 == null) {
			return map1 == null;
		}
		if (map1.isEmpty()) {
			return map2.isEmpty();
		}
		if (map2.isEmpty()) {
			return map1.isEmpty();
		}

		// get first entry of maps
		Entry<T, K> e1 = map1.entrySet().iterator().next();
		Entry<L, J> e2 = map2.entrySet().iterator().next();

		// get class of each object
		Class c_e1_k = e1.getKey().getClass();
		Class c_e1_o = e1.getValue().getClass();
		Class c_e2_k = e2.getKey().getClass();
		Class c_e2_o = e2.getValue().getClass();

		// compare class of object
		if (c_e1_k.equals(c_e2_k) && c_e1_o.equals(c_e2_o)) {
			// compare list of keys and list of objects
			return Tools.compareList(new LinkedList<T>(map1.keySet()),
				new LinkedList<L>(map2.keySet()))
					&& Tools.compareList(new LinkedList<K>(map1.values()),
						new LinkedList<J>(map2.values()));
		} else {
			return false;
		}
	}

	/**
	 * transform all character into is correspondant URL code (%Hexa)
	 * 
	 * @param string
	 *            the string to transform
	 * @return a valid string if no error occurs, the initial string otherwise
	 */
	public static String transformIntoEncodedURLString(String string) {
		try {
			return URLEncoder.encode(string, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
			return string;
		}
	}

	/**
	 * Check if the given string seams to be an URI, or if it is more a filepath
	 * 
	 * @param string
	 *            the string to check
	 * @return <i>true</i> if the string looks like an URI, <i>false</i> if it
	 *         looks like a filepath
	 */
	public static boolean seamsToBeAnURI(String string) {
		File f = new File(string);
		return !f.exists();
	}
	
	

}
