/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.Map.Entry;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.CreationOfRecursionException;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.automaton.Automaton;
import fr.inria.icocoa.servicemodel.automaton.ServiceModel2Automata;
import fr.inria.icocoa.servicemodel.flow.CapabilityFlow;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;
import fr.inria.icocoa.servicemodel.semantictype.SemanticTypeFactory;

/**
 * This class implements the main part of the model "Capability-centered". This Capability represent a functionnality of a Service. This capability can be Atomic or Composite (has a complex worklow that refer other Capabilities).<br> 
 * Each Capability can have some inputs, outputs and even some properties.<br><br>
 * 
 * The Capability is linked to data concerbing the real Services thanks GroundingProperty object.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class Capability implements Serializable, Transferable<Capability> {

	/**
	 * Variable to initialise field to a non-null value
	 */
	public static final String UNKNOWN = "!!!Unknow!!!";

	/**
	 * SerialVersion of the class
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The static value to set the type of the Capability to Subcapability
	 */
	public static final int SUBCAPABILITY = 0;

	/**
	 * The static value to set the type of the Capability to Provided
	 */
	public static final int PROVIDED = 1;

	/**
	 * The static value to set the type of the Capability to Required
	 */
	public static final int REQUIRED = 2;

	/**
	 * The type of the Capability : 0 - SubCapability, 1 - Provided, 2 -
	 * required
	 */
	private int typeOfCapability = 0;

	/**
	 * The semantic type of the Capability
	 */
	private SemanticType semanticType = null;

	/**
	 * the local typeName of the Capability
	 */
	private String localName = UNKNOWN;

	/**
	 * The Map of inputs (InputName -> InputObject)
	 */
	private Map<String, Input> inputs;

	/**
	 * The Map of outputs (OutputName -> OutputObject)
	 */
	private Map<String, Output> outputs;

	/**
	 * The Map of properties (PropertyName -> PropertyObject)
	 */
	private Map<String, Property> properties; // properties are non-functional

	// properties (Category, QoS, Context)

	/**
	 * The ControlConstruct object, in case of the Capability contains
	 * subcapabilities
	 */
	private ControlConstruct CC = null;

	/**
	 * The list of subcapabilities, in case of the Capability does not
	 * correspond to an Atomic process
	 */
	private List<Capability> subCapabilities;

	/**
	 * The service in which the capability is contained
	 */
	private Service service = null;

	/**
	 * The Grounding Property associated with the Capability
	 */
	private GroundingProperty grounding = null;

	/**
	 * The typeName typeName of the associated conversation
	 */
	private String conversationURI = UNKNOWN;

	/**
	 * The URI of the capability
	 */
	private String capabilityURI = UNKNOWN;

	/**
	 * Indicate if the capability as an atomic process or not
	 */
	private boolean atomic = false;

	/**
	 * The number of copy of this Capability
	 */
	private int nbrCopies = -1; // from anupam model

	/**
	 * The Automaton corresponding to the Capability
	 */
	private Automaton automaton = null;

	/**
	 * The list of input data binding corresponding to the Capability
	 */
	private List<ParameterBinding> inputsBinding = null;

	/**
	 * The list of output data binding corresponding to the Capability
	 */
	private List<ParameterBinding> outputsBinding = null;

	/**
	 * The Capability Flow corresponding to the capability (InvokeCapabilityFlow
	 * object if Atomic, CompositeCapabilityFlow if not Atomic)
	 */
	private CapabilityFlow flow = null;

	/* **************** */
	/* * Constructors * */
	/* **************** */

	/**
	 * Default constructor of the Class.
	 */
	public Capability() {
		this(Capability.UNKNOWN + "#" + Capability.UNKNOWN, null,
				Capability.UNKNOWN, Capability.SUBCAPABILITY);
	}

	/**
	 * Constructor of the Class.
	 * 
	 * @param rdfTypeURI
	 *            the rdf type of the Capability (URI)
	 */
	public Capability(String rdfTypeURI) {
		this(rdfTypeURI, null, Capability.UNKNOWN, Capability.SUBCAPABILITY);
	}

	/**
	 * Constructor of the Class.
	 * 
	 * @param rdfTypeURI
	 *            the rdf type of the Capability (URI)
	 * @param localName
	 *            the local typeName of the Capability
	 */
	public Capability(String rdfTypeURI, String localName) {
		this(rdfTypeURI, null, localName, Capability.SUBCAPABILITY);

	}

	/**
	 * Constructor of the Class.
	 * 
	 * @param rdfTypeURI
	 *            the rdf type of the Capability (URI)
	 * @param currentService
	 *            the typeName of the associated service
	 * @param localName
	 *            the local typeName of the Capability
	 * 
	 */
	public Capability(String rdfTypeURI, Service currentService,
			String localName) {
		this(rdfTypeURI, currentService, localName, Capability.SUBCAPABILITY);
	}

	/**
	 * Constructor of the Class.
	 * 
	 * @param rdfTypeURI
	 *            the type of the Capability (URI)
	 * @param currentService
	 *            the typeName of the associated service
	 * @param localName
	 *            the local typeName of the Capability
	 * @param type
	 *            the type of Capability
	 */
	public Capability(String rdfTypeURI, Service currentService,
			String localName, int type) {
		this.service = currentService;
		this.semanticType = SemanticTypeFactory.getInstance().getObject(Tools.getSecondPart(rdfTypeURI),Tools.getFirstPart(rdfTypeURI));
		this.localName = localName;
		this.inputs = new HashMap<String, Input>();
		this.outputs = new HashMap<String, Output>();
		this.properties = new HashMap<String, Property>();
		this.subCapabilities = new ArrayList<Capability>();
		this.typeOfCapability = type;
		this.outputsBinding = new ArrayList<ParameterBinding>();
		this.inputsBinding = new ArrayList<ParameterBinding>();
		this.flow = null;
	}

	/* ********************** */
	/* * Add/Remove methods * */
	/* ********************** */

	/**
	 * This method add a new input in the Input list
	 * 
	 * @param input
	 *            the input object to add in the list of inputs
	 */
	public void addInput(Input input) {
		this.inputs.put(input.getLocalName(), input);
	}

	/**
	 * This method add a new output in the Output list
	 * 
	 * @param output
	 *            the input object to add in the list of outputs
	 */
	public void addOutput(Output output) {
		this.outputs.put(output.getLocalName(), output);
	}

	/**
	 * This method add a new property in the Property list
	 * 
	 * @param property
	 *            the input object to add in the list of properties
	 */
	public void addProperty(Property property) {
		this.properties.put(property.getName(), property);
	}

	/**
	 * This method add a new capability in the subcapabilities list, and throw
	 * an exception if something occurs. <br>
	 * <b>Warning</b>: SDSCAE don't allow a Capability to reference itself as a
	 * subcapabily
	 * 
	 * @param capability
	 *            the capability object to add in the list of subcapabilities
	 * @throws CreationOfRecursionException
	 *             this Exception is thrown if your try to create an non
	 *             acceptable level of recursivity (ie in the case of you try to
	 *             add the current object as a subcapability of this current
	 *             object).
	 */
	public void addSubCapability_E(Capability capability)
			throws CreationOfRecursionException {
		if (capability.equalsWithoutSubCapabilities(this)) {
			throw new CreationOfRecursionException();
		}
		this.subCapabilities.add(capability);
	}

	/**
	 * This method add a new capability in the subcapabilities list. <br>
	 * <b>Warning</b>: SDSCAE don't allow a Capability to reference itself as a
	 * subcapabily
	 * 
	 * @param capability
	 *            the capability object to add in the list of subcapabilities
	 * @return <i>true</i> if no problem occurs, <i>false</i> otherwise.
	 */
	public boolean addSubCapability(Capability capability) {
		try {
			this.addSubCapability_E(capability);
			return true;
		} catch (CreationOfRecursionException e) {
			return false;
		}
	}

	/**
	 * This method change the field "provided" of inputs and outputs
	 * 
	 * @param newValueOfProvided
	 *            the new value for the "provided" field
	 */
	public void setIOAsProvided(boolean newValueOfProvided) {
		// change value for inputs
		for (Iterator<Input> it = this.getInputs().values().iterator(); it.hasNext();) {
			it.next().setProvided(newValueOfProvided);
		}

		// change value for outputs
		for (Iterator<Output> it = this.getOutputs().values().iterator(); it.hasNext();) {
			it.next().setProvided(newValueOfProvided);
		}
	}

	/* ********************** */
	/* * Overwrited methods * */
	/* ********************** */

	@Override
	public String toString() {
		return this.toString("");
	}

	/**
	 * This method generate a String to represent a Capability object
	 * 
	 * @param prefix
	 *            the prefix add to each new line
	 * @return the generate String
	 */
	public String toString(String prefix) {
		// String result = "Capability \n" + prefix + "\t- typeName(type)=" +
		// this.name;

		String result = "Capability \n"
				+ prefix
				+ "\t- type(type)="
				+ this.getSemanticType()
				+ ",\n"
				+ prefix
				+ "\t- localName="
				+ getLocalName()
				+ ",\n"
				+ prefix
				+ "\t- URI="
				+ getCapabilityURI()
				+ ",\n"
				+ prefix
				+ "\t- isAtomic="
				+ isAtomic()
				+ ",\n"
				+ prefix
				+ "\t- conversationURI="
				+ getConversationURI()
				+ ",\n"
				+ prefix
				+ "\t- Service="
				+ (this.getService() == null ? "null"
						: this.getService().getLocalName()) + ",\n" + prefix
				+ "\t- inputs={";
		// display inputs
		for (Iterator i = this.getInputs().values().iterator(); i.hasNext();) {
			result += i.next().toString();
			if (i.hasNext()) {
				result += ',';
			}
		}
		result += "},\n" + prefix + "\t- outputs={";
		// display outputs
		for (Iterator i = this.getOutputs().values().iterator(); i.hasNext();) {
			result += i.next().toString();
			if (i.hasNext()) {
				result += ',';
			}
		}
		result += "},\n" + prefix + "\t- inputs binding={";
		// display InputsBindings
		for (Iterator i = this.getInputsBinding().iterator(); i.hasNext();) {
			result += "\n" + prefix + "\t- " + i.next().toString();
			if (i.hasNext()) {
				result += ',';
			}
		}
		result += "},\n" + prefix + "\t- outputs binding={";
		// display OutputsBindings
		for (Iterator i = this.getOutputsBinding().iterator(); i.hasNext();) {
			result += "\n" + prefix + "\t- " + i.next().toString();
			if (i.hasNext()) {
				result += ',';
			}
		}
		result += "},\n" + prefix + "\t- ControlContruct=" + getCC() + ",\n"
				+ prefix + "\t- capabilities";
		// diplay Subcapabilities
		if (this.getSubCapabilities().size() == 0) {
			result += "\n\t\t" + prefix + "No subcapabilities !";
		} else {
			for (int i = 0; i < this.getSubCapabilities().size(); i++) {
				result += "\n\t\t"
						+ prefix
						+ (this.getSubCapabilities().get(i)).toString(prefix
								+ "\t");
				if (i != this.getSubCapabilities().size() - 1) {
					result += ',';
				}
			}
		}

		// display Flow
		if (this.getFlow() != null) {
			result += "\n\t" + prefix + "- Flow:\n"
					+ this.getFlow().toString(prefix + "\t\t");
		}

		// display Grounding
		if (this.getGrounding() != null) {
			result += this.getGrounding().printGrounding(prefix);
		}
		return result;

	}

	@Override
	public boolean equals(Object _capa) {
		if (_capa instanceof Capability) {
			Capability capa = (Capability) _capa;
			// same general information
			return this.equalsWithoutSubCapabilities(_capa)
					&& Tools.compareList(this.getSubCapabilities(),
						capa.getSubCapabilities());

		} else {
			return false;
		}
	}

	/**
	 * This method test if the two Capability are the same, without concidering
	 * the SubCapabilities , neither the type of Capability
	 * 
	 * @param _capa
	 *            the capability to compare the current object
	 * @return <i>true</i> if they are equivalent, <i>false</i> otherwise
	 */
	public boolean equalsWithoutSubCapabilities(Object _capa) {
		if (_capa instanceof Capability) {
			Capability capa = (Capability) _capa;
			// same general information
			return Tools.compareString(this.getTypeName(), capa.getTypeName())
					&& Tools.compareString(this.getLocalName(),
						capa.getLocalName())
					&& Tools.compareString(this.getCapabilityURI(),
						capa.getCapabilityURI())
					&& Tools.compareString(this.getConversationURI(),
						capa.getConversationURI())
					&& Tools.compareString(this.getOntology(),
						capa.getOntology())
					&& (this.getCC() == null ? capa.getCC() == null
							: this.getCC().equals(capa.getCC()))
					&& this.isAtomic() == capa.isAtomic()
					// && this.getTypeOfCapability() ==
					// capa.getTypeOfCapability()
					// && (this.getService() == null ? capa.getService() == null
					// : this.getService()
					// .equalsWithoutCapabilities(capa.getService()))
					&& (this.getGrounding() == null ? capa.getGrounding() == null
							: this.getGrounding().equals(capa.getGrounding()))

					&& Tools.compareMap(this.getInputs(), capa.getInputs())
					&& Tools.compareMap(this.getOutputs(), capa.getOutputs())
					&& Tools.compareList(this.getInputsBinding(),
						capa.getInputsBinding())
					&& Tools.compareList(this.getOutputsBinding(),
						capa.getOutputsBinding());
		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		long hash = 7;
		hash = 31
				* hash
				+ (null == this.getTypeName() ? 0
						: this.getTypeName().hashCode());
		hash = 31
				* hash
				+ (null == this.getCapabilityURI() ? 0
						: this.getCapabilityURI().hashCode());
		hash = 31
				* hash
				+ (null == this.getConversationURI() ? 0
						: this.getConversationURI().hashCode());
		hash = 31
				* hash
				+ (null == this.getLocalName() ? 0
						: this.getLocalName().hashCode());
		hash = 31
				* hash
				+ (null == this.getOntology() ? 0
						: this.getOntology().hashCode());
		hash = 31 * hash + (null == this.getCC() ? 0 : this.getCC().hashCode());
		hash = 31 * hash + (this.isAtomic() ? 0 : 1);
		hash = 31 * hash + this.getTypeOfCapability();
		/*
		 * hash = 31 hash + (null == this.getService() ? 0 :
		 * this.getService().hashCode());
		 */
		hash = 31
				* hash
				+ (null == this.getGrounding() ? 0
						: this.getGrounding().hashCode());
		hash = 31 * hash
				+ (null == this.getInputs() ? 0 : this.getInputs().hashCode());
		hash = 31
				* hash
				+ (null == this.getOutputs() ? 0 : this.getOutputs().hashCode());
		hash = 31
				* hash
				+ (null == this.getSubCapabilities() ? 0
						: this.getSubCapabilities().hashCode());
		hash = 31
				* hash
				+ (null == this.getSubCapabilities() ? 0
						: this.getSubCapabilities().hashCode());
		hash = 31
				* hash
				+ (null == this.getInputsBinding() ? 0
						: this.getInputsBinding().hashCode());
		hash = 31
				* hash
				+ (null == this.getOutputsBinding() ? 0
						: this.getOutputsBinding().hashCode());

		return (int) (hash % Integer.MAX_VALUE);
	}

	/* ********************* */
	/* * Getter and Setter * */
	/* ********************* */

	/**
	 * Access one of the subcapability thanks to its typeName
	 * 
	 * @param subCapabilityTypeName
	 *            the typeName of the required capability
	 * @return <b>the first</b> subcapability which typeName correspond to the
	 *         given parameter, <b>null</b> if no subCapability has this
	 *         typeName.
	 */
	public Capability getSubCapabilityFromTypeName(String subCapabilityTypeName) {
		if (!this.getSubCapabilities().isEmpty()) {
			for (Iterator subCapaIt = this.getSubCapabilities().iterator(); subCapaIt.hasNext();) {
				Capability currentSubCapa = (Capability) subCapaIt.next();
				if (currentSubCapa.getTypeName().equals(subCapabilityTypeName)) {
					return currentSubCapa;
				}
			}
		}
		return null;
	}

	/**
	 * Access one of the subcapability thanks to its localName
	 * 
	 * @param subCapabilityLocalName
	 *            the typeName of the required capability
	 * @return <b>the first</b> subcapability which localName correspond to the
	 *         given parameter, <b>null</b> if no subCapability has this
	 *         localName.
	 */
	public Capability getSubCapabilityFromLocalName(
			String subCapabilityLocalName) {
		if (!this.getSubCapabilities().isEmpty()) {
			for (Iterator subCapaIt = this.getSubCapabilities().iterator(); subCapaIt.hasNext();) {
				Capability currentSubCapa = (Capability) subCapaIt.next();
				if (currentSubCapa.getLocalName().equals(subCapabilityLocalName)) {
					return currentSubCapa;
				}
			}
		}
		return null;
	}

	/**
	 * @param typeName
	 *            the typeName to set
	 */
	/*public void setTypeName(String typeName) {
		this.semanticType.setTypeName(typeName);
	}*/

	/**
	 * @return the typeName
	 */
	public String getTypeName() {
		return this.semanticType.getTypeName();
	}

	/**
	 * @param localName
	 *            the localName to set
	 */
	public void setLocalName(String localName) {
		this.localName = localName;
	}

	/**
	 * @return the localName
	 */
	public String getLocalName() {
		return localName;
	}

	/**
	 * @return the ontology
	 */
	public String getOntology() {
		return this.semanticType.getOntology();
	}	

	/**
	 * @param inputs
	 *            the inputs to set
	 */
	public void setInputs(Map<String, Input> inputs) {
		this.inputs = inputs;
	}

	/**
	 * @return the inputs
	 */
	public Map<String, Input> getInputs() {
		return inputs;
	}

	/**
	 * @param outputs
	 *            the outputs to set
	 */
	public void setOutputs(Map<String, Output> outputs) {
		this.outputs = outputs;
	}

	/**
	 * @return the outputs
	 */
	public Map<String, Output> getOutputs() {
		return outputs;
	}

	/**
	 * @param properties
	 *            the properties to set
	 */
	public void setProperties(Map<String, Property> properties) {
		this.properties = properties;
	}

	/**
	 * @return the properties
	 */
	public Map<String, Property> getProperties() {
		return properties;
	}

	/**
	 * @param cC
	 *            the cC to set
	 */
	public void setCC(ControlConstruct cC) {
		CC = cC;
	}

	/**
	 * @return the cC
	 */
	public ControlConstruct getCC() {
		return CC;
	}

	/**
	 * @param subCapabilities
	 *            the subCapabilities to set
	 */
	public void setSubCapabilities(List<Capability> subCapabilities) {
		this.subCapabilities = subCapabilities;
	}

	/**
	 * @return the subCapabilities
	 */
	public List<Capability> getSubCapabilities() {
		return subCapabilities;
	}

	/**
	 * @param service
	 *            the service to set
	 */
	public void setService(Service service) {
		this.service = service;
	}

	/**
	 * @return the service
	 */
	public Service getService() {
		return service;
	}

	/**
	 * @return the semanticType
	 */
	public SemanticType getSemanticType() {
		return semanticType;
	}

	/**
	 * @param semanticType the semanticType to set
	 */
	public void setSemanticType(SemanticType semanticType) {
		this.semanticType = semanticType;
	}

	/**
	 * @return the grounding
	 */
	public GroundingProperty getGrounding() {
		return grounding;
	}

	/**
	 * @param grounding
	 *            the grounding to set
	 */
	public void setGrounding(GroundingProperty grounding) {
		this.grounding = grounding;
	}

	/**
	 * @return the conversationURI
	 */
	public String getConversationURI() {
		return conversationURI;
	}

	/**
	 * @param conversationURI
	 *            the conversationURI to set
	 */
	public void setConversationURI(String conversationURI) {
		this.conversationURI = conversationURI;
	}

	/**
	 * @return the capabilityURI
	 */
	public String getCapabilityURI() {
		return capabilityURI;
	}

	/**
	 * @param capabilityURI
	 *            the capabilityURI to set
	 */
	public void setCapabilityURI(String capabilityURI) {
		this.capabilityURI = capabilityURI;
	}

	public boolean isAtomic() {
		return this.atomic;
	}

	/**
	 * @param atomic
	 *            the atomic to set
	 */
	public void setAtomic(boolean atomic) {
		this.atomic = atomic;
	}

	/**
	 * @return the typeOfCapability
	 */
	public int getTypeOfCapability() {
		return typeOfCapability;
	}

	/**
	 * @param typeOfCapability
	 *            the typeOfCapability to set
	 */
	public void setTypeOfCapability(int typeOfCapability) {
		this.typeOfCapability = typeOfCapability;
	}

	/**
	 * 
	 * @return the type of the IO
	 */
	public String getType() {
		return this.getOntology() + "#" + this.getTypeName();
	}

	/**
	 * Set a new rdtType value
	 * 
	 * @param newType :
	 *            the new type to set
	 */
	public void setType(String newType) {
		this.getSemanticType().setType(newType);
	}

	/**
	 * @return the inputsBinding
	 */
	public List<ParameterBinding> getInputsBinding() {
		return inputsBinding;
	}

	/**
	 * @param inputsBinding
	 *            the inputsBinding to set
	 */
	public void setInputsBinding(List<ParameterBinding> inputsBinding) {
		this.inputsBinding = inputsBinding;
	}

	/**
	 * @return the outputsBinding
	 */
	public List<ParameterBinding> getOutputsBinding() {
		return outputsBinding;
	}

	/**
	 * @param outputsBinding
	 *            the outputsBinding to set
	 */
	public void setOutputsBinding(List<ParameterBinding> outputsBinding) {
		this.outputsBinding = outputsBinding;
	}

	/**
	 * @return the automaton
	 */
	public Automaton getAutomaton() {
		if (this.automaton == null)
			this.automaton = new ServiceModel2Automata().generate_One_Level_Automaton(this);

		return automaton;
	}

	/**
	 * @param automaton
	 *            the automaton to set
	 */
	public void setAutomaton(Automaton automaton) {
		this.automaton = automaton;
	}

	/**
	 * @return the flow
	 */
	public CapabilityFlow getFlow() {
		return flow;
	}

	/**
	 * @param flow
	 *            the flow to set
	 */
	public void setFlow(CapabilityFlow flow) {
		this.flow = flow;
	}

	/**
	 * @return the nbrCopies
	 */
	public int getNbrCopies() {
		return nbrCopies;
	}

	/**
	 * @param nbrCopies
	 *            the nbrCopies to set
	 */
	public void setNbrCopies(int nbrCopies) {
		this.nbrCopies = nbrCopies;
	}

	/* ******************** */
	/* * Matching methods * */
	/* ******************** */

	/**
	 * Go through the list of inputs and outputs to find the given parameter
	 * 
	 * @param io
	 *            the IO to find
	 * @return true if found in a list, false otherwise
	 */
	public boolean isUsing(IO io) {
		if (io instanceof Input) {
			// go through the map of inputs
			for (Iterator<Input> it = this.getInputs().values().iterator(); it.hasNext();) {
				if (it.next().equals(io)) {
					return true;
				}
			}
		} else if (io instanceof Output) {
			// go through the map of outputs
			for (Iterator<Output> it = this.getOutputs().values().iterator(); it.hasNext();) {
				if (it.next().equals(io)) {
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Search the first atomic capability that use the given IO. The search is
	 * recursive.
	 * 
	 * @param io
	 *            the IO we search
	 * @return null if no Capability use this IO (if it's a composite process)
	 *         or if this Capability don't use this IO (if it's an atomic), the
	 *         first Capability in the process using it otherwise
	 */
	public Capability getFirstCapaUsingIO(IO io) {
		if (this.isAtomic()) {
			if (this.isUsing(io)) {
				return this;
			} else {
				return null;
			}
		} else {
			Capability result = null;
			for (Iterator<Capability> it = this.getSubCapabilities().iterator(); it.hasNext()
					&& result == null;) {
				result = it.next().getFirstCapaUsingIO(io);
			}
			return result;
		}
	}

	/**
	 * Search the first atomic sub-capability that use the given IO. The search
	 * is recursive.
	 * 
	 * @param io
	 *            the IO we search
	 * @return null if no Subcapability use this IO (if it's a composite
	 *         process) or if this Capability don't use this IO (if it's an
	 *         atomic), the first IO with such type in the process using it
	 *         otherwise
	 */
	public Input getFirstAtomicSubCapaUsingSuchIO(Input io) {
		if (this.isAtomic()) {
			for (Entry<String, Input> inputEntry : this.getInputs().entrySet()) {
				if (inputEntry.getValue().getType().equals(io.getType()))
					return inputEntry.getValue();
			}
			return null;
		} else {
			Input result = null;
			for (Iterator<Capability> it = this.getSubCapabilities().iterator(); it.hasNext()
					&& result == null;) {
				result = it.next().getFirstAtomicSubCapaUsingSuchIO(io);
			}
			return result;
		}
	}

	/**
	 * Search the last atomic sub-capability that use the given IO.
	 * 
	 * @param io
	 *            the IO we search
	 * @return null if no Subcapability use this IO (if it's a composite
	 *         process) or if this Capability don't use this IO (if it's an
	 *         atomic), the last IO with such type in the process using it
	 *         otherwise
	 */
	public Output getLastAtomicSubCapaUsingSuchIO(Output io) {

		// DebugPrinter.print("this = "+this,DebugPrinter.INFO);

		Output lastUsed = null;

		// creation and initialisation of the visit queue
		Queue<Capability> toVisit = new LinkedList<Capability>();
		toVisit.add(this);

		// visit each capability
		while (toVisit.size() > 0) {
			// DebugPrinter.print((List<Capability>)toVisit,"toVisit",DebugPrinter.INFO);

			Capability current = toVisit.poll();
			// DebugPrinter.print(current.getLocalName()+" is
			// "+((current.isAtomic())?"atomic":"composite"),DebugPrinter.INFO);

			if (current.isAtomic()) {
				// this is the last capability with such type
				for (Entry<String, Output> outputEntry : current.getOutputs().entrySet()) {
					if (outputEntry.getValue().getType().equals(io.getType()))
						lastUsed = outputEntry.getValue();
				}
			} else {
				// we add all subcapabilities to the list of capability to visit
				toVisit.addAll(current.getSubCapabilities());
			}
		}

		return lastUsed;
	}

	/**
	 * Create the end of the String corresponding to the call of this
	 * functionality (this method don't check if all required inputs as a Value)
	 * (miss the URL of the service)
	 * 
	 * @param params
	 *            this map contains the value of each param (inputLocalName ->
	 *            paramValue).
	 * @return the complete http request so as to invoke the functionality <br>
	 *         <ul>
	 *         <li> <b>ERROR - null</b> if no Grounding is found </li>
	 *         <li> <b>ERROR - uncomplete list of param</b> in case of of at
	 *         least one param is missing. </li>
	 *         </ul>
	 */
	public String formatInvokation(Map<String, String> params) {
		final String errorNull = "ERROR - null";
		final String errorParam = "ERROR - uncomplete list of param";

		GroundingProperty gp = this.getGrounding();
		if (gp == null)
			return errorNull + " (" + this.getLocalName() + ")";
		else {
			String result = "";

			// add the method name "?method="
			result += "?method=" + gp.getOperationName();

			// add the list of inputs
			Iterator<Input> it = this.getInputs().values().iterator();

			while (it.hasNext()) {
				Input in = it.next();

				// test if all params are available
				if (params.containsKey(in.getLocalName())) {
					// test the grounding
					if (in.getGroundingValue().equals(IO.UNKNOWN))
						return errorNull + " (" + in.getLocalName() + ")";

					result += "&"
							+ in.getGroundingValue()
							+ "="
							+ Tools.transformIntoEncodedURLString(params.get(in.getLocalName()));

				} else
					return errorParam + " (" + in.getLocalName() + ")";

			}

			return result;
		}
	}

	/**
	 * Check if the current Capability match with the given one
	 * 
	 * @param capa
	 *            the capability to match with.
	 * @param matcher
	 *            the matcher object responsible of the match
	 * @return a CapabilityMatch object, containing all matching information if
	 *         the match is ok, <i>null</i> otherwise
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	public CapabilityMatch match(Capability capa, Matcher matcher)
			throws SemanticTypeNotKnowException {
		return matcher.capabilityMatch(this, capa);
	}

	public int containsMoreInformation(Capability otherObject) {
		return otherObject.countNullOrUnknowField()
				- this.countNullOrUnknowField();
	}

	public int countNullOrUnknowField() {
		return (((this.getTypeName() == null) || this.getTypeName().equals(
			Capability.UNKNOWN)) ? 1 : 0)
				+ (((this.getLocalName() == null) || this.getLocalName().equals(
					Capability.UNKNOWN)) ? 1 : 0)
				+ (((this.getOntology() == null) || this.getOntology().equals(
					Capability.UNKNOWN)) ? 1 : 0)
				+ (((this.getCapabilityURI() == null) || this.getCapabilityURI().equals(
					Capability.UNKNOWN)) ? 1 : 0)
				+ (((this.getConversationURI() == null) || this.getConversationURI().equals(
					Capability.UNKNOWN)) ? 1 : 0)
				+ ((this.getCC() == null) ? 1 : 0)
				+ ((this.getService() == null) ? 1 : 0)
				+ ((this.getInputs() == null) ? 1 : 0)
				+ ((this.getOutputs() == null) ? 1 : 0)
				+ ((this.getProperties() == null) ? 1 : 0)
				+ ((this.getSubCapabilities() == null) ? 1 : 0)
				+ ((this.getGrounding() == null) ? 1 : 0);
	}

	public void transfertInformation(Capability otherObject) {
		this.setLocalName(otherObject.getLocalName());
		this.setType(otherObject.getType());
		this.setAtomic(otherObject.isAtomic());
		this.setCapabilityURI(otherObject.getCapabilityURI());
		this.setCC(otherObject.getCC());
		this.setConversationURI(otherObject.getConversationURI());
		this.setGrounding(otherObject.getGrounding());
		this.setInputs(otherObject.getInputs());
		this.setOutputs(otherObject.getOutputs());
		this.setProperties(otherObject.getProperties());
		this.setService(otherObject.getService());
		this.setSubCapabilities(otherObject.getSubCapabilities());
		this.setTypeOfCapability(otherObject.getTypeOfCapability());
		this.setInputsBinding(otherObject.getInputsBinding());
		this.setOutputsBinding(otherObject.getOutputsBinding());
	}

	/**
	 * this method increase the number of copy with one unit
	 * 
	 */
	public void incNbrCopies() {
		this.setNbrCopies(this.getNbrCopies() + 1);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.automaton = null;
		this.capabilityURI = null;
		this.CC = null;
		this.flow = null;
		this.grounding = null;
		this.localName = null;
		this.semanticType = null;
		this.service = null;

		this.subCapabilities.clear();
		this.subCapabilities = null;

		this.inputs.clear();
		this.inputs = null;

		this.inputsBinding.clear();
		this.inputsBinding = null;

		this.outputs.clear();
		this.outputs = null;

		this.outputsBinding.clear();
		this.outputsBinding = null;	

		this.properties.clear();
		this.properties = null;	
	}



}
