/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel;

import java.io.Serializable;
import java.lang.reflect.Field;

/**
 * This class will store all information about the grounding (for one
 * capability) <-> The main information required to generate the link with the real Service
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jan 30, 2007 2:18:01 PM
 */
public class GroundingProperty implements Serializable {

	/**
	 * Variable to initialise field to a non-null value
	 */
	public static final String UNKNOWN = "!!!Unknow!!!";

	/**
	 * SerialVersion of the class
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The name of the WSDL file corresponding to the operation
	 */
	private String wsdlFileName = UNKNOWN;

	/**
	 * The name of the operation corresponding to the capability (if it's a
	 * "Atomic" Capability)
	 */
	private String operationName = UNKNOWN;

	/**
	 * The port type of the operation corresponding to the capability (if it's a
	 * "Atomic" Capability)
	 */
	private String operationPortType = UNKNOWN;

	/**
	 * The name of the message send to the calling web service
	 */
	private String wsdlInputMessage = UNKNOWN;

	/**
	 * The name of the message receive from the calling web service
	 */
	private String wsdlOutputMessage = UNKNOWN;

	/**
	 * The name name of the associated conversation
	 * 
	 */
	private String conversationURI = UNKNOWN;

	/* *********** */
	/* * Methods * */
	/* *********** */

	@Override
	public String toString() {
		return this.printGrounding("");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.conversationURI = null;
		this.operationName = null;
		this.operationPortType = null;
		this.wsdlFileName = null;
		this.wsdlInputMessage = null;
		this.wsdlOutputMessage = null;
	}

	/**
	 * Print the Grounding of this class
	 * 
	 * @param prefix
	 *            the prefix to print before each writen line
	 * @return a String exposing all value of grounding
	 */
	public String printGrounding(String prefix) {
		Class c = this.getClass();
		Field[] f = c.getDeclaredFields();
		String result = prefix + "\n\t\t\t- Grounding";
		for (int i = 0; i < f.length; i++) {
			try {
				if (!f[i].getName().equals("UNKNOWN")
						&& !f[i].getName().equals("serialVersionUID"))
					result += prefix + "\n\t\t\t\t> " + f[i].getName() + " = "
							+ f[i].get(this);
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}
		return result;
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof GroundingProperty) {
			GroundingProperty gp = (GroundingProperty) o;
			return gp.getConversationURI().equals(this.getConversationURI())
					&& gp.getOperationName().equals(this.getOperationName())
					&& gp.getOperationPortType().equals(
						this.getOperationPortType())
					&& gp.getWsdlFileName().equals(gp.getWsdlFileName())
					&& gp.getWsdlInputMessage().equals(gp.getWsdlInputMessage())
					&& gp.getWsdlOutputMessage().equals(
						gp.getWsdlOutputMessage());

		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {

		long hash = 7;
		hash = 31
				* hash
				+ (null == this.getConversationURI() ? 0
						: this.getConversationURI().hashCode());
		hash = 31
				* hash
				+ (null == this.getOperationName() ? 0
						: this.getOperationName().hashCode());
		hash = 31
				* hash
				+ (null == this.getOperationPortType() ? 0
						: this.getOperationPortType().hashCode());
		hash = 31
				* hash
				+ (null == this.getWsdlFileName() ? 0
						: this.getWsdlFileName().hashCode());
		hash = 31
				* hash
				+ (null == this.getWsdlInputMessage() ? 0
						: this.getWsdlInputMessage().hashCode());
		hash = 31
				* hash
				+ (null == this.getWsdlOutputMessage() ? 0
						: this.getWsdlOutputMessage().hashCode());

		return (int) (hash % Integer.MAX_VALUE);
	}

	/* ********************* */
	/* * Getter and Setter * */
	/* ********************* */

	/**
	 * @return the conversationURI
	 */
	public String getConversationURI() {
		return conversationURI;
	}

	/**
	 * @param conversationURI
	 *            the conversationURI to set
	 */
	public void setConversationURI(String conversationURI) {
		this.conversationURI = conversationURI;
	}

	/**
	 * @return the operationName
	 */
	public String getOperationName() {
		return operationName;
	}

	/**
	 * @param operationName
	 *            the operationName to set
	 */
	public void setOperationName(String operationName) {
		this.operationName = operationName;
	}

	/**
	 * @return the operationPortType
	 */
	public String getOperationPortType() {
		return operationPortType;
	}

	/**
	 * @param operationPortType
	 *            the operationPortType to set
	 */
	public void setOperationPortType(String operationPortType) {
		this.operationPortType = operationPortType;
	}

	/**
	 * @return the wsdlFileName
	 */
	public String getWsdlFileName() {
		return wsdlFileName;
	}

	/**
	 * @param wsdlFileName
	 *            the wsdlFileName to set
	 */
	public void setWsdlFileName(String wsdlFileName) {
		this.wsdlFileName = wsdlFileName;
	}

	/**
	 * @return the wsdlInputMessage
	 */
	public String getWsdlInputMessage() {
		return wsdlInputMessage;
	}

	/**
	 * @param wsdlInputMessage
	 *            the wsdlInputMessage to set
	 */
	public void setWsdlInputMessage(String wsdlInputMessage) {
		this.wsdlInputMessage = wsdlInputMessage;
	}

	/**
	 * @return the wsdlOutputMessage
	 */
	public String getWsdlOutputMessage() {
		return wsdlOutputMessage;
	}

	/**
	 * @param wsdlOutputMessage
	 *            the wsdlOutputMessage to set
	 */
	public void setWsdlOutputMessage(String wsdlOutputMessage) {
		this.wsdlOutputMessage = wsdlOutputMessage;
	}

}
