/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel;

import java.io.Serializable;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;
import fr.inria.icocoa.servicemodel.semantictype.SemanticTypeFactory;

/**
 * This class extract common base of Input and Output. This class contains fields relative to the link with i/o of real service.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jan 30, 2007 8:59:59 AM
 */
public class IO implements Serializable {

	/**
	 * Variable to initialise field to a non-null value
	 */
	public static final String UNKNOWN = "!!!Unknow!!!";
	
	/**
	 * The semantic semanticType of the IO
	 */
	private SemanticType semanticType = null;

	/**
	 * SerialVersion of the class
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * the local typeName of the IO
	 */
	private String localName = UNKNOWN;
	
	/**
	 * The typeName of the operation in the WSDL file
	 */
	private String groundingValue = UNKNOWN;

	/**
	 * The Capability from wich this object is an IO
	 */
	private Capability parentCapability = null;

	/**
	 * Indicate if it is a provided variable or not (appear in the service as
	 * Input/Output of a provided capability)
	 */
	private boolean provided = false;

	protected IO() {
		super();
	}

	/**
	 * Constructor of the class
	 * 
	 * @param typeName
	 *            the typeName of the concept (whithout ontology typeName)
	 * @param ontology
	 *            the typeName of the ontology from which the concept is extract
	 * @param localName
	 *            the typeName of the IO in the file
	 */
	protected IO(String typeName, String ontology, String localName) {
		this.semanticType = SemanticTypeFactory.getInstance().getObject(typeName, ontology);
		this.localName = localName;
	}

	/**
	 * Constructor of an IO Object by giving the local URI of the individual and
	 * the rdfType of this one.
	 * 
	 * @param localNameURI
	 *            the URI of the individual coding the IO
	 * @param rdfTypeURI
	 *            the rdf Type (= semantic Type of the IO)
	 */
	protected IO(String localNameURI, String rdfTypeURI) {
		this(Tools.getSecondPart(rdfTypeURI), Tools.getFirstPart(rdfTypeURI),
				localNameURI);		
	}	

	/**
	 * @return the typeName
	 */
	public String getTypeName() {
		return semanticType.getTypeName();
	}

	/**
	 * @param localName
	 *            the localName to set
	 */
	public void setLocalName(String localName) {
		this.localName = localName;
	}

	/**
	 * @return the localName
	 */
	public String getLocalName() {
		return localName;
	}
	

	/**
	 * @return the ontology
	 */
	public String getOntology() {
		return this.semanticType.getOntology();
	}

	/**
	 * @return the groundingValue
	 */
	public String getGroundingValue() {
		return groundingValue;
	}

	/**
	 * @param groundingValue
	 *            the groundingValue to set
	 */
	public void setGroundingValue(String groundingValue) {
		this.groundingValue = groundingValue;
	}

	/**
	 * @return the URI
	 */
	public String getURI() {
		return this.localName;
	}

	/**
	 * @return the provided
	 */
	public boolean isProvided() {
		return provided;
	}

	/**
	 * @param provided
	 *            the provided to set
	 */
	public void setProvided(boolean provided) {
		this.provided = provided;
	}

	/**
	 * 
	 * @return the semanticType of the IO
	 */
	public String getType() {
		return this.getOntology() + "#" + this.getTypeName();
	}

	/**
	 * @return the parentCapability
	 */
	public Capability getParentCapability() {
		return parentCapability;
	}

	/**
	 * @param parentCapability
	 *            the parentCapability to set
	 */
	public void setParentCapability(Capability parentCapability) {
		this.parentCapability = parentCapability;
	}
	
	

	/**
	 * @return the semanticType
	 */
	public SemanticType getSemanticType() {
		return semanticType;
	}

	/**
	 * @param semanticType the semanticType to set
	 */
	public void setSemanticType(SemanticType semanticType) {
		this.semanticType = semanticType;
	}

	/**
	 * Set a new rdtType value
	 * 
	 * @param newType :
	 *            the new semanticType to set
	 */
	public void setType(String newType) {
		this.getSemanticType().setType(newType);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result
				+ ((this.getGroundingValue() == null) ? 0 : this.getGroundingValue().hashCode());
		result = PRIME * result
				+ ((this.getLocalName() == null) ? 0 : this.getLocalName().hashCode());
		result = PRIME * result
				+ ((this.getOntology() == null) ? 0 : this.getOntology().hashCode());
		result = PRIME * result + (provided ? 1231 : 1237);
		result = PRIME * result
				+ ((this.getTypeName() == null) ? 0 : this.getTypeName().hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final IO other = (IO) obj;
		if (this.getGroundingValue() == null) {
			if (other.getGroundingValue() != null)
				return false;
		} else if (!this.getGroundingValue().equals(other.getGroundingValue()))
			return false;
		if (this.getLocalName() == null) {
			if (other.getLocalName() != null)
				return false;
		} else if (!this.getLocalName().equals(other.getLocalName()))
			return false;
		if (this.getOntology() == null) {
			if (other.getOntology() != null)
				return false;
		} else if (!this.getOntology().equals(other.getOntology()))
			return false;
		if (provided != other.provided)
			return false;
		if (this.getTypeName() == null) {
			if (other.getTypeName() != null)
				return false;
		} else if (!this.getTypeName().equals(other.getTypeName()))
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.groundingValue = null;
		this.localName = null;
		this.semanticType = null;
		this.parentCapability = null;
	}

	
	/**
	 * Check if the IO match with the given semanticType <br>
	 * This method return
	 * <ul>
	 * <li><b>MatchingType.EXACT</b> if the match is exact</li>,
	 * <li><b>MatchingType.PLUGIN</b> if the current semanticType subsum the
	 * "semanticType",</li>
	 * <li><b>MatchingType.SUBSUM</b> if the "semanticType" subsum the current
	 * semanticType,</li>
	 * <li><b>MatchingType.FAIL</b> if there is no match</li>
	 * </ul>
	 * 
	 * 
	 * @param semanticType
	 *            the semanticType we want to check the matching with.
	 * @param matcher
	 *            the matcher object responsible of the match
	 * @return an integer corresponding to a specified MatchingType value
	 * @throws SemanticTypeNotKnowException
	 *             if one semanticType is not found
	 */
	public int match(String semanticType, Matcher matcher)
			throws SemanticTypeNotKnowException {
		return matcher.ioMatch(this.getType(), semanticType);

	}

	/**
	 * Count the number of Null or UNKNOW Field of the current value
	 * 
	 * @return this number
	 */
	public int countNullOrUnknowField() {
		return (((this.getTypeName() == null) || this.getTypeName().equals(
			IO.UNKNOWN)) ? 1 : 0)
				+ (((this.getLocalName() == null) || this.getLocalName().equals(
					IO.UNKNOWN)) ? 1 : 0)
				+ (((this.getOntology() == null) || this.getOntology().equals(
					IO.UNKNOWN)) ? 1 : 0)
				+ (((this.getGroundingValue() == null) || this.getGroundingValue().equals(
					IO.UNKNOWN)) ? 1 : 0);
	}

	/**
	 * Transfert all the field from otherObject to the current object (erase the
	 * already existing content)
	 * 
	 * @param otherObject
	 *            the object from which value will be extracted.
	 */
	protected void transfertInformation(IO otherObject) {
		this.setGroundingValue(otherObject.getGroundingValue());
		this.setLocalName(otherObject.getLocalName());
		this.setType(otherObject.getType());
		this.setProvided(otherObject.isProvided());
	}

}