/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel;

import java.io.Serializable;

/**
 * This class represent a parameter binding (a link between two Input/Output). 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Feb 5, 2007 9:18:40 AM
 */
public class ParameterBinding implements Serializable, Cloneable {

	/**
	 * Variable to initialise field to a non-null value
	 */
	public static final String UNKNOWN = "!!!Unknow!!!";

	/**
	 * Particular string that mean that the target/source Capability is the
	 * parent one
	 */
	public static final String THEPARENTPERFORM = "#TheParentPerform";

	/**
	 * SerialVersion of the class
	 */
	private static final long serialVersionUID = 1L;

	/** The name of the target variable */
	private String targetVariable = UNKNOWN;

	/** The name of the source variable */
	private String sourceVariable = UNKNOWN;

	/** The name of the capability corresponding to the source variable */
	private String sourceCapability = UNKNOWN;

	/** The name of the capability corresponding to the target variable */
	private String targetCapability = UNKNOWN;

	/** Declare if it is an InputBinding or an OutputBinding */
	private boolean isInputBinding = false;

	/**
	 * @param targetVariable
	 *            The name of the target variable
	 * @param sourceVariable
	 *            The name of the source variable
	 * @param sourceCapability
	 *            The name of the capability corresponding to the source
	 *            variable
	 * @param isInputBinding
	 *            Declare if it is an InputBinding or an OutputBinding
	 */
	public ParameterBinding(String targetVariable, String sourceVariable,
			String sourceCapability, boolean isInputBinding) {
		this(targetVariable, ParameterBinding.UNKNOWN, sourceVariable,
				sourceCapability, isInputBinding);
	}

	/**
	 * @param targetVariable
	 *            The name of the target variable
	 * @param targetCapability
	 *            The name of the capability corresponding to the target
	 *            variable
	 * @param sourceVariable
	 *            The name of the source variable
	 * @param sourceCapability
	 *            The name of the capability corresponding to the source
	 *            variable
	 * @param isInputBinding
	 *            Declare if it is an InputBinding or an OutputBinding
	 */
	public ParameterBinding(String targetVariable, String targetCapability,
			String sourceVariable, String sourceCapability,
			boolean isInputBinding) {
		this.targetVariable = targetVariable;
		this.sourceVariable = sourceVariable;
		this.sourceCapability = sourceCapability;
		this.targetCapability = targetCapability;
		this.isInputBinding = isInputBinding;
	}

	/**
	 * @return the sourceCapability
	 */
	public String getSourceCapability() {
		return sourceCapability;
	}

	/**
	 * @param sourceCapability
	 *            the sourceCapability to set
	 */
	public void setSourceCapability(String sourceCapability) {
		this.sourceCapability = sourceCapability;
	}

	/**
	 * @return the sourceVariable
	 */
	public String getSourceVariable() {
		return sourceVariable;
	}

	/**
	 * @param sourceVariable
	 *            the sourceVariable to set
	 */
	public void setSourceVariable(String sourceVariable) {
		this.sourceVariable = sourceVariable;
	}

	/**
	 * @return the targetVariable
	 */
	public String getTargetVariable() {
		return targetVariable;
	}

	/**
	 * @param targetVariable
	 *            the targetVariable to set
	 */
	public void setTargetVariable(String targetVariable) {
		this.targetVariable = targetVariable;
	}

	/**
	 * @return the isInputBinding
	 */
	public boolean isInputBinding() {
		return isInputBinding;
	}

	/**
	 * @return the targetCapability
	 */
	public String getTargetCapability() {
		return targetCapability;
	}

	/**
	 * @param targetCapability
	 *            the targetCapability to set
	 */
	public void setTargetCapability(String targetCapability) {
		this.targetCapability = targetCapability;
	}

	/**
	 * @param isInputBinding
	 *            the isInputBinding to set
	 */
	public void setInputBinding(boolean isInputBinding) {
		this.isInputBinding = isInputBinding;
	}

	@Override
	public String toString() {
		// if (!isInputBinding) {
		return "Binding from " + sourceCapability + "@" + sourceVariable
				+ " to " + targetCapability + "@" + targetVariable;
		/*
		 * } else { return "Binding from " + sourceVariable + " to " +
		 * sourceCapability + "." + targetVariable; }
		 */
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof ParameterBinding) {
			ParameterBinding pb = (ParameterBinding) o;
			return pb.getSourceCapability().equals(this.getSourceCapability())
					&& pb.getSourceVariable().equals(this.getSourceVariable())
					&& pb.getTargetCapability().equals(
						this.getTargetCapability())
					&& pb.getTargetVariable().equals(this.getTargetVariable())
					&& pb.isInputBinding() == this.isInputBinding();
		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		long hash = 7;
		hash = 31
				* hash
				+ (null == this.getSourceCapability() ? 0
						: this.getSourceCapability().hashCode());
		hash = 31
				* hash
				+ (null == this.getSourceVariable() ? 0
						: this.getSourceVariable().hashCode());
		hash = 31
				* hash
				+ (null == this.getTargetVariable() ? 0
						: this.getTargetVariable().hashCode());
		hash = 31
				* hash
				+ (null == this.getTargetCapability() ? 0
						: this.getTargetCapability().hashCode());
		hash = 31 * hash + (isInputBinding() ? 0 : 1);

		return (int) (hash % Integer.MAX_VALUE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	public ParameterBinding clone() throws CloneNotSupportedException {
		return new ParameterBinding(this.getTargetVariable(),
				this.getTargetCapability(), this.getSourceVariable(),
				this.sourceCapability, this.isInputBinding());
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.sourceCapability = null;
		this.targetCapability = null;
		this.sourceVariable = null;
		this.targetVariable = null;
	}

}
