/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.CapabilityAlreadyRegisteredException;
import fr.inria.icocoa.exception.SameURIException;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.automaton.Automaton;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.ServiceMatch;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;
import fr.inria.icocoa.servicemodel.semantictype.SemanticTypeFactory;
import fr.inria.icocoa.servicemodel.toolsobjects.ProcessElements;

/**
 * A Service is a set of Functionality describe in a file in language based on
 * XML. This language make the distinction between Required, Provided and other
 * capabilities.<br>
 * <br>
 * The functionality are register in maps (3 maps corresponding to Required,
 * provided and other). <br>
 * WARNING : a service can't contain two Capabilities with the same
 * localName(type).
 * 
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 * @author <a href="mailto:Graham.Thomson@inria.fr">Graham Thomson</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 */
public class Service implements Serializable, ProcessInterface {

	/**
	 * the serial version of the class
	 */
	private static final long serialVersionUID = 8577139272851860254L;

	/**
	 * Variable to initialise field to a non-null value
	 */
	public static final String UNKNOWN = "!!!Unknow!!!";

	/** The localName of the service */
	protected String localName = UNKNOWN;

	/** The base localName of the service */
	protected String base = UNKNOWN;

	
	/**
	 * The semantic type of the Capability
	 */
	protected SemanticType semanticType = null;

	/**
	 * Map of capabilities declared as provider in a Service Description
	 * (CapabilityName -> CabilityObject)
	 */
	protected Map<String, Capability> provided_Capabilities = null;

	/**
	 * Map of Capabilities declared as required in a Task (CapabilityName ->
	 * CabilityObject)
	 */
	protected Map<String, Capability> required_Capabilities = null;

	/**
	 * Remaining capabilities, that are neither required nor provided
	 * (CapabilityName -> CabilityObject)
	 */
	protected Map<String, Capability> capabilities = null;

	/** Tag to indicate if the service is a task or not (default not) */
	protected boolean isUserTask = false;

	/** Map of all binded_varaible (targetVariableName -> BindingObject) */
	private Map<String, ParameterBinding> binded_variable = null;

	/** The URL of the owlDescriptionFile */
	protected String owlDescriptionFile = UNKNOWN;

	/** The URL of the owlDescriptionFile */
	protected String wsdlDescriptionFile = UNKNOWN;

	/** The content of the WSDL file associated to the service */
	protected String wsdlContent = UNKNOWN;
	
	/** The Qos properties data */
	protected QoSProperty qos = null;

	/** indicate if the Service is a composite service or not */
	private boolean isComposedService = false;
	
	/** The  global automaton associated to the service/task */
	private Automaton automaton = null;

	/* **************** */
	/* * Constructors * */
	/* **************** */

	/**
	 * Default constructor. Initialise Maps.
	 * 
	 */
	public Service() {
		this(UNKNOWN);
	}

	/**
	 * Constructor that initialise Maps and localName
	 * 
	 * @param localName
	 *            the localName of the service !
	 */
	public Service(String localName) {
		this.semanticType = SemanticTypeFactory.getInstance().getObject();
		provided_Capabilities = new HashMap<String, Capability>();
		required_Capabilities = new HashMap<String, Capability>();
		capabilities = new HashMap<String, Capability>();
		binded_variable = new HashMap<String, ParameterBinding>();
		this.localName = localName;
	}

	/**
	 * Construct a new service object giving its main properties
	 * @param qualifiedName the name of the service that will be used by the system to find it again
	 * @param type the Semantic type of the service (the complete URI of the concept)
	 * @param wsdl the URL of the wsdl description file
	 * @param isComposed indicate if the created service is composite or no.
	 */
	public Service(String qualifiedName, String type, String wsdl,
			boolean isComposed) {
		this.semanticType = SemanticTypeFactory.getInstance().getObject(Tools.getSecondPart(type), Tools.getFirstPart(type));
		setQualifiedName(qualifiedName);
		this.wsdlDescriptionFile = wsdl;
		this.isComposedService = isComposed;
	}

	

	/* ****************************** */
	/* * Overwrite of Object method * */
	/* ****************************** */

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String result = "Service :\n--------\n\t-localName : " + localName
				+ "\n\t-Type : " + this.getType() + "\n\t-Base : "
				+ this.getBase() + "\n\t-DescriptionOwlFile : "
				+ this.getOwlDescriptionFile() + "\n\t-WSDLDescriptionFile : "
				+ this.getWsdlDescriptionFile()
				+ "\n\t-Number Of RequiredCapability : "
				+ this.getRequiredCapabilities().size();
		for (int i = 0; i < this.getRequiredCapabilities().size(); i++) {
			result += "\n\t\t-> "
					+ (this.getRequiredCapabilities().get(i)).getTypeName()
					+ " - "
					+ (this.getRequiredCapabilities().get(i)).getLocalName();
		}
		result += "\n\t-Number Of ProvidedCapability : "
				+ this.getProvidedCapabilities().size();
		for (int i = 0; i < this.getProvidedCapabilities().size(); i++) {
			result += "\n\t\t-> "
					+ (this.getProvidedCapabilities().get(i)).getTypeName()
					+ " - "
					+ (this.getProvidedCapabilities().get(i)).getLocalName();
		}
		result += "\n\t-Number Of Capability : "
				+ this.getCapabilities().size();
		for (int i = 0; i < this.getCapabilities().size(); i++) {
			result += "\n\t\t-> "
					+ (this.getCapabilities().get(i)).getTypeName() + " - "
					+ (this.getCapabilities().get(i)).getLocalName();
		}
		if (this.getQos() != null)
			result += "\n\t-Qos : " + this.getQos().toString();
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}

		if (obj instanceof Service) {
			Service s = (Service) obj;
			return this.equalsWithoutCapabilities(obj)
					&& Tools.compareMap(this.getProvided_Capabilities(),
						s.getProvided_Capabilities())
					&& Tools.compareMap(this.getRequired_Capabilities(),
						s.getRequired_Capabilities())
					&& Tools.compareMap(this.getCapabilitiesMap(),
						s.getCapabilitiesMap());
		} else {
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		long hash = 7;
		hash = 31
				* hash
				+ (null == this.getLocalName() ? 0
						: this.getLocalName().hashCode());
		hash = 31 * hash
				+ (null == this.getType() ? 0 : this.getType().hashCode());
		hash = 31 * hash
				+ (null == this.getBase() ? 0 : this.getBase().hashCode());
		hash = 31
				* hash
				+ (null == this.getOwlDescriptionFile() ? 0
						: this.getOwlDescriptionFile().hashCode());
		hash = 31
				* hash
				+ (null == this.getWsdlDescriptionFile() ? 0
						: this.getWsdlDescriptionFile().hashCode());
		hash = 31 * hash + (this.isUserTask() ? 0 : 1);
		hash = 31
				* hash
				+ (null == this.getCapabilitiesMap() ? 0
						: this.getCapabilitiesMap().hashCode());
		hash = 31
				* hash
				+ (null == this.getProvided_Capabilities() ? 0
						: this.getProvided_Capabilities().hashCode());
		hash = 31
				* hash
				+ (null == this.getRequired_Capabilities() ? 0
						: this.getRequired_Capabilities().hashCode());
		hash = 31
				* hash
				+ (null == this.getBinded_variable() ? 0
						: this.getBinded_variable().hashCode());

		hash = 31 * hash
				+ (null == this.getQos() ? 0 : this.getQos().hashCode());

		return (int) (hash % Integer.MAX_VALUE);
	}

	/**
	 * This method permit to test if two service are equals without considering
	 * the Capabilities
	 * 
	 * @param o :
	 *            the object to compare
	 * @return <i>true</i> if the two object are equivalent, <i>false</i>
	 *         otherwise
	 */
	public boolean equalsWithoutCapabilities(Object o) {
		// cannot equals with a null object
		if (o == null) {
			return false;
		}

		if (o instanceof Service) {
			Service s = (Service) o;
			if (s != null) {
				boolean compareLocalName = Tools.compareString(
					this.getLocalName(), s.getLocalName());
				boolean compareBase = Tools.compareString(this.getBase(),
					s.getBase());
				boolean compareType = Tools.compareString(this.getType(),
					s.getType());
				boolean compareOwlDescription = Tools.compareString(
					this.getOwlDescriptionFile(), s.getOwlDescriptionFile());
				boolean compareWSDLDescription = Tools.compareString(
					this.getWsdlDescriptionFile(), s.getWsdlDescriptionFile());
				boolean testQos = ((this.getQos() == null) ? (s.getQos() == null)
						: ((s.getQos() == null) ? false : this.getQos().equals(
							s.getQos())));
				boolean testUserTask = this.isUserTask() == s.isUserTask();
				boolean testBindedVariables = Tools.compareMap(
					this.getBinded_variable(), s.getBinded_variable());
				return compareLocalName && compareBase && compareType
						&& compareOwlDescription && compareWSDLDescription
						&& testQos && testUserTask && testBindedVariables;
				/*
				 * return Tools.compareString(this.getLocalName(), s
				 * .getLocalName()) && Tools.compareString(this.getBase(),
				 * s.getBase()) && Tools.compareString(this.getType(),
				 * s.getType()) &&
				 * Tools.compareString(this.getOwlDescriptionFile(), s
				 * .getOwlDescriptionFile()) &&
				 * Tools.compareString(this.getWsdlDescriptionFile(), s
				 * .getWsdlDescriptionFile()) && ((this.getQos() == null) ?
				 * (s.getQos() == null) : ((s.getQos() == null) ? false :
				 * this.getQos() .equals(s.getQos()))) && this.isUserTask() ==
				 * s.isUserTask() && Tools.compareMap(this.getBinded_variable(),
				 * s .getBinded_variable());
				 */
			} else
				return false;
		} else
			return false;

	}

	/**
	 * This method permit to expand a service and print the result on the
	 * defined System.out
	 * 
	 * @param out :
	 *            the Stream where the result will be printed
	 * 
	 */
	public void print(java.io.PrintStream out) {

		/* print main information */
		out.println("Service :\n--------\n\t-localName : " + localName
				+ "\n\t-Type : " + this.getSemanticType() + "\n\t-Base : "
				+ this.getBase() + "\n\t-DescriptionOwlFile : "
				+ this.getOwlDescriptionFile() + "\n\t-WSDLDescriptionFile : "
				+ this.getWsdlDescriptionFile()
				+ "\n\t-Number Of RequiredCapability : "
				+ this.getRequiredCapabilities().size());

		/* print required capabilities */
		for (int i = 0; i < this.getRequiredCapabilities().size(); i++) {
			out.println("\n\t\t-> "
					+ (this.getRequiredCapabilities().get(i)).toString("\t\t"));
		}

		/* print provided capabilities */
		out.println("\n\t-Number Of ProvidedCapability : "
				+ this.getProvidedCapabilities().size());
		for (int i = 0; i < this.getProvidedCapabilities().size(); i++) {
			out.println("\n\t\t-> "
					+ (this.getProvidedCapabilities().get(i)).toString("\t\t"));
		}

		/* print other capabilities */
		out.println("\n\t-Number Of Other Capability (Subcapabilities) : "
				+ this.getCapabilities().size());
		for (int i = 0; i < this.getCapabilities().size(); i++) {
			out.println("\n\t\t-> "
					+ (this.getCapabilities().get(i)).toString("\t\t"));
		}

		/* print QoS */
		if (this.getQos() != null)
			out.println("\n\t Qos : " + this.getQos().toString());

	}

	/* ************************** */
	/* * Add/Get method for map * */
	/* ************************** */

	// ///////////////
	// Add methods //
	// ///////////////
	/**
	 * This method add a capability in the map of Capabilities <br>
	 * <b>WARNING:</b><br>
	 * <ul>
	 * <li>a service can't contain two Capabilities with the same
	 * localName(type).</li>
	 * <li>if two capability, not equivalent, but has the same Conversation URI
	 * (not correct) will be add in this map, then
	 * </ul>
	 * 
	 * @param c
	 *            the capability to add
	 * @throws CapabilityAlreadyRegisteredException
	 *             this exception is thrown if you try two register two time a
	 *             equivalent Capability in the service.
	 * @throws SameURIException
	 *             this exception is thrown when you try to register a
	 *             Capability with the same CapabilityURI that another
	 *             Capability, already registered in this map
	 * 
	 */
	public void addCapability_E(Capability c)
			throws CapabilityAlreadyRegisteredException, SameURIException {
		if (this.containsTheCapability(c)) {
			throw new CapabilityAlreadyRegisteredException(
					"This capability is already registered in this service");
		}

		if (this.capabilities.containsKey(c.getCapabilityURI())) {
			throw new SameURIException(
					"An non equivalent Capability but with the same Capability URI has been ever register in the map of Capabilities");
		}

		c.setTypeOfCapability(Capability.SUBCAPABILITY);
		this.capabilities.put(c.getCapabilityURI(), c);
	}

	/**
	 * This method add a capability in the map of provided Capabilities <br>
	 * <b>WARNING:</b><br>
	 * <ul>
	 * <li>a service can't contain two Capabilities with the same
	 * localName(type).</li>
	 * <li>if two capability, not equivalent, but has the same Conversation URI
	 * (not correct) will be add in this map, then
	 * </ul>
	 * 
	 * @param c
	 *            the capability to add
	 * @throws CapabilityAlreadyRegisteredException
	 *             this exception is thrown if you try two register two time a
	 *             equivalent Capability in the service.
	 * @throws SameURIException
	 *             this exception is thrown when you try to register a
	 *             Capability with the same CapabilityURI that another
	 *             Capability, already registered in this map
	 * 
	 */
	public void addProvidedCapability_E(Capability c)
			throws CapabilityAlreadyRegisteredException, SameURIException {
		if (this.containsTheCapability(c)) {
			throw new CapabilityAlreadyRegisteredException(
					"This capability is already registered in this service");
		}

		if (this.provided_Capabilities.containsKey(c.getCapabilityURI())) {
			throw new SameURIException(
					"An non equivalent Capability but with the same Capability URI has been ever register in the map of Capabilities");
		}

		c.setTypeOfCapability(Capability.PROVIDED);
		this.provided_Capabilities.put(c.getCapabilityURI(), c);
	}

	/**
	 * This method add a capability in the map of required Capabilities <br>
	 * <b>WARNING:</b><br>
	 * <ul>
	 * <li>a service can't contain two Capabilities with the same
	 * localName(type).</li>
	 * <li>if two capability, not equivalent, but has the same Conversation URI
	 * (not correct) will be add in this map, then
	 * </ul>
	 * 
	 * @param c
	 *            the capability to add
	 * @throws CapabilityAlreadyRegisteredException
	 *             this exception is thrown if you try two register two time a
	 *             equivalent Capability in the service.
	 * @throws SameURIException
	 *             this exception is thrown when you try to register a
	 *             Capability with the same CapabilityURI that another
	 *             Capability, already registered in this map
	 * 
	 */
	public void addRequiredCapability_E(Capability c)
			throws CapabilityAlreadyRegisteredException, SameURIException {
		if (this.containsTheCapability(c)) {
			throw new CapabilityAlreadyRegisteredException(
					"This capability is already registered in this service");
		}

		if (this.required_Capabilities.containsKey(c.getCapabilityURI())) {
			throw new SameURIException(
					"An non equivalent Capability but with the same Capability URI has been ever register in the map of Capabilities");
		}

		c.setTypeOfCapability(Capability.REQUIRED);
		this.required_Capabilities.put(c.getCapabilityURI(), c);
	}

	/**
	 * This method add a capability in the map of Capabilities <br>
	 * <b>WARNING:</b><br>
	 * <ul>
	 * <li>a service can't contain two Capabilities with the same
	 * localName(type).</li>
	 * <li>if two capability, not equivalent, but has the same Conversation URI
	 * (not correct) will be add in this map, then
	 * </ul>
	 * 
	 * @param c
	 *            the capability to add
	 * @return <i>true</i> if the Capability has been had successfully,
	 *         <i>false</i> otherwise
	 */
	public boolean addCapability(Capability c) {
		try {
			this.addCapability_E(c);
			return true;
		} catch (CapabilityAlreadyRegisteredException e) {
			return false;
		} catch (SameURIException e2) {
			return false;
		}
	}

	/**
	 * This method add a capability in the map of Provided Capabilities<br>
	 * <b>WARNING:</b><br>
	 * <ul>
	 * <li>a service can't contain two Capabilities with the same
	 * localName(type).</li>
	 * <li>if two capability, not equivalent, but has the same Conversation URI
	 * (not correct) will be add in this map, then
	 * </ul>
	 * 
	 * @param c
	 *            the capability to add
	 * @return <i>true</i> if the Capability has been had successfully,
	 *         <i>false</i> otherwise
	 */
	public boolean addProvidedCapability(Capability c) {
		try {
			this.addProvidedCapability_E(c);
			return true;
		} catch (CapabilityAlreadyRegisteredException e) {
			return false;
		} catch (SameURIException e2) {
			return false;
		}
	}

	/**
	 * This method add a capability in the map of Required Capabilities<br>
	 * <b>WARNING:</b><br>
	 * <ul>
	 * <li>a service can't contain two Capabilities with the same
	 * localName(type).</li>
	 * <li>if two capability, not equivalent, but has the same Conversation URI
	 * (not correct) will be add in this map, then
	 * </ul>
	 * 
	 * @param c
	 *            the capability to add
	 * @return <i>true</i> if the Capability has been had successfully,
	 *         <i>false</i> otherwise
	 */
	public boolean addRequiredCapability(Capability c) {
		try {
			this.addRequiredCapability_E(c);
			return true;
		} catch (CapabilityAlreadyRegisteredException e) {
			return false;
		} catch (SameURIException e2) {
			return false;
		}
	}

	/**
	 * This method add a parameter of binding in the corresponding map.
	 * 
	 * @param b
	 *            the binding to add
	 */
	public void addParameterBinding(ParameterBinding b) {
		this.binded_variable.put(b.getTargetVariable(), b);
	}

	// ///////////////
	// Get methods //
	// ///////////////

	/**
	 * This method test if the given capability is contained in the Service.
	 * <br>
	 * <b>Remark:</b> the capabilities are compare without comparing
	 * subcapabilities
	 * 
	 * @param c
	 *            The capability object
	 * @return <i>true</i> if the Capability is contained into the Service
	 *         (search first in Capability, then Required then Provided),
	 *         <i>false</i> otherwise.
	 */
	public boolean containsTheCapability(Capability c) {
		boolean found = false;

		// store original type
		int originalType = c.getTypeOfCapability();

		// walk through map of Capability
		Iterator<Capability> it = this.getCapabilitiesMap().values().iterator();
		c.setTypeOfCapability(Capability.SUBCAPABILITY);// change type to be
		// compared without
		// error
		while (!found && it.hasNext()) {
			found = it.next().equalsWithoutSubCapabilities(c);
		}

		// walk through map of Required Capability
		it = this.getRequired_Capabilities().values().iterator();
		c.setTypeOfCapability(Capability.REQUIRED);// change type to be
		// compared without error
		while (!found && it.hasNext()) {
			found = it.next().equalsWithoutSubCapabilities(c);
		}

		// walk through map of Provided Capability
		it = this.getProvided_Capabilities().values().iterator();
		c.setTypeOfCapability(Capability.PROVIDED);// change type to be
		// compared without error
		while (!found && it.hasNext()) {
			found = it.next().equalsWithoutSubCapabilities(c);
		}

		// restore the original type
		c.setTypeOfCapability(originalType);
		return found;
	}

	/* ****************************** */
	/* * Interface ProcessInterface * */
	/* ****************************** */

	/**
	 * get the map (InputURI->Input Object) thanks to the CapabilityURI
	 * 
	 * @param capability_uri
	 *            the URI of the capability (ex :
	 *            http://www.hitech-projects.com/euprojects/amigo/VerySimpleService.owl#ServiceAOp1Capability)
	 * @return the required map, <i>null</i> if there is no capability with
	 *         such URI
	 */
	public Map<String, Input> getInputs(String capability_uri) {
		if (this.capabilities.containsKey(capability_uri)) {
			return this.capabilities.get(capability_uri).getInputs();
		} else if (this.provided_Capabilities.containsKey(capability_uri)) {
			return this.provided_Capabilities.get(capability_uri).getInputs();
		} else if (this.required_Capabilities.containsKey(capability_uri)) {
			return this.required_Capabilities.get(capability_uri).getInputs();
		} else {
			return null;
		}
	}

	/**
	 * get the map (OutputURI->Output Object) thanks to the CapabilityURI
	 * 
	 * @param capability_uri
	 *            the URI of the capability (ex :
	 *            http://www.hitech-projects.com/euprojects/amigo/VerySimpleService.owl#ServiceAOp1Capability)
	 * @return the required map, <i>null</i> if there is no capability with
	 *         such URI
	 */
	public Map<String, Output> getOutputs(String capability_uri) {
		if (this.capabilities.containsKey(capability_uri)) {
			return this.capabilities.get(capability_uri).getOutputs();
		} else if (this.provided_Capabilities.containsKey(capability_uri)) {
			return this.provided_Capabilities.get(capability_uri).getOutputs();
		} else if (this.required_Capabilities.containsKey(capability_uri)) {
			return this.required_Capabilities.get(capability_uri).getOutputs();
		} else {
			return null;
		}
	}

	/**
	 * Get the list of the URI of the required capabilities contained in this
	 * Service Object
	 * 
	 * @return the list of URI of required capabilities, empty list if there is
	 *         no Required Capability
	 */
	public List<String> getRequiredTask() {
		List<String> result = new ArrayList<String>();
		for (Iterator<Capability> it = this.required_Capabilities.values().iterator(); it.hasNext();) {
			result.add(it.next().getCapabilityURI());
		}
		return result;
	}

	/**
	 * Get the URI of the conversation/process of a Capability from its URI
	 * 
	 * @param capability_uri
	 *            the URI of the Capability we want to obtain the Process URI
	 *            (ex :
	 *            http://www.hitech-projects.com/euprojects/amigo/VerySimpleService.owl#ServiceAOp1Capability)
	 * @return the URI of the corresponding process, <i>null</i> if no
	 *         capability with such URI exists (in this service)
	 */
	public String getTaskProcess(String capability_uri) {
		if (this.capabilities.containsKey(capability_uri)) {
			return this.capabilities.get(capability_uri).getConversationURI();
		} else if (this.provided_Capabilities.containsKey(capability_uri)) {
			return this.provided_Capabilities.get(capability_uri).getConversationURI();
		} else if (this.required_Capabilities.containsKey(capability_uri)) {
			return this.required_Capabilities.get(capability_uri).getConversationURI();
		} else {
			return null;
		}
	}

	/**
	 * Get if a Capability is an atomic one from its URI
	 * 
	 * @param capability_uri
	 *            the URI of the Capability we want to obtain the atomicity (ex :
	 *            http://www.hitech-projects.com/euprojects/amigo/VerySimpleService.owl#ServiceAOp1Capability)
	 * @return <i>true</i> if this capability is atomic, <i>false</i>
	 *         otherwise.
	 */
	public boolean isAtomic(String capability_uri) {
		if (this.capabilities.containsKey(capability_uri)) {
			return this.capabilities.get(capability_uri).isAtomic();
		} else if (this.provided_Capabilities.containsKey(capability_uri)) {
			return this.provided_Capabilities.get(capability_uri).isAtomic();
		} else if (this.required_Capabilities.containsKey(capability_uri)) {
			return this.required_Capabilities.get(capability_uri).isAtomic();
		} else {
			return false;
		}
	}

	/**
	 * Get if a Capability is a composite one from its URI
	 * 
	 * @param capability_uri
	 *            the URI of the Capability we want to obtain the atomicity (ex :
	 *            http://www.hitech-projects.com/euprojects/amigo/VerySimpleService.owl#ServiceAOp1Capability)
	 * @return <i>true</i> if this capability is composite, <i>false</i>
	 *         otherwise.
	 */
	public boolean isComposite(String capability_uri) {
		return !this.isAtomic(capability_uri);
	}

	/**
	 * Get a the Capability object corresponding to a required capability URI
	 * 
	 * @param capabilityURI
	 *            the URI of the Capability we want to obtain the atomicity (ex :
	 *            http://www.hitech-projects.com/euprojects/amigo/VerySimpleService.owl#ServiceAOp1Capability)
	 * @return the required Capability object, <i>null</i> if no Required
	 *         Capability with such URI exists.
	 */
	public Capability getRequiredCapability(String capabilityURI) {
		return this.required_Capabilities.get(capabilityURI);
	}

	/* ********************* */
	/* * Getter and Setter * */
	/* ********************* */

	/**
	 * @return the localName
	 */
	public String getLocalName() {
		return localName;
	}

	public String getQualifiedName() {
		return getBase() + "#" + getLocalName();
	}

	public void setQualifiedName(String qname) {
		String[] split = qname.split("#");
		assert split.length == 2 : "Unable to split qualified name: " + qname;

		setBase(split[0]);
		setLocalName(split[1]);
	}

	/**
	 * @param localName
	 *            the localName to set
	 */
	public void setLocalName(String localName) {
		this.localName = localName;
	}

	/**
	 * @return the owlDescriptionFile
	 */
	public String getOwlDescriptionFile() {
		return owlDescriptionFile;
	}

	/**
	 * @param owlDescriptionFile
	 *            the owlDescriptionFile to set
	 */
	public void setOwlDescriptionFile(String owlDescriptionFile) {
		this.owlDescriptionFile = owlDescriptionFile;
	}
	
	

	/**
	 * @return the semanticType
	 */
	public SemanticType getSemanticType() {
		return semanticType;
	}

	/**
	 * @param semanticType the semanticType to set
	 */
	public void setSemanticType(SemanticType semanticType) {
		this.semanticType = semanticType;
	}

	/**
	 * 
	 * @return the type of the IO
	 */
	public String getType() {
		return this.getSemanticType().getOntology() + "#" + this.getSemanticType().getTypeName();
	}

	/**
	 * Set a new rdtType value
	 * 
	 * @param newType :
	 *            the new type to set
	 */
	public void setType(String newType) {
		this.getSemanticType().setType(newType);
	}


	/**
	 * @param _provided_Capabilities
	 *            the provided_Capabilities to set
	 */
	public void setProvidedCapabilities(List<Capability> _provided_Capabilities) {
		for (Iterator<Capability> it = _provided_Capabilities.iterator(); it.hasNext();) {
			this.addProvidedCapability(it.next());
		}
	}

	/**
	 * @return the provided_Capabilities
	 */
	public List<Capability> getProvidedCapabilities() {
		return new ArrayList<Capability>(this.provided_Capabilities.values());
	}

	/**
	 * @param _required_Capabilities
	 *            the required_Capabilities to set
	 */
	public void setRequiredCapabilities(List<Capability> _required_Capabilities) {
		for (Iterator<Capability> it = _required_Capabilities.iterator(); it.hasNext();) {
			this.addRequiredCapability(it.next());
		}
	}

	/**
	 * @return the required_Capabilities
	 */
	public List<Capability> getRequiredCapabilities() {
		return new ArrayList<Capability>(this.required_Capabilities.values());
	}

	/**
	 * @param _capabilities
	 *            the capabilities to set
	 */
	public void setCapabilities(List<Capability> _capabilities) {
		for (Iterator<Capability> it = _capabilities.iterator(); it.hasNext();) {
			this.addCapability(it.next());
		}

	}

	/**
	 * @return the capabilities
	 */
	public List<Capability> getCapabilities() {
		return new ArrayList<Capability>(this.capabilities.values());
	}

	/**
	 * @return the capabilities
	 */
	public Map<String, Capability> getCapabilitiesMap() {
		return capabilities;
	}

	/**
	 * @param capabilities
	 *            the capabilities to set
	 */
	public void setCapabilitiesMap(Map<String, Capability> capabilities) {
		this.capabilities = capabilities;
	}

	/**
	 * @return the isUserTask
	 */
	public boolean isUserTask() {
		return isUserTask;
	}

	public boolean isComposedService() {
		return isComposedService;
	}

	/**
	 * @param isUserTask
	 *            the isUserTask to set
	 */
	public void setUserTask(boolean isUserTask) {
		this.isUserTask = isUserTask;
	}

	/**
	 * @return the provided_Capabilities
	 */
	public Map<String, Capability> getProvided_Capabilities() {
		return provided_Capabilities;
	}

	/**
	 * @param provided_Capabilities
	 *            the provided_Capabilities to set
	 */
	public void setProvided_Capabilities(
			Map<String, Capability> provided_Capabilities) {
		this.provided_Capabilities = provided_Capabilities;
	}

	/**
	 * @return the required_Capabilities
	 */
	public Map<String, Capability> getRequired_Capabilities() {
		return required_Capabilities;
	}

	/**
	 * @param required_Capabilities
	 *            the required_Capabilities to set
	 */
	public void setRequired_Capabilities(
			Map<String, Capability> required_Capabilities) {
		this.required_Capabilities = required_Capabilities;
	}

	/**
	 * @return the binded_variable
	 */
	public Map<String, ParameterBinding> getBinded_variable() {
		return binded_variable;
	}

	/**
	 * @param binded_variable
	 *            the binded_variable to set
	 */
	public void setBinded_variable(Map<String, ParameterBinding> binded_variable) {
		this.binded_variable = binded_variable;
	}

	/**
	 * @return the base
	 */
	public String getBase() {
		return base;
	}

	/**
	 * @param base
	 *            the base to set
	 */
	public void setBase(String base) {
		this.base = base;
	}

	/**
	 * @return the wsdlDescriptionFile
	 */
	public String getWsdlDescriptionFile() {
		return wsdlDescriptionFile;
	}

	/**
	 * @param wsdlDescriptionFile
	 *            the wsdlDescriptionFile to set
	 */
	public void setWsdlDescriptionFile(String wsdlDescriptionFile) {
		this.wsdlDescriptionFile = wsdlDescriptionFile;
	}

	/**
	 * @return the qos
	 */
	public QoSProperty getQos() {
		return qos;
	}

	/**
	 * @param qos
	 *            the qos to set
	 */
	public void setQos(QoSProperty qos) {
		this.qos = qos;
	}
	
	/**
	 * @return the wsdlContent
	 */
	public String getWsdlContent() {
		return wsdlContent;
	}

	/**
	 * @param wsdlContent the wsdlContent to set
	 */
	public void setWsdlContent(String wsdlContent) {
		this.wsdlContent = wsdlContent;
	}
	
	/**
	 * @param automaton the automaton to set
	 */
	public void setAutomaton(Automaton automaton) {
		this.automaton = automaton;
	}

	/**
	 * @param capabilities
	 *            the capabilities to set
	 */
	public void setCapabilities(Map<String, Capability> capabilities) {
		this.capabilities = capabilities;
	}

	

	/* ********************* */
	/* * Capability search * */
	/* ********************* */
	
	
	/**
	 * Get the provided Capability from the specified URI
	 * 
	 * @param capabilityURI
	 *            the URI of the provided Capability
	 * @return the Capability object if a capability with this URI is contained
	 *         into the list of provided capability, null otherwise
	 */
	public Capability getProvidedCapability(String capabilityURI) {
		return this.provided_Capabilities.get(capabilityURI);
	}

	/**
	 * Get a Capability from the specified URI
	 * 
	 * @param capabilityURI
	 *            the URI of the Capability
	 * @return the Capability object if a capability with this URI is contained
	 *         into the list of capability, null otherwise
	 */
	public Capability getCapability(String capabilityURI) {
		return this.capabilities.get(capabilityURI);
	}

	/**
	 * Get a Capability from the specified RDFType/Name. This method search into
	 * the map of capabilities (provided and other) and return the list of
	 * results found. The order to search is provided then other one
	 * (subcapabilities).
	 * 
	 * @param capabilityTypeName
	 *            the Type of the Capability
	 * @return a List of Capability objects if one or many capabilities with
	 *         this typeName is contained into a list of capability, empty list
	 *         otherwise
	 * 
	 */
	public List<Capability> getCapabilityFromTypeName(String capabilityTypeName) {
		List<Capability> result = new ArrayList<Capability>();

		/*
		 * DebugPrinter.print(new ArrayList<String>(this.provided_Capabilities.keySet()),"provided",
		 * 1); DebugPrinter.print(new ArrayList<String>(this.required_Capabilities.keySet()),"required",
		 * 1); DebugPrinter.print(new ArrayList<String>(this.capabilities.keySet()),"other",
		 * 1);
		 */

		// search into provided capabilities
		for (Iterator<Capability> it = this.provided_Capabilities.values().iterator(); it.hasNext();) {
			Capability tmp = it.next();
			if (tmp.getTypeName().equals(capabilityTypeName)) {
				result.add(tmp);
			}
		}

		// search into other capabilities
		for (Iterator<Capability> it = this.capabilities.values().iterator(); it.hasNext();) {
			Capability tmp = it.next();
			if (tmp.getTypeName().equals(capabilityTypeName)) {
				result.add(tmp);
			}
		}

		return result;
	}

	/**
	 * Get a Capability from the specified local name. This method search into
	 * the map of capabilities (provided and other) and return the list of
	 * results found. The order to search is provided, then required then other
	 * one (subcapabilities).
	 * 
	 * @param capabilityLocalName
	 *            the local name of the Capability
	 * @return the Capability object if with this localName is contained into a
	 *         list of capability, empty list otherwise
	 * 
	 */
	public Capability getCapabilityFromLocalName(String capabilityLocalName) {
		Capability result = null;

		/*
		 * DebugPrinter.print(new ArrayList<String>(this.provided_Capabilities.keySet()),"provided",
		 * 1); DebugPrinter.print(new ArrayList<String>(this.required_Capabilities.keySet()),"required",
		 * 1); DebugPrinter.print(new ArrayList<String>(this.capabilities.keySet()),"other",
		 * 1);
		 */

		// search into provided capabilities
		for (Iterator<Capability> it = this.provided_Capabilities.values().iterator(); it.hasNext()
				&& result == null;) {
			Capability tmp = it.next();
			if (tmp.getLocalName().equals(capabilityLocalName)) {
				result = tmp;
			}
		}

		// search into required capabilities
		/*
		 * for (Iterator<Capability> it = this.required_Capabilities.values()
		 * .iterator(); it.hasNext() && result == null;) { Capability tmp =
		 * it.next(); if (tmp.getLocalName().equals(capabilityLocalName)) {
		 * result = tmp; } }
		 */

		// search into other capabilities
		for (Iterator<Capability> it = this.capabilities.values().iterator(); it.hasNext()
				&& result == null;) {
			Capability tmp = it.next();
			if (tmp.getLocalName().equals(capabilityLocalName)) {
				result = tmp;
			}
		}

		return result;
	}

	/**
	 * Get a Capability from the specified URI. This method search into the map
	 * of capabilities (provided and other) and return the first result found.
	 * The order to search is provided then other
	 * 
	 * @param capabilityURI
	 *            the URI of the provided Capability
	 * @return the Capability object if a capability with this Name is contained
	 *         into a list of capability, null otherwise
	 * 
	 */
	public Capability getCapabilityFromURI(String capabilityURI) {
		Capability result = null;

		// search into provided capabilities
		result = this.getProvided_Capabilities().get(capabilityURI);

		// search into required capabilities
		/*
		 * if (result == null) { result =
		 * this.getRequired_Capabilities().get(capabilityURI); }
		 */

		// search into other capabilities
		if (result == null) {
			result = this.getCapabilitiesMap().get(capabilityURI);
		}

		return result;
	}

	/**
	 * Search the first atomic capability that use the given IO. The search is
	 * recursive and start with the Capablity which have the given localName
	 * 
	 * @param io
	 *            the IO we search
	 * @param currentCapabilityTypeName :
	 *            the current capability typeName
	 * @return <i>null</i> if no Capability use this IO, the first Capability
	 *         in the process using it otherwise
	 */
	public Capability findFirstCapabilityUsing(IO io,
			String currentCapabilityTypeName) {
		List<Capability> entryPoint_list = this.getCapabilityFromTypeName(currentCapabilityTypeName);
		if (entryPoint_list.isEmpty())
			return null;
		return entryPoint_list.get(0).getFirstCapaUsingIO(io);
	}

	/**
	 * Search the first atomic capability that use the given IO. The search is
	 * recursive and start with the Capablity which have the given typeName
	 * 
	 * @param io
	 *            the IO we search
	 * @param currentCapabilityTypeName :
	 *            the current capability typeName
	 * @return <i>null</i> if no Capability use this IO, the first Capability
	 *         in the process u sing it otherwise
	 */
	public Capability findLastCapabilityUsing(IO io,
			String currentCapabilityTypeName) {
		List<Capability> entryPoint_list = this.getCapabilityFromTypeName(currentCapabilityTypeName);

		/*
		 * DebugPrinter.print("currentCapabilityTypeName =
		 * "+currentCapabilityTypeName,DebugPrinter.INFO);
		 * DebugPrinter.print(entryPoint_list,"entryPoint_list",
		 * DebugPrinter.INFO); DebugPrinter.print("service localname =
		 * "+this.getLocalName(), DebugPrinter.INFO);
		 * DebugPrinter.print("searched IO = "+io, DebugPrinter.INFO);
		 */

		if (entryPoint_list.isEmpty())
			return null;

		Capability entryPoint = entryPoint_list.get(0);
		Capability lastUsed = null;

		// creation and initialisation of the visit queue
		Queue<Capability> toVisit = new LinkedList<Capability>();
		toVisit.add(entryPoint);

		// visit each capability
		while (toVisit.size() > 0) {
			Capability current = toVisit.poll();
			if (current.isAtomic()) {
				// this is the last capability that used the io
				if (current.isUsing(io)) {
					lastUsed = current;
				}
			} else {
				// we add all subcapabilities to the list of capability to visit
				toVisit.addAll(current.getSubCapabilities());
			}

		}

		return lastUsed;
	}

	/**
	 * Get the Capability which has the given ConversationName
	 * 
	 * @param conversationName :
	 *            the localName of the conversation of the capability
	 * @return the Capability whit the given Conversation localName, null
	 *         otherwise
	 */
	public Capability getCapabilityFromConversationName(String conversationName) {
		Capability result = null;

		// search into required capabilities
		for (Iterator<Capability> it = this.required_Capabilities.values().iterator(); it.hasNext()
				&& result == null;) {
			Capability tmp = it.next();
			if (tmp.getConversationURI().equals(conversationName)) {
				result = tmp;
			}
		}

		// search into provided capabilities
		/*
		 * for (Iterator<Capability> it = this.provided_Capabilities.values()
		 * .iterator(); it.hasNext() && result == null;) { Capability tmp =
		 * it.next(); if (tmp.getConversationURI().equals(conversationName)) {
		 * result = tmp; } }
		 */

		// search into other capabilities
		for (Iterator<Capability> it = this.capabilities.values().iterator(); it.hasNext()
				&& result == null;) {
			Capability tmp = it.next();
			if (tmp.getConversationURI().equals(conversationName)) {
				result = tmp;
			}
		}

		return result;
	}

	/**
	 * Get a full matching between the task and the current service
	 * 
	 * @param userTask :
	 *            the task to match with
	 * @return a ServiceMatch object containing the matching information; if
	 *         some capabilities could be matched, <i>null</i> if no matching
	 *         is possible or if the matching is not complete
	 * @throws SemanticTypeNotKnowException
	 *             if a type is not knowed
	 */
	public ServiceMatch match(UserTask userTask, Matcher matcher)
			throws SemanticTypeNotKnowException {
		return matcher.serviceMatch(this, userTask);

	}

	/**
	 * Get a list of all Capability object contained into the service object
	 * (concat list of required, provided and other services)
	 * 
	 * @return the list of all Capability, an <i>empty list</i> if the service
	 *         contains no Capability
	 */
	public List<Capability> getAllCapabilities() {
		List<Capability> result = new ArrayList<Capability>();

		result.addAll(this.getProvidedCapabilities());
		result.addAll(this.getRequiredCapabilities());
		result.addAll(this.getCapabilities());

		return result;
	}

	/**
	 * @return the automaton
	 */
	public Automaton getAutomaton() {
		/*if (this.automaton == null)
			this.automaton = ;*/
		return new Automaton(this);
	}

	

	public ProcessElements getProcessElements(String capability_uri) {
		// TODO Auto-generated method stub
		return null;
	}

	public String getServiceName() {
		return this.getLocalName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {

		this.base = null;
		this.qos = null;
		this.localName = null;
		this.owlDescriptionFile = null;
		this.semanticType = null;
		this.wsdlDescriptionFile = null;

		this.binded_variable.clear();
		this.binded_variable = null;

		this.capabilities.clear();
		this.capabilities = null;		

		this.provided_Capabilities.clear();
		this.provided_Capabilities = null;
		
		this.required_Capabilities.clear();
		this.required_Capabilities = null;		
	}
}
