/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * This class is an extension of Service to specifically describe the task send by a user.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class UserTask extends Service {

	/**
	 * SerialVersion of the class
	 */
	private static final long serialVersionUID = 1L;

	public UserTask(Service service) {
		this.localName = service.getLocalName();
		this.semanticType = service.getSemanticType();
		this.base = service.getBase();
		this.setCapabilities(service.getCapabilities());
		this.setRequiredCapabilities(service.getRequiredCapabilities());
		this.isUserTask = true;
		this.owlDescriptionFile = service.getOwlDescriptionFile();
	}

	/**
	 * Get the Capability which has the given ConversationName
	 * 
	 * @param conversationName :
	 *            the localName of the conversation of the capability
	 * @return the Capability whit the given Conversation localName, null
	 *         otherwise
	 */
	@Override
	public Capability getCapabilityFromConversationName(String conversationName) {
		Capability result = null;

		// search into required capabilities
		for (Iterator<Capability> it = this.required_Capabilities.values().iterator(); it.hasNext()
				&& result == null;) {
			Capability tmp = it.next();
			if (tmp.getConversationURI().equals(conversationName)) {
				result = tmp;
			}
		}

		// search into other capabilities
		for (Iterator<Capability> it = this.capabilities.values().iterator(); it.hasNext()
				&& result == null;) {
			Capability tmp = it.next();
			if (tmp.getConversationURI().equals(conversationName)) {
				result = tmp;
			}
		}

		return result;
	}

	/**
	 * Get a Capability from the specified URI. This method search into the map
	 * of capabilities (provided and other) and return the first result found.
	 * The order to search is provided then other
	 * 
	 * @param capabilityURI
	 *            the URI of the provided Capability
	 * @return the Capability object if a capability with this Name is contained
	 *         into a list of capability, null otherwise
	 * 
	 */
	@Override
	public Capability getCapabilityFromURI(String capabilityURI) {
		Capability result = null;

		// search into required capabilities
		result = this.getRequired_Capabilities().get(capabilityURI);

		// search into other capabilities
		if (result == null) {
			result = this.getCapabilitiesMap().get(capabilityURI);
		}

		return result;
	}

	/**
	 * Get a Capability from the specified local name. This method search into
	 * the map of capabilities (provided and other) and return the list of
	 * results found. The order to search is provided, then required then other
	 * one (subcapabilities).
	 * 
	 * @param capabilityLocalName
	 *            the local name of the Capability
	 * @return the Capability object if with this localName is contained into a
	 *         list of capability, empty list otherwise
	 * 
	 */
	@Override
	public Capability getCapabilityFromLocalName(String capabilityLocalName) {
		Capability result = null;

		// search into required capabilities
		for (Iterator<Capability> it = this.required_Capabilities.values().iterator(); it.hasNext()
				&& result == null;) {
			Capability tmp = it.next();
			if (tmp.getLocalName().equals(capabilityLocalName)) {
				result = tmp;
			}
		}

		// search into other capabilities
		for (Iterator<Capability> it = this.capabilities.values().iterator(); it.hasNext()
				&& result == null;) {
			Capability tmp = it.next();
			if (tmp.getLocalName().equals(capabilityLocalName)) {
				result = tmp;
			}
		}

		return result;
	}

	/**
	 * Get a Capability from the specified RDFType/Name. This method search into
	 * the map of capabilities (provided and other) and return the list of
	 * results found. The order to search is provided then other one
	 * (subcapabilities).
	 * 
	 * @param capabilityTypeName
	 *            the Type of the Capability
	 * @return a List of Capability objects if one or many capabilities with
	 *         this typeName is contained into a list of capability, empty list
	 *         otherwise
	 * 
	 */
	@Override
	public List<Capability> getCapabilityFromTypeName(String capabilityTypeName) {
		List<Capability> result = new ArrayList<Capability>();

		/*
		 * DebugPrinter.print(new ArrayList<String>(this.provided_Capabilities.keySet()),"provided",
		 * 1); DebugPrinter.print(new ArrayList<String>(this.required_Capabilities.keySet()),"required",
		 * 1); DebugPrinter.print(new ArrayList<String>(this.capabilities.keySet()),"other",
		 * 1);
		 */

		// search into required capabilities
		for (Iterator<Capability> it = this.required_Capabilities.values().iterator(); it.hasNext();) {
			Capability tmp = it.next();
			if (tmp.getTypeName().equals(capabilityTypeName)) {
				result.add(tmp);
			}
		}

		// search into other capabilities
		for (Iterator<Capability> it = this.capabilities.values().iterator(); it.hasNext();) {
			Capability tmp = it.next();
			if (tmp.getTypeName().equals(capabilityTypeName)) {
				result.add(tmp);
			}
		}

		return result;
	}

}
