/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
package fr.inria.icocoa.servicemodel.automaton;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.servicemodel.Service;

/**
 * The class that reprensent the automaton associated with a Statefull service.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class Automaton implements Serializable {

	/**
	 * the version of the class (required for serialisation)
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The initial state of the automaton
	 */
	private InitialState initialState;

	/**
	 * The list of final states of the automaton
	 */
	private List<State> finalStatesList;

	/**
	 * The list of all states of the Automaton
	 */
	private List<State> statesList;

	/**
	 * The list of all symbol associated with the automaton
	 */
	private List<Symbol> symbolsList;

	/**
	 * this is a hashmap of 2 dimensions of Lists it is in the form (line
	 * S0(state) column a(symbol) contains states S1,S2 )
	 */
	private HashMap<State, HashMap<Symbol, List<State>>> transitions;

	/**
	 * this is also a double hash map of lists, which corresponds to the
	 * epsilon-closure of the transitions hashmap
	 */
	private HashMap<State, HashMap<Symbol, List<State>>> epsilonClosure;

	/**
	 * this is a hashmap of one dimension, where keys are states and values are
	 * lists of following symbols; an entry exists only if the states has
	 * non-empty following symbols
	 */
	private HashMap<State, List<Symbol>> followingSymbols;

	/**
	 * a hash map where objects of type Transition are stored and accessed using
	 * Strings representing the concatenation of the transition elements in the
	 * form "state1-symbol-state2"
	 */
	private HashMap<String, Transition> transitionsProperties;//

	// QoS Disabling: private List QoSAttributes;
	// private HashMap QoSRequirements;

	/**
	 * The Stack of state use for some algorithm
	 */
	private List<State> pile = new ArrayList<State>();

	// private List pileDesSuivants = new ArrayList();

	// QoS Disabling: private QoSFormulaTree formulaTree;

	/**
	 * the epsilon-closure of the TransitionProperties structure
	 */
	private HashMap<String, Transition> transitionPropertiesClosure;

	/**
	 * The static Epsilon symbol use for algorithm on automaton
	 */
	public static final Symbol EPSILON = new Symbol("epsilon");/*
	 * voire si on
	 * ne peut pas
	 * travailler
	 * avec ce
	 * epsilon
	 */

	/**
	 * Default constructor for automaton
	 */
	public Automaton() {
		this.statesList = new ArrayList<State>();
		this.finalStatesList = new ArrayList<State>();
		this.symbolsList = new ArrayList<Symbol>();
		this.transitions = new HashMap<State, HashMap<Symbol, List<State>>>();
		this.epsilonClosure = new HashMap<State, HashMap<Symbol, List<State>>>();
		this.followingSymbols = new HashMap<State, List<Symbol>>();
		this.transitionsProperties = new HashMap<String, Transition>();
		this.transitionPropertiesClosure = new HashMap<String, Transition>();

	}

	/**
	 * Create the automaton associated to a Service
	 * 
	 * @param s
	 *            the service we required the automaton
	 */
	public Automaton(Service s) {
		finalStatesList = new ArrayList<State>();
		statesList = new ArrayList<State>();
		symbolsList = new ArrayList<Symbol>();
		transitions = new HashMap<State, HashMap<Symbol, List<State>>>();
		followingSymbols = new HashMap<State, List<Symbol>>();
		epsilonClosure = new HashMap<State, HashMap<Symbol, List<State>>>();
		transitionsProperties = new HashMap<String, Transition>();
		// QoS Disabling: QoSRequirements = new HashMap();

		DebugPrinter.print("automaton::<init>", DebugPrinter.DEBUG);

		Automaton resultingAutomaton = new ServiceModel2Automata()
				.OpenAllBoxes(s);
		// MainPackage.PrintMethods.printTabTrans(resultingAutomaton.getTransitionTable());
		this.statesList = resultingAutomaton.statesList;
		this.symbolsList = resultingAutomaton.symbolsList;
		this.initialState = resultingAutomaton.initialState;
		this.finalStatesList = resultingAutomaton.finalStatesList;
		this.transitions = resultingAutomaton.transitions;

		/* Do the epsilon closure */
		for (Iterator e = this.statesList.iterator(); e.hasNext();) {
			State CurrentState = (State) e.next();
			this.fermeture(CurrentState);
		}
		// MainPackage.PrintMethods.printTabTrans(this.Transitions);
		/* Fill the following symbols */
		for (Iterator e = this.statesList.iterator(); e.hasNext();) {
			State obj = (State) e.next();
			List<Symbol> suiv = this.getSuivant(obj);
			this.followingSymbols.put(obj, suiv);
		}
		// MainPackage.PrintMethods.printTabTrans(this.EpsilonClosure);
		// MainPackage.PrintMethods.printTabSuiv(this);
		// -----------------------------------------------
		this.removeInaccessibleStates();
		// MainPackage.PrintMethods.printTabTrans(this.EpsilonClosure);
		// --------------------Fill the transitionPropeties map
		this.fillTransionProperties();
		// ---------------Do the transitionPropertiesClosure
		this.transitionPropertiesClosure = new HashMap<String, Transition>();
		// DebugPrinter.print(this, "Automaton", 1);
		this.transitionPropertiesClosure = this.transitionClosure();
		this.updateFinalStatesAfterClosure();
		// MainPackage.PrintMethods.printTransitionProperties(this.TransitionPropertiesClosure);
		/** ********************************************** */
		// TODO:completer cette partie........
		// QoS Disabling: QoSFormulaTree myFormulaTree = new
		// QoSFormulaTree(this,QoSRequirements);
		// QoS Disabling: this.setFormulaTree(myFormulaTree);
		/** ********************************************** */

	}

	/**
	 * Create the aggregated automaton of a list of services
	 * 
	 * @param serviceList
	 *            the list of service we want the global automaton
	 */
	public Automaton(List<Service> serviceList) {
		finalStatesList = new ArrayList<State>();
		statesList = new ArrayList<State>();
		symbolsList = new ArrayList<Symbol>();
		transitions = new HashMap<State, HashMap<Symbol, List<State>>>();
		followingSymbols = new HashMap<State, List<Symbol>>();
		epsilonClosure = new HashMap<State, HashMap<Symbol, List<State>>>();
		transitionsProperties = new HashMap<String, Transition>();
		Automaton resultingAutomaton = AutomataUnion
				.connectServices(serviceList);
		this.statesList = resultingAutomaton.statesList;
		this.symbolsList = resultingAutomaton.symbolsList;
		this.initialState = resultingAutomaton.initialState;
		this.finalStatesList = resultingAutomaton.finalStatesList;
		this.transitions = resultingAutomaton.transitions;
		/* Do the epsilon closure */
		for (Iterator e = this.statesList.iterator(); e.hasNext();) {
			State CurrentState = (State) e.next();
			this.fermeture(CurrentState);
		}
		// MainPackage.PrintMethods.printTabTrans(this.Transitions);
		/* Fill the following symbols */
		for (Iterator e = this.statesList.iterator(); e.hasNext();) {
			State obj = (State) e.next();
			List<Symbol> suiv = this.getSuivant(obj);
			this.followingSymbols.put(obj, suiv);
		}
		// MainPackage.PrintMethods.printTabTrans(this.EpsilonClosure);
		// MainPackage.PrintMethods.printTabSuiv(this);
		// -----------------------------------------------
		this.removeInaccessibleStates();
		// --------------------Fill the transitionPropeties map
		this.fillTransionProperties();
		// ---------------Do the transitionPropertiesClosure
		this.transitionPropertiesClosure = new HashMap<String, Transition>();
		this.transitionPropertiesClosure = this.transitionClosure();
		this.updateFinalStatesAfterClosure();
	}

	/**
	 * Fill the bidimentionnal table that correspond to the transition between State.
	 *
	 */
	private void fillTransionProperties() {
		// TODO:adapt this method in order to take into account the QoS
		// parameters
		// from the services descriptions...
		for (Iterator it = this.transitions.keySet().iterator(); it.hasNext();) {
			State currentState = (State) it.next();
			for (Iterator it2 = ((HashMap) this.transitions.get(currentState))
					.keySet().iterator(); it2.hasNext();) {
				Symbol currentSymbol = (Symbol) it2.next();
				List currentArrivaList = (List) DoubleHashMap.getValue(
						this.transitions, currentState, currentSymbol);
				if (currentArrivaList != null) {
					for (Iterator it3 = currentArrivaList.iterator(); it3
							.hasNext();) {
						State currentArrivalState = (State) it3.next();
						String transitionKey = currentState.getName()
								+ currentSymbol.getName()
								+ currentArrivalState.getName();
						Transition transition = new Transition(currentState,
								currentSymbol, currentArrivalState);
						/*
						 * TODO:fill the QoS attributes from the AMIGO-S Service
						 * description transition.setQoSQttributes()
						 */
						this.transitionsProperties.put(transitionKey,
								transition);
					}
				}
			}
		}
	}

	// -------------------------Public
	// Methods--------------------------------------
	/**
	 * @return the initialState
	 */
	public InitialState getInitialState() {
		return (this.initialState);
	}

	/**
	 * @return the statesList
	 */
	public List<State> getStatesList() {
		return statesList;
	}

	/**
	 * @return the symbolsList
	 */
	public List<Symbol> getSymbolsList() {
		return symbolsList;
	}

	/**
	 * @return the symbolsList
	 */
	public List<Symbol> getSymbols() {
		return this.getSymbolsList();
	}

	/**
	 * @return the transitions
	 */
	public HashMap<State, HashMap<Symbol, List<State>>> getTransition() {
		return transitions;
	}

	/**
	 * @return the transitionsProperties
	 */
	public HashMap<String, Transition> getTransitionsProperties() {
		return transitionsProperties;
	}

	/**
	 * @return the transitions
	 */
	public HashMap<State, HashMap<Symbol, List<State>>> getTransitionTable() {
		return this.getTransition();
	}

	/**
	 * @return the statesList
	 */
	public List<State> getStates() {
		return this.getStatesList();
	}

	/**
	 * @return the finalStatesList
	 */
	public List<State> getFinalStatesList() {
		return finalStatesList;
	}

	/**
	 * @return the finalStatesList
	 */
	public List<State> getFinalStates() {
		return this.getFinalStatesList();
	}

	/*
	 * QoS Disabling: public QoSFormulaTree getFormulaTree() { return
	 * formulaTree; }
	 */

	// public HashMap getTransitionPropertiesClosure() {
	// return transitionPropertiesClosure;
	// }
	/*
	 * public List getQoSAttributes() { return QoSAttributes; }
	 */
	/*
	 * public HashMap getQoSRequirements() { return QoSRequirements; }
	 */

	// ----------------------------------------------------------------------------

	/**
	 * Check if the automaton contains a state with a given name
	 * @param s the name of the required state
	 * @return <i>true</i> if the automaton contains a state with such name, <i>false</i> otherwise
	 */
	public boolean containsAStateNamed(String s) {
		for (Iterator e = this.statesList.iterator(); e.hasNext();) {
			State state = (State) e.next();
			if (state.getName().equals(s)) {
				return (true);
			}
		}
		return (false);
	}

	/**
	 * Check if the automaton contains a symbol with a given name
	 * @param s the name of the required symbol
	 * @return <i>true</i> if the automaton contains a symbol with such name, <i>false</i> otherwise
	 */
	public boolean containsASymbolNamed(String s) {
		for (Iterator e = this.symbolsList.iterator(); e.hasNext();) {
			Symbol symbol = (Symbol) e.next();
			if (symbol.getName().equals(s)) {
				return (true);
			}
		}
		return (false);
	}

	/**
	 * Get the list symbol that "go out" the given state
	 * @param Etat the state we required the outgoing symbols
	 * @return the list of out-going symbol for the given state 
	 */
	private List<Symbol> getSuivant(State Etat) {
		List<Symbol> suivant;
		HashMap<Symbol, List<State>> symbols = this.epsilonClosure.get(Etat);
		if (symbols != null) {
			suivant = new ArrayList<Symbol>(symbols.keySet());
		} else {
			suivant = null;
		}
		return (suivant);
	}

	/**
	 * Get a state object from its name
	 * @param stateName the name of the required state
	 * @return the state with the given name, <b>null</b> if the automaton contains no state with such name
	 */
	public State getStateNamed(String stateName) {
		for (Iterator e = this.statesList.iterator(); e.hasNext();) {
			State actualState = (State) e.next();
			if (actualState.getName().equals(stateName)) {
				return (actualState);
			}
		}
		return null;
	}

	/**
	 * Get a symbol object from its name
	 * @param symbolName the name of the required symbol
	 * @return the symbol with the given name, <b>null</b> if the automaton contains no symbol with such name
	 */
	public Symbol getSymbolNamed(String symbolName) {
		for (Iterator e = this.symbolsList.iterator(); e.hasNext();) {
			Symbol actualSymbol = (Symbol) e.next();
			if (actualSymbol.getName().equals(symbolName)) {
				return (actualSymbol);
			}
		}
		return (null);

	}

	/**
	 * This method calculate the closure of a Given state
	 * @param Etat the state object we want to compute the closure
	 */
	@SuppressWarnings("unchecked")
	public void fermeture(State Etat) {
		// TODO: rendre final certains tats apres fermeture si il n'y a que des
		// transitions vides qui les spare d'un etat final....
		if (pile.contains(Etat)) {
			return;
		}
		// System.out.println("fermeture de : "+Etat.getName());
		pile.add(Etat);
		/*
		 * System.out.println("-------------------");
		 * MainPackage.PrintMethods.printStatesList(pile);
		 */
		if (this.epsilonClosure.get(Etat) == null) {
			HashMap<Symbol, List<State>> ferm = new HashMap<Symbol, List<State>>();
			if (this.transitions.get(Etat) != null) {
				Set<Symbol> outGoingSymbols = (this.transitions.get(Etat))
						.keySet();
				if (outGoingSymbols.contains(Automaton.EPSILON)) {
					// si il n'y a pas du tout de epsilon dans les symbols
					// recopier les transitions...
					for (Iterator<Symbol> e2 = this.symbolsList.iterator(); e2
							.hasNext();) {
						Symbol currentTerminal = e2.next();
						if (!currentTerminal.equals(Automaton.EPSILON)) {
							List<State> obj = (List) DoubleHashMap.getValue(
									this.transitions, Etat, currentTerminal);
							if (obj != null) {
								ferm.put(currentTerminal, obj);
							}/*
							 * else{ System.out.println("getValue null pour
							 * "+Etat.getName()+"
							 * "+currentTerminal.getName()); }
							 */
						}
					}
					/*
					 * System.out.println(Etat.getName()+"--------------");
					 * MainPackage.PrintMethods.printMapOfList(ferm,
					 * this.SymbolsList);
					 */
					/*
					 * System.out.println(Etat.getName()+"--------------");
					 * printStatesList((List)ferm.get(getSymbolNamed("b")));
					 */
					List<State> T_Etat_epsilon = (List) DoubleHashMap.getValue(
							this.transitions, Etat, Automaton.EPSILON);
					if (T_Etat_epsilon != null) {
						for (Iterator<State> e1 = T_Etat_epsilon.iterator(); e1
								.hasNext();) {
							State CurrentState = e1.next();
							if (!CurrentState.equals(Etat)) {
								// /------------------------------------
								// HashMap toAdd =fermeture(CurrentState);
								fermeture(CurrentState);
								HashMap<Symbol, List<State>> toAdd = this.epsilonClosure
										.get(CurrentState);
								if (toAdd == null) {
									return;
								}
								// this.EpsilonClosure.put(CurrentState,toAdd);
								// -------------------------------------
								for (Iterator<Symbol> e2 = toAdd.keySet()
										.iterator(); e2.hasNext();) {
									Symbol CurrentTerminal = e2.next();
									if (!CurrentTerminal
											.equals(Automaton.EPSILON)) {
										List<State> tempv = toAdd
												.get(CurrentTerminal);
										List<State> aremplir = ferm
												.get(CurrentTerminal);
										if (tempv != null) {
											if (aremplir == null) {
												aremplir = new ArrayList<State>();
											}
											for (Iterator e3 = tempv.iterator(); e3
													.hasNext();) {
												State element = (State) e3
														.next();
												if (!aremplir.contains(element)) {
													aremplir.add(element);
												}
											}
										}
										if (aremplir != null) {
											/*
											 * System.out.println("aremplir pour
											 * "+Etat.getName()+"
											 * "+CurrentTerminal.getName());
											 * MainPackage.PrintMethods.printStatesList(aremplir);
											 */
											ferm.put(CurrentTerminal, aremplir);
										}
									}
								}
							}
						}
					}
				} else {
					// System.out.println(Etat.getName()+" pas de epsilon");
					ferm = this.transitions.get(Etat);
				}
			}
			/*
			 * System.out.println(Etat.getName() + "--------------");
			 * MainPackage.PrintMethods.printMapOfList(ferm, this.SymbolsList);
			 */
			// ////------------------------------------
			this.epsilonClosure.put(Etat, ferm);
			// return (ferm);
			// ////------------------------------------
		} // else
		// ////------------------------------------
		// this.EpsilonClosure.put(Etat,(HashMap)this.EpsilonClosure.get(Etat));
		// return((HashMap)this.EpsilonClosure.get(Etat));
		// ////------------------------------------
	}

	/**
	 * Check all states and remove inaccessible
	 *
	 */
	@SuppressWarnings("unchecked")
	private void removeInaccessibleStates() {
		/*
		 * First Step : extract in the pileDesAccessibles list the list of all
		 * accessible states in the epsilonClosure map
		 */
		State currentState = null;
		List<State> pileDesAccessibles = new ArrayList<State>();
		List<State> alreadyTested = new ArrayList<State>();
		List<State> statesToTest = new ArrayList<State>();
		statesToTest.add(this.initialState);
		while (statesToTest.size() > 0) {
			currentState = statesToTest.remove(0);
			alreadyTested.add(currentState);
			pileDesAccessibles.add(currentState);
			HashMap<Symbol, List<State>> currentMap = this.epsilonClosure
					.get(currentState);
			if (currentMap != null) {
				for (Iterator<Symbol> it = currentMap.keySet().iterator(); it
						.hasNext();) {
					Symbol currentSymbol = it.next();
					List<State> currentArrivalList = (List) DoubleHashMap
							.getValue(this.epsilonClosure, currentState,
									currentSymbol);
					if (currentArrivalList != null
							&& currentArrivalList.size() > 0) {
						for (Iterator it2 = currentArrivalList.iterator(); it2
								.hasNext();) {
							State currentAccessibleState = (State) it2.next();
							if (!alreadyTested.contains(currentAccessibleState)) {
								statesToTest.add(currentAccessibleState);
							}
						}
					}
				}
			}
		}
		/*
		 * System.out.println("Pile des accessibles : ");
		 * MainPackage.PrintMethods.printStatesList(pileDesAccessibles);
		 */
		/*
		 * Second Step : remove the transitions that does not start by an
		 * accessible state
		 */
		for (Iterator<State> it3 = statesList.iterator(); it3.hasNext();) {
			State currentStateToCheck = it3.next();
			if (!pileDesAccessibles.contains(currentStateToCheck)) {
				this.epsilonClosure.remove(currentStateToCheck);
			}
		}
		/*
		 * System.out.println("Fermeture aprs modif : ");
		 * MainPackage.PrintMethods.printTabTrans(this.EpsilonClosure);
		 */

	}

	/**
	 * Get the closure of transition map
	 * @return a new map of transition
	 */
	@SuppressWarnings("unchecked")
	private HashMap<String, Transition> transitionClosure() {
		HashMap<String, Transition> closure = new HashMap<String, Transition>();
		for (Iterator<State> it = this.epsilonClosure.keySet().iterator(); it
				.hasNext();) {
			State currentState = it.next();
			// System.out.println("automaton::transitionClosure
			// counter="+counter);
			// PrintMethods.printTabTrans(this.EpsilonClosure);
			/*
			 * if((HashMap)this.EpsilonClosure.get(currentState)==null){
			 * PrintMethods.printTabTrans(this.EpsilonClosure);
			 * 
			 * System.out.println("automaton::transitionClosure::curent state=
			 * "+currentState.getName()); }
			 */
			for (Iterator<Symbol> it2 = (this.epsilonClosure.get(currentState))
					.keySet().iterator(); it2.hasNext();) {
				Symbol currentSymbol = it2.next();
				List<State> currentArrivalList = (List) DoubleHashMap.getValue(
						this.epsilonClosure, currentState, currentSymbol);
				if (currentArrivalList != null && currentArrivalList.size() > 0) {
					for (Iterator<State> it3 = currentArrivalList.iterator(); it3
							.hasNext();) {
						State currentArrivalState = it3.next();
						String currentKey = currentState.getName()
								+ currentSymbol.getName()
								+ currentArrivalState.getName();

						String accessKeyInTransitionProperties = getTransitionEndsWith(currentSymbol
								.getName()
								+ currentArrivalState.getName());
						Transition transition = this.getTransitionProperties()
								.get(accessKeyInTransitionProperties);

						closure.put(currentKey, transition);
					}

				}
			}
		}
		return (closure);
	}

	/*
	 * Attention !! dans cette methode, on considere que dans l'automate de
	 * l'env on ne peut pas avoir deux transitions du type s0-a-s1 et s2-a-s1
	 * (qui rejoignent le meme etat avec le meme terminal...)
	 */
	/**
	 * Get the transition name which name ends with the given string<br>
	 * <u>WARNING: </u> In this method, we consider that the global automaton cannot have two transitions as s0-a-s1 and s2-a-s1 that reach the same final state with the same last symbol
	 * @param s the end of the name of the transition
	 * @return the entiere name of the transition that ends with the given string, <b>null</b> if no transition has a name that ends with the given string
	 */
	private String getTransitionEndsWith(String s) {
		for (Iterator<String> it = this.getTransitionProperties().keySet()
				.iterator(); it.hasNext();) {
			String key = it.next();
			if (key.endsWith(s)) {
				return (key);
			}
		}
		return null;
	}

	// --------------------------------------------------------------------------

	/**
	 * Get the name of the transition between to state by fllowing a symbol
	 * @param state1Name the name of the started state
	 * @param state2Name the name of the ended state
	 * @param symbolName the name of the symbol to follow
	 * @return the completeName of the transition
	 */
	public static String getTransition(String state1Name, String symbolName,
			String state2Name) {
		return (state1Name + symbolName + state2Name);
	}

	public HashMap<String, Transition> getTransitionProperties() {
		return (this.transitionsProperties);
	}

	// -------------------------------------------------------------------------
	public HashMap<State, HashMap<Symbol, List<State>>> getEpsilonClosure() {
		return (this.epsilonClosure);
	}

	// -------------------------------------------------------------------------
	public HashMap<State, List<Symbol>> getFollowingSymbols() {
		return (this.followingSymbols);
	}

	// ---------------------------Setters----------------------------------------------
	public void setInitialState(InitialState initialState) {
		this.initialState = initialState;
	}

	/*
	 * QoS Disabling: public void setFormulaTree(QoSFormulaTree formulaTree) {
	 * this.formulaTree = formulaTree; }
	 */

	public void setTransitionPropertiesClosure(
			HashMap<String, Transition> TransitionPropertiesClosure) {
		this.transitionPropertiesClosure = TransitionPropertiesClosure;
	}

	/*
	 * public void setQoSAttributes(List QoSAttributes) { this.QoSAttributes =
	 * QoSAttributes; }
	 */
	/*
	 * public void setQoSRequirements(HashMap QoSReq) { this.QoSRequirements =
	 * QoSReq; }
	 */

	// -------------------------------------------------------------------------
	public void setFinalStatesList(List<State> finalStatesList) {
		this.finalStatesList = finalStatesList;
	}

	// -------------------------------------------------------------------------
	public void setEpsilonClosure(
			HashMap<State, HashMap<Symbol, List<State>>> epsilonClosure) {
		this.epsilonClosure = epsilonClosure;
	}

	// -------------------------------------------------------------------------
	public List<Symbol> getFollowingSymbols(State state) {
		return this.followingSymbols.get(state);
	}

	/**
	 * Get the list of states that out-go from the given one following the given symbol
	 * @param state the current state
	 * @param symbol the symbol we follows
	 * @return the list of states following the given state according to the given symbol
	 */
	@SuppressWarnings("unchecked")
	public List<State> getFollowingState(State state, Symbol symbol) {
		return (List<State>) DoubleHashMap.getValue(this.epsilonClosure, state,
				symbol);
	}

	/**
	 * Add a new transition in the table of transition
	 * @param previousState the state that start the transition
	 * @param symbol the symbol follows to go from starting state to ended state
	 * @param newState the state that ended the transition
	 */
	public void addTransition(State previousState, Symbol symbol, State newState) {
		DoubleHashMap.putValueAtList(this.transitions, newState, // value
				previousState, // state
				symbol); // symbol
	}

	/**
	 * Add a new state in the list of state
	 * @param state the state to add
	 */
	public void addState(State state) {
		if (!this.containsAStateNamed(state.getName()))
			this.statesList.add(state);
	}

	/**
	 * Add a new Symbol in the list of symbol. The Symbol object will be created according its name.
	 * @param symbolName the symbol (its name) to add
	 */
	public void addSymbol(String symbolName) {
		if (this.getSymbolNamed(symbolName) == null) {
			if (symbolName.equals("epsilon")
					&& !this.containsASymbolNamed(symbolName))
				this.symbolsList.add(new Symbol(symbolName));
			else if (symbolName.equals("epsilon"))
				this.symbolsList.add(EPSILON);
		}
	}

	/**
	 * Add a new state in the list of final state 
	 * @param finalState the new state to add
	 */
	public void addFinalState(State finalState) {
		if (!this.finalStatesList.contains(finalState))
			this.finalStatesList.add(finalState);
	}

	/**
	 * Integrate a table of transition of another automaton, to the table of the current automaton
	 * @param transitionTable the transition table to integrate
	 */
	@SuppressWarnings("unchecked")
	public void addTransitions(
			HashMap<State, HashMap<Symbol, List<State>>> transitionTable) {
		/*
		 * System.out.println("-------------"+this.getInitialState().getName());
		 * MainPackage.PrintMethods.printTabTrans(transitionTable);
		 * MainPackage.PrintMethods.printSymbols(this.SymbolsList);
		 */
		for (Iterator<State> statesIt = transitionTable.keySet().iterator(); statesIt
				.hasNext();) {
			State currentState = statesIt.next();
			if (!this.containsAStateNamed(currentState.getName())) {
				this.statesList.add(currentState);
			}
			HashMap<Symbol, List<State>> currentCell = transitionTable
					.get(currentState);
			for (Iterator<Symbol> symbolIt = currentCell.keySet().iterator(); symbolIt
					.hasNext();) {
				Symbol currentSymbol = symbolIt.next();
				List<State> currentArrivalList = (List) DoubleHashMap.getValue(
						transitionTable, currentState, currentSymbol);
				if (currentArrivalList != null && currentArrivalList.size() > 0) {
					if (!this.containsASymbolNamed(currentSymbol.getName())) {
						this.symbolsList.add(currentSymbol);
						// System.out.println("added symbol
						// "+currentSymbol.getName()+" in
						// "+this.getInitialState().getName());
					}
					if (DoubleHashMap.getValue(this.getTransitionTable(),
							currentState, currentSymbol) == null) {
						DoubleHashMap.putValue(this.getTransitionTable(),
								new ArrayList(), currentState, currentSymbol);
					}
					for (ListIterator<State> it = currentArrivalList
							.listIterator(); it.hasNext();) {
						State currentElement = it.next();
						if (!this.containsAStateNamed(currentElement.getName())) {
							this.statesList.add(currentElement);
						}
						// for(int i=0;i<currentArrivalList.size();i++){
						// State currentElement =
						// (State)currentArrivalList.get(i);
						DoubleHashMap.putValueAtList(this.getTransitionTable(),
								currentElement, currentState, currentSymbol);
					}
				}
			}
		}
		/*
		 * System.out.println("***********pendant");
		 * MainPackage.PrintMethods.printStatesList(this.StatesList);
		 */
		/*
		 * System.out.println("***********pendant");
		 * MainPackage.PrintMethods.printTabTrans(this.Transitions);
		 */

	}

	/**
	 * Remove a transition
	 * @param currentState the starting state of the transition
	 * @param currentSymbol the symbol to follows to go from starting state to ending state
	 * @param currentArrivalState the ending state of the transition
	 */
	public void removeTransition(State currentState, Symbol currentSymbol,
			State currentArrivalState) {
		List arrivalStatesList = (List) DoubleHashMap.getValue(
				this.transitions, currentState, currentSymbol);
		arrivalStatesList.remove(currentArrivalState);
		if (arrivalStatesList.size() == 0) {
			DoubleHashMap.putValue(this.transitions, null, currentState,
					currentSymbol);
			this.symbolsList.remove(currentSymbol);/*
			 * en principe je devrai
			 * tester si c'est la
			 * dernire transition qui
			 * utilise currentSymbol
			 * avant de l'enlever...
			 */
		}

	}

	/**
	 * Create a transition between state1 and state2 following symbol s
	 * @param state1 the starting state of the transition
	 * @param s the symbol to follow
	 * @param state2 the ending state of the transition
	 */
	@SuppressWarnings("unchecked")
	public void generateTransition(State state1, Symbol s, State state2) {
		if (((List<State>) DoubleHashMap.getValue(this.transitions, state1, s)) == null) {
			List<State> tempList = new ArrayList<State>();
			tempList.add(state2);
			DoubleHashMap.putValue(this.transitions, tempList, state1, s);
		} else {
			DoubleHashMap.putValueAtList(this.transitions, state2, state1, s);
		}
	}

	/**
	 * Add a list of final states in the list of final states associated to the current automaton
	 * @param finalStates the list a final states to integrate
	 */
	public void addFinalStatesList(List<State> finalStates) {
		for (Iterator<State> it = finalStates.iterator(); it.hasNext();) {
			if (!this.finalStatesList.contains(finalStates))
				this.finalStatesList.add(it.next());
		}

	}

	/**
	 * Create a copy of the list of final states
	 * @param finalStates the list of final states
	 * @param cpt the current object counter
	 */
	public void copyFinalStatesList(List<State> finalStates, int cpt) {
		for (Iterator<State> it = finalStates.iterator(); it.hasNext();) {
			State currentFinalState = it.next();
			State currentCopy = copyOfState(currentFinalState, cpt);
			currentCopy.setFinal(true);
			if (!this.finalStatesList.contains(currentCopy))
				this.finalStatesList.add(currentCopy);
			if (!this.statesList.contains(currentCopy))
				this.statesList.add(currentCopy);
		}
	}

	/**
	 * Create a copy of the current state and update name
	 * @param stateToCopy the state to copy
	 * @param cpt the current object counter
	 * @return a new State object that is a copy to initial state but with an updated name.
	 */
	private State copyOfState(State stateToCopy, int cpt) {
		return (new State(stateToCopy.getName() + cpt));
	}

	/**
	 * Copy the table of transition 
	 * @param transitionTable the table of transition to copy
	 * @param cpt the current object counter
	 */
	@SuppressWarnings("unchecked")
	public void copyTransitions(
			HashMap<State, HashMap<Symbol, List<State>>> transitionTable,
			int cpt) {
		for (Iterator statesIt = transitionTable.keySet().iterator(); statesIt
				.hasNext();) {
			State currentState = (State) statesIt.next();
			HashMap<Symbol, List<State>> currentCell = transitionTable
					.get(currentState);
			for (Iterator symbolIt = currentCell.keySet().iterator(); symbolIt
					.hasNext();) {
				Symbol currentSymbol = (Symbol) symbolIt.next();
				List<State> currentArrivalList = (List) DoubleHashMap.getValue(
						transitionTable, currentState, currentSymbol);
				if (currentArrivalList != null && currentArrivalList.size() > 0) {
					State equivalentState = this.getStateNamed(currentState
							.getName()
							+ cpt);
					if (equivalentState == null) {
						equivalentState = new State(currentState.getName()
								+ cpt);
						if (!this
								.containsAStateNamed(equivalentState.getName())) {
							this.statesList.add(equivalentState);
						}
					}

					/*
					 * Symbol equivalentSymbol =
					 * this.getSymbolNamed(currentSymbol.getName());
					 * if(equivalentSymbol==null){ equivalentSymbol = new
					 * Symbol(currentSymbol.getName());
					 * if(!this.containsASymbolNamed(equivalentSymbol.getName()))
					 * this.SymbolsList.add(equivalentSymbol); }
					 */
					Symbol equivalentSymbol = currentSymbol;
					if (!this.containsASymbolNamed(equivalentSymbol.getName()))
						this.symbolsList.add(equivalentSymbol);

					for (ListIterator it = currentArrivalList.listIterator(); it
							.hasNext();) {
						State currentElement = (State) it.next();
						State equivalentElement = this
								.getStateNamed(currentElement.getName() + cpt);
						if (equivalentElement == null) {

							equivalentElement = new State(currentElement
									.getName()
									+ cpt);

							if (!this.containsAStateNamed(equivalentElement
									.getName()))
								this.statesList.add(equivalentElement);
						}
						DoubleHashMap.putValueAtList(this.getTransitionTable(),
								equivalentElement, equivalentState,
								equivalentSymbol);
					}
				}
			}
		}
	}

	/**
	 * remove a list of transition objects
	 * @param transitionsToRemove the list of transition to remove
	 */
	public void removeTransitionsList(List<Transition> transitionsToRemove) {
		for (Iterator<Transition> it = transitionsToRemove.iterator(); it
				.hasNext();) {
			Transition currentTransition = it.next();
			removeTransition(currentTransition.getStartState(),
					currentTransition.getSymbol(), currentTransition
							.getEndState());
		}

	}

	/**
	 * Add a list of transitions
	 * @param transitionTablesToAdd the list of transitions to add
	 */
	public void addTransitionTablesList(
			List<HashMap<State, HashMap<Symbol, List<State>>>> transitionTablesToAdd) {
		Iterator<HashMap<State, HashMap<Symbol, List<State>>>> it = transitionTablesToAdd
				.iterator();
		while (it.hasNext()) {
			HashMap<State, HashMap<Symbol, List<State>>> currentTableToAdd = it
					.next();
			/*
			 * System.out.println("---------------table ajouter dans
			 * "+this.getInitialState().getName());
			 * MainPackage.PrintMethods.printTabTrans(currentTableToAdd);
			 */
			this.addTransitions(currentTableToAdd);
		}
	}

	/**
	 * Generate a list of empty transition from the list of transitions 
	 * @param emptyTransitionsToGenerate the list of transitions
	 */
	public void generateTransitionsList(
			List<Transition> emptyTransitionsToGenerate) {
		// System.out.println("---------------generated transition
		// in"+this.getInitialState().getName());
		for (Iterator<Transition> it = emptyTransitionsToGenerate.iterator(); it
				.hasNext();) {
			Transition currentTransition = it.next();
			this.generateTransition(currentTransition.getStartState(),
					currentTransition.getSymbol(), currentTransition.getEndState());
			if (!this.containsAStateNamed(currentTransition.getEndState().getName()))
				this.statesList.add(currentTransition.getEndState());
			// System.out.println(currentTransition.StartState.getName()+"
			// "+currentTransition.Symbol.getName()+"
			// "+currentTransition.EndState.getName());
		}

	}

	/**
	 * Add a number of transition into the transition map
	 * @param transitionsToAdd the list of transition to add
	 */
	public void addTransitionList(List<Transition> transitionsToAdd) {
		for (Iterator it = transitionsToAdd.iterator(); it.hasNext();) {
			Transition currentTransition = (Transition) it.next();
			this.addTransition(currentTransition.getStartState(), currentTransition
					.getSymbol(), currentTransition.getEndState());
		}
	}

	/**
	 * Update the list of final state once the epsilon-closure algorithm ended
	 */
	@SuppressWarnings("unchecked")
	private void updateFinalStatesAfterClosure() {
		// List potentialFinalStates = new ArrayList();
		List<State> firstStates = new ArrayList<State>();
		for (Iterator<Symbol> it = (this.epsilonClosure.get(this
				.getInitialState())).keySet().iterator(); it.hasNext();) {
			Symbol currentSymbol = it.next();
			List<State> currentArrivalList = (List) ((HashMap) this.epsilonClosure
					.get(this.getInitialState())).get(currentSymbol);
			for (Iterator<State> it2 = currentArrivalList.iterator(); it2
					.hasNext();) {
				State currentArrivalState = it2.next();
				if (!firstStates.contains(currentArrivalState)) {
					firstStates.add(currentArrivalState);
				}
			}
		}
		for (Iterator it = this.epsilonClosure.keySet().iterator(); it
				.hasNext();) {
			State currentState = (State) it.next();
			HashMap currentMap = this.epsilonClosure.get(currentState);
			if (currentMap != null && !currentMap.isEmpty()) {
				for (Iterator it2 = (currentMap).keySet().iterator(); it2
						.hasNext();) {
					Symbol currentSymbol = (Symbol) it2.next();
					List currentArrivalList = (List) currentMap
							.get(currentSymbol);
					for (Iterator it3 = currentArrivalList.iterator(); it3
							.hasNext();) {
						State currentArrivalState = (State) it3.next();
						if (firstStates.contains(currentArrivalState)) {
							// if(!potentialFinalStates.contains(currentState)
							// && !currentState.equals(this.getInitialState())){
							if (!currentState.isFinalState()
									&& !currentState.isInitialState()) {
								// System.out.println(currentState.getName()+"
								// is potentially final");
								// potentialFinalStates.add(currentState);
								currentState.setFinal(true);
								finalStatesList.add(currentState);
							}
						}
					}
				}
			} else {
				// if(!potentialFinalStates.contains(currentState)){
				if (!currentState.isFinalState()
						&& !currentState.isInitialState()) {
					// System.out.println(currentState.getName()+" is
					// potentially final suivants null");
					// potentialFinalStates.add(currentState);
					currentState.setFinal(true);
					finalStatesList.add(currentState);
				}

			}
		}

		/*
		 * List pileAlreadyVisited = new ArrayList(); List tampon = new
		 * ArrayList(); List potentialFinalStates = new ArrayList();
		 * tampon.add(this.getInitialState()); while(!tampon.isEmpty()){ State
		 * currentState = (State)tampon.remove(0);
		 * pileAlreadyVisited.add(currentState); HashMap currentMap =
		 * (HashMap)this.EpsilonClosure.get(currentState); for(Iterator it =
		 * currentMap.keySet().iterator();it.hasNext();){ Symbol currentSymbol =
		 * (Symbol)it.next(); List currentArrivalList =
		 * (List)currentMap.get(currentSymbol); for(Iterator
		 * it2=currentArrivalList.iterator();it2.hasNext();){ State
		 * currentArrivalState = (State)it2.next(); if(!
		 * pileAlreadyVisited.contains(currentArrivalState)){
		 * tampon.add(currentArrivalState); }else{
		 * if(!potentialFinalStates.contains(currentState)){
		 * System.out.println(currentState.getName()+" is potentially final");
		 * potentialFinalStates.add(currentState); } } } } }
		 */
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.epsilonClosure.clear();
		this.epsilonClosure = null;
		this.finalStatesList.clear();
		this.finalStatesList = null;
		this.followingSymbols.clear();
		this.followingSymbols = null;
		this.initialState = null;
		this.pile.clear();
		this.pile = null;
		this.statesList.clear();
		this.statesList = null;
		this.symbolsList.clear();
		this.symbolsList = null;
		this.transitionPropertiesClosure.clear();
		this.transitionPropertiesClosure = null;
		this.transitions.clear();
		this.transitions = null;
		this.transitionsProperties.clear();
		this.transitionsProperties = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String result = super.toString() + "\n\tInit:"
				+ this.initialState.getName() + "\n\tFinal:\n";

		for (State state : this.finalStatesList) {
			result += "\t\t" + state.getName() + "\n";
		}

		// TODO Auto-generated method stub
		return result;
	}

}
