/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.ControlConstruct;
import fr.inria.icocoa.servicemodel.Service;

/**
 * Some method used to generate Automaton from a Service
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class ServiceModel2Automata {
	//public static DoubleHashMap dhm = new DoubleHashMap();

	/**
	 * Stack of automaton used during algorithm
	 */
	private HashMap automataStack = new HashMap();

	/**
	 * Create an Automaton from a Service, by opening all inclusive workflows (corresponding to each submethods)
	 * @param s the service we want create the automaton from
	 * @return the Automaton associated to the service
	 */
	@SuppressWarnings("unchecked")
	public Automaton OpenAllBoxes(Service s) {
		if (!s.isUserTask()) {
			Iterator it = s.getProvidedCapabilities().iterator();
			List listOfAutomata = new ArrayList();
			while (it.hasNext()) {
				Capability currentProvidedCapability = (Capability) it.next();
				Automaton currentAutomaton = substitute(currentProvidedCapability);
				listOfAutomata.add(currentAutomaton);
			}
			Automaton union = AutomataUnion.makeUnion(listOfAutomata);
			return (union);
		} else {
			Iterator it = s.getRequiredCapabilities().iterator();
			List listOfAutomata = new ArrayList();
			while (it.hasNext()) {
				Capability currentProvidedCapability = (Capability) it.next();
				Automaton currentAutomaton = substitute(currentProvidedCapability);
				listOfAutomata.add(currentAutomaton);
			}
			Automaton union = AutomataUnion.makeUnion(listOfAutomata);
			return (union);
		}

	}

	/**
	 * This function in used to generate an automaton for a given capability
	 * @param c the capability we want to obtain the associated automaton
	 * @return the automaton associated to the capability
	 */
	public Automaton generate_One_Level_Automaton(Capability c) {
		Automaton result = null;
		if (c.getSubCapabilities().isEmpty()) {
			result = new Automaton();
			result.setInitialState(new InitialState(c.getTypeName() + "0"));
			result.addFinalState(new FinalState(c.getTypeName() + "1"));
			result.addState(result.getInitialState());
			result.addState(result.getFinalStates().get(0));
			Symbol newSymbol = new Symbol(c.getTypeName());
			newSymbol.setServiceName(c.getService().getServiceName());
			result.getSymbols().add(newSymbol);
			result.addTransition(result.getInitialState(), newSymbol,
				result.getFinalStates().get(0));
		} else {
			switch (c.getCC().getValue()) {
			case ControlConstruct.SEQUENCE_VALUE:
				result = AutomataFactory.generateSequenceAutomaton(c);
				break;
			case ControlConstruct.CHOICE_VALUE:
				result = AutomataFactory.generateChoiceAutomaton(c);
				break;
			case ControlConstruct.ANY_ORDER_VALUE:
				result = AutomataFactory.generateAnyOrderAutomaton(c);
				break;
			case ControlConstruct.IF_THEN_ELSE_VALUE:
				result = AutomataFactory.generateIfThenElseAutomaton(c);
				break;
			case ControlConstruct.ITERATE_VALUE:
				result = AutomataFactory.generateIterateAutomaton(c);
				break;
			case ControlConstruct.REPEAT_UNTIL_VALUE:
				result = AutomataFactory.generateRepeatUntilAutomaton(c);
				break;
			case ControlConstruct.REPEAT_WHILE_VALUE:
				result = AutomataFactory.generateRepeatWhileAutomaton(c);
				break;
			case ControlConstruct.SPLIT_VALUE:
				result = AutomataFactory.generateSplitAutomaton(c);
				break;
			case ControlConstruct.SPLIT_JOIN_VALUE:
				result = AutomataFactory.generateSplitJoinAutomaton(c);
				break;
			default:
				break;
			}
		}
		return (result);
	}

	
	@SuppressWarnings("unchecked")
	public Automaton substitute(Capability c) {
		// System.out.println("substitute "+c.getName());
		List transitionsToRemove = new ArrayList();
		List transitionTablesToCopy = new ArrayList();
		List emptyTransitionsToGenerate = new ArrayList();
		boolean addEpsilon = false;
		Automaton alreadyCalculated = (Automaton) automataStack.get(c);
		if (alreadyCalculated == null) {
			c.incNbrCopies();
			Automaton tempAutomaton = copyOf(c.getAutomaton(), c.getNbrCopies());
			if (c.getSubCapabilities().isEmpty()) {
				return (tempAutomaton);
			} else {
				// for(int j = 0;j<tempAutomaton.getStates().size();j++){
				// State currentState = (State)tempAutomaton.getStates().get(j);
				for (Iterator stateIt = tempAutomaton.getTransitionTable().keySet().iterator(); stateIt.hasNext();) {
					// for(Iterator stateIt =
					// tempAutomaton.getStates().iterator();stateIt.hasNext();){
					State currentState = (State) stateIt.next();
					HashMap currentMap = tempAutomaton.getTransitionTable().get(
						currentState);
					// for(int i=0;i<tempAutomaton.getSymbols().size();i++){
					// Symbol currentSymbol =
					// (Symbol)tempAutomaton.getSymbols().get(i);
					for (Iterator symbolIt = currentMap.keySet().iterator(); symbolIt.hasNext();) {
						// for(ListIterator
						// symbolIt=tempAutomaton.getSymbols().listIterator();symbolIt.hasNext();){
						Symbol currentSymbol = (Symbol) symbolIt.next();
						if (currentSymbol != Automaton.EPSILON) {
							// List arrivalStatesList =
							// (List)((HashMap)tempAutomaton.getTransitionTable().get(currentState)).get(currentSymbol);
							List arrivalStatesList = (List) DoubleHashMap.getValue(
								tempAutomaton.getTransitionTable(),
								currentState, currentSymbol);
							if (arrivalStatesList != null) { /*
																 * If it is a
																 * real
																 * transition
																 */
								// for(int k=0;k<arrivalStatesList.size();k++){
								// State
								// currentArrivalState=(State)arrivalStatesList.get(k);
								for (ListIterator arrivalStatesListIt = arrivalStatesList.listIterator(); arrivalStatesListIt.hasNext();) {
									State currentArrivalState = (State) arrivalStatesListIt.next();
									Capability subCapability = c.getSubCapabilityFromLocalName(currentSymbol.getName());
									// DebugPrinter.print("currentSymbol.getName()
									// =
									// "+currentSymbol.getName(),DebugPrinter.INFO);

									// DebugPrinter.print("subCapability
									// "+subCapability,DebugPrinter.INFO);
									Automaton automatonToConnect = substitute(subCapability);
									// System.out.println("----Retour de
									// substitute de---
									// "+c.getName()+"------"+subCapability.getName()+"
									// --------------------");
									// MainPackage.PrintMethods.printTabTrans(automatonToConnect.getTransitionTable(),
									// automatonToConnect.getStates(),
									// automatonToConnect.getSymbols());*/
									// tempAutomaton.copyStatesList(automatonToConnect.getStates(),currentCopiesOfsubCapa);
									// tempAutomaton.copySymbols(automatonToConnect.getSymbols());
									/*
									 * System.out.println("---------------
									 * "+c.getName()+" Avant
									 * --------------------");
									 * MainPackage.PrintMethods.printTabTrans(tempAutomaton.getTransitionTable());
									 * System.out.println("--------------- To
									 * Connect --------------------");
									 * MainPackage.PrintMethods.printTabTrans(automatonToConnect.getTransitionTable());
									 */
									transitionTablesToCopy.add(automatonToConnect.getTransitionTable());
									/*
									 * System.out.println("---------------
									 * "+c.getName()+" Aprs
									 * --------------------");
									 * MainPackage.PrintMethods.printTabTrans(tempAutomaton.getTransitionTable());
									 */
									transitionsToRemove.add(new Transition(
											currentState, currentSymbol,
											currentArrivalState));
									if (tempAutomaton.getSymbolNamed("epsilon") == null) {
										addEpsilon = true;
									}
									emptyTransitionsToGenerate.add(new Transition(
											currentState,
											Automaton.EPSILON,
											automatonToConnect.getInitialState()));
									automatonToConnect.getInitialState().isInitial = false;
									for (Iterator finalStatesIt = automatonToConnect.getFinalStates().iterator(); finalStatesIt.hasNext();) {
										State currentFinalStateToConnect = (State) finalStatesIt.next();
										currentFinalStateToConnect.isFinal = false;
										emptyTransitionsToGenerate.add(new Transition(
												currentFinalStateToConnect,
												Automaton.EPSILON,
												currentArrivalState));
									}
								}
							}
						}
					}
				}
				if (addEpsilon)
					tempAutomaton.getSymbols().add(Automaton.EPSILON);
				tempAutomaton.removeTransitionsList(transitionsToRemove);
				tempAutomaton.addTransitionTablesList(transitionTablesToCopy);
				tempAutomaton.generateTransitionsList(emptyTransitionsToGenerate);
				automataStack.put(c, tempAutomaton);
				/*
				 * System.out.println("--------------- "+c.getName()+" Aprs
				 * substitute--------------------");
				 * MainPackage.PrintMethods.printTabTrans(tempAutomaton.getTransitionTable());
				 */
				return (tempAutomaton);
			}
		} else {
			c.incNbrCopies();
			return (copyOf(alreadyCalculated, c.getNbrCopies()));
		}
	}

	/**
	 * Copy an automaton
	 * @param automaton the automaton to copy
	 * @param cpt the actual index
	 * @return the copy of the automaton
	 */
	private Automaton copyOf(Automaton automaton, int cpt) {
		/*
		 * System.out.println("-----------------L'Original----------------");
		 * MainPackage.PrintMethods.printTabTrans(automaton.getTransitionTable());
		 */
		Automaton tempAutomaton = new Automaton();
		/* Copy the initial state */
		tempAutomaton.setInitialState(new InitialState(
				automaton.getInitialState().getName() + cpt));
		tempAutomaton.getStates().add(tempAutomaton.getInitialState());
		/* Copy of the final states list */
		tempAutomaton.copyFinalStatesList(automaton.getFinalStates(), cpt);
		/* Copy of the states list */
		// tempAutomaton.copyStatesList(automaton.getStates(),cpt);
		/* Copy of the symbols list */
		// tempAutomaton.copySymbols(automaton.getSymbols());
		/* Copy of the transitions table */
		tempAutomaton.copyTransitions(automaton.getTransitionTable(), cpt);
		/*
		 * System.out.println("************
		 * "+tempAutomaton.getInitialState().getName()+" **********");
		 * MainPackage.PrintMethods.printTabTrans(tempAutomaton.getTransitionTable(),
		 * tempAutomaton.getStates(), tempAutomaton.getSymbols());
		 */
		/*
		 * System.out.println("-----------------La Copie----------------");
		 * MainPackage.PrintMethods.printTabTrans(tempAutomaton.getTransitionTable());
		 */
		return tempAutomaton;
	}

	
}
