/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.AmigoSParserLink;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.MapContainer;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.ParserThread;

/**
 * This class offer a tool to parse an Amigo-S description file.<b> This
 * description must be include between tags : <rdf:RDF></rdf:RDF> </b>. If the
 * document doesn't contains such tags, then an error message will be return !
 * the process may continue without Exception, but the result will not be
 * consistant with the contains of the file.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 2, 2007 4:51:39 PM
 */
public class Parser {

	/** The starting point of the AmigoS description */
	public static final String RDF = "rdf:RDF";

	/**
	 * The String that permit to split concept prefix and xml language (use in
	 * method createMapOfPrefix)
	 */
	private static final String SPLITTER = ":";

	/**
	 * THe head of the chain of subparser
	 */
	private AmigoSParserLink parserChainHead;

	/**
	 * A reference to the container that will contains all the created object
	 */
	private MapContainer mapContainer;

	public Parser(AmigoSParserLink head, MapContainer mc) {
		this.parserChainHead = head;
		this.mapContainer = mc;
	}

	/**
	 * @return the mapContainer
	 */
	public MapContainer getMapContainer() {
		return mapContainer;
	}

	/**
	 * @param mapContainer
	 *            the mapContainer to set
	 */
	public void setMapContainer(MapContainer mapContainer) {
		this.mapContainer = mapContainer;
	}

	/**
	 * @return the parserChainHead
	 */
	public AmigoSParserLink getParserChainHead() {
		return parserChainHead;
	}

	/**
	 * @param parserChainHead
	 *            the parserChainHead to set
	 */
	public void setParserChainHead(AmigoSParserLink parserChainHead) {
		this.parserChainHead = parserChainHead;
	}

	/**
	 * launch the parsing of a part of code
	 * 
	 * @param rootNode
	 *            the root node of the part of code to parse
	 * @throws ParsingException
	 *             this exception is thrown when an error occurs during parsing
	 */
	public void parse(Node rootNode) throws ParsingException {
		this.getParserChainHead().parse(rootNode);
	}

	/**
	 * Parse an AmigoS document and obtain a Service object
	 * 
	 * @param XLMDocumentPath
	 *            the Path of the document containing the XML description to
	 *            parse
	 * @return the Service object corresponding to the AmigoS description
	 * @throws ParserConfigurationException
	 *             if a problem occurs with configuration of DOMParser
	 * @throws IOException
	 *             if a problem occurs with the given file
	 * @throws SAXException
	 *             if a problem occurs during parsing
	 * @throws InterruptedException
	 *             if a problem occurs due to multithreading
	 */
	public Service parse(String XLMDocumentPath)
			throws ParserConfigurationException, SAXException, IOException,
			InterruptedException {

		/* Generate the DOM tree, and initialise DOM Parser */
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		FileInputStream fis = new FileInputStream(new File(XLMDocumentPath));

		// DOM parsing of the document => construct the object tree
		Document doc = builder.parse(fis);

		/* Walk through the DOM tree to get the information */

		// get a Factory of Parser
		ParserFactory pf = ParserFactory.getInstanceOfFactory();

		// update the map of suffix (for factory and for the current parser */
		pf.setPrefixs(this.createMapOfPrefix(doc));

		// get a new instance of the chain of responsability with updated prefix
		this.setParserChainHead(pf.getInstanceChainOfResponsability(this.mapContainer));

		// create new thread conatining the parser and a ref on the child
		Thread t = new ParserThread(doc, this);

		// start the thread
		t.start();

		// wait the end of the thread
		t.join();

		return null;
	}

	/**
	 * Generate the map of the prefix of concept to calculate (later) the tag
	 * that will be recognise during parsing
	 * 
	 * @param doc
	 *            the document from which prefix of concept needs to be
	 *            extracted
	 * @return a Map ontologyURI -> prefixValue extracted from the document.
	 */
	private Map<String, String> createMapOfPrefix(Document doc) {
		Map<String, String> prefixs = new HashMap<String, String>();

		// get the node coresponding to the starting point of the Amiog-S
		// description (contains the list of prefix and the associated URI of
		// ontologies as attributes)
		Node rdf = doc.getElementsByTagName(Parser.RDF).item(0);

		// get the attributes of the node and list them
		NamedNodeMap attributes = rdf.getAttributes();
		for (int i = 0; i < attributes.getLength(); i++) {
			Node tmp = attributes.item(i);
			/*
			 * DebugPrinter.print("\n\t- node "+i+" : "+tmp.getLocalName() + " - " +
			 * tmp.getNodeName() + " - " + tmp.getBaseURI()+" -
			 * "+tmp.getNodeValue(), DebugPrinter.INFO);
			 */

			// extract the prefix from the nodeName
			String localName = tmp.getNodeName();
			String prefix = "";
			try {
				prefix = localName.split(Parser.SPLITTER)[1];
			} catch (IndexOutOfBoundsException e) {
				// nothing to do, it means that the associated prefix is ""
			}

			// store the data in the map
			prefixs.put(tmp.getNodeValue(), prefix);
		}

		// DebugPrinter.print(prefixs, "prefixs", DebugPrinter.INFO);

		return prefixs;
	}

	@Override
	public String toString() {
		if (this.getParserChainHead() != null)
			return this.getParserChainHead().toString();
		else
			return "null";
	}

}
