/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParserNotFoundException;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.AmigoSParserLink;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.ThreadCreatorParserLink;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.MapContainer;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.SharedLock;

/**
 * This factory gather the list of instance of AmigoSParserLink that needs to be
 * linked to form a parser using design pattern <i>Chain of Responibility</i>;
 * and the list of suffix for concept to parse. <br>
 * An instance of ThreadCreatorParserLink will be automaticly added at the end
 * of the list of instance. <br>
 * <br>
 * This factory follows the design pattern "Singleton" so you can obtain an
 * instance of the factory by Calling ParserFactory.getInstance(), but you can
 * reinitialise the factory by calling ParserFactory.init().
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 2, 2007 3:17:21 PM
 */
public class ParserFactory {

	// ////////////////////////
	// /// Private Fields /////
	// ////////////////////////

	/** The current instance */
	private static ParserFactory instance = null;

	/** The map (JavaClassName -> JavaObject) of subparser that have to be linked */
	private Map<String, AmigoSParserLink> subparsers = new HashMap<String, AmigoSParserLink>();

	/** The current instance of the head of the chain of responsability */
	private AmigoSParserLink headInstance = null;

	/**
	 * The map (JavaClassName -> SharedLock) of locks assocoated to each
	 * subparser
	 */
	private Map<String, SharedLock> locks = new HashMap<String, SharedLock>();

	/** The map of prefix that needs to be injected to each instance of parser */
	private Map<String, String> prefixs = new HashMap<String, String>();

	// /////////////////////////////////
	// /// Modificators for fields /////
	// /////////////////////////////////

	/**
	 * Add a new subparser to the list of subparser
	 * 
	 * @param parser
	 *            the parser to add
	 */
	private void addParser(AmigoSParserLink parser) {
		String className = parser.getClass().getName();
		this.subparsers.put(className, parser);
		this.locks.put(className, new SharedLock());
	}

	/**
	 * Remove a new subparser to the list of subparser
	 * 
	 * @param parserClassName
	 *            the java class name of the parser to remove
	 */
	public void removeParser(String parserClassName) {
		this.subparsers.remove(parserClassName);
		this.locks.remove(parserClassName);
	}

	/**
	 * Add a new subparser to the list of subparser
	 * 
	 * @param parserClassName
	 *            the java class name of the parser to remove
	 * @throws ParserNotFoundException
	 *             exception thrown when the given class has not been found or
	 *             is not a child of the class AmigoSParserLink
	 */
	public void addParser(String parserClassName)
			throws ParserNotFoundException {
		try {
			// creation of the shared lock
			SharedLock lock = new SharedLock();

			Class c = Class.forName(parserClassName);

			try {
				Constructor constructor = c.getConstructor(lock.getClass());

				try {
					Object o = constructor.newInstance(new Object[] { lock });

					if (o instanceof AmigoSParserLink) {
						this.addParser((AmigoSParserLink) o);
					} else {
						throw new ParserNotFoundException("The class "
								+ parserClassName + " doesn't extends "
								+ AmigoSParserLink.class.getName());
					}
				} catch (IllegalArgumentException e) {
					throw new ParserNotFoundException(e);
				} catch (InvocationTargetException e) {
					throw new ParserNotFoundException(e);
				}

			} catch (SecurityException e) {
				throw new ParserNotFoundException(
						"The class "
								+ parserClassName
								+ " doesn't allow to access a constructor with only one parameter (type="
								+ lock.getClass().getName() + ")");
			} catch (NoSuchMethodException e) {
				throw new ParserNotFoundException(
						"The class "
								+ parserClassName
								+ "  doesn't contain any constructor with only one parameter (type="
								+ lock.getClass().getName() + ")");
			}

		} catch (ClassNotFoundException e) {
			throw new ParserNotFoundException(e);
		} catch (InstantiationException e) {
			throw new ParserNotFoundException(e);
		} catch (IllegalAccessException e) {
			throw new ParserNotFoundException(e);
		}

	}

	/**
	 * Get a String representation of the parsers contained into the factory
	 * 
	 * @return a string representation of this object
	 */
	@Override
	public String toString() {
		String result = "Parser created will contained :";
		for (String apl : new ArrayList<String>(this.subparsers.keySet())) {
			result += "\t- " + apl;
		}
		return result;
	}

	/**
	 * @return the prefixs
	 */
	public Map<String, String> getPrefixs() {
		return prefixs;
	}

	/**
	 * @param prefixs
	 *            the prefixs to set
	 */
	public void setPrefixs(Map<String, String> prefixs) {
		this.prefixs = prefixs;
	}

	/**
	 * @return the locks
	 */
	public Map<String, SharedLock> getLocks() {
		return locks;
	}

	/**
	 * @return the headInstance
	 */
	private AmigoSParserLink getHeadInstance() {
		return headInstance;
	}

	/**
	 * @param headInstance
	 *            the headInstance to set
	 */
	private void setHeadInstance(AmigoSParserLink headInstance) {
		this.headInstance = headInstance;
	}

	/**
	 * @param locks
	 *            the locks to set
	 */
	public void setLocks(Map<String, SharedLock> locks) {
		this.locks = locks;
	}

	/**
	 * @return the subparsers
	 */
	public Map<String, AmigoSParserLink> getSubparsers() {
		return subparsers;
	}

	/**
	 * @param subparsers
	 *            the subparsers to set
	 */
	public void setSubparsers(Map<String, AmigoSParserLink> subparsers) {
		this.subparsers = subparsers;
	}

	// /////////////////////////////////////////
	// //// Methode relative to "Factory" //////
	// /////////////////////////////////////////

	/**
	 * Get an instance of object Parser
	 * 
	 * @param mc
	 *            the container of the different map used for parsing
	 * @return the given instance
	 */
	public Parser getInstanceOfParser(MapContainer mc) {

		if (this.prefixs.size() == 0) {
			DebugPrinter.print(
				"The parser that will be created has no associated prefix (ex: xmlns:lang=\"http://www.hitech-projects.com/euprojects/amigo/Amigo-S.owl#\".\nPlease check that your description starts with "
						+ Parser.RDF
						+ " and that this tag has attributes as given in previous exemple.",
				DebugPrinter.WARN);
		}

		/*
		 * Agregate a new MapContainer with the linked chain of parser and
		 * return an instance of parser
		 */
		return new Parser(this.getHeadInstance(), mc);
	}

	/**
	 * Get an instance of object Parser (create a new MapContainer instance)
	 * 
	 * @return the given instance
	 */
	public Parser getInstanceOfParser() {
		// get the old value of the debug mode
		boolean oldValue = DebugPrinter.isDebugMode();

		// disable debugmode
		DebugPrinter.setDebugMode(false);

		// get the instance of parser (without error message)
		Parser p = this.getInstanceOfParser(new MapContainer());

		// restore value of debug mode
		DebugPrinter.setDebugMode(oldValue);

		// return the result
		return p;
	}

	/**
	 * get a clone of the chain of responsability (by cloning each subparser)
	 * and associate the gived MapContainer
	 * 
	 * @param mc
	 *            the container of the different map used for parsing
	 * @return the head of the chain of responsability
	 */
	public AmigoSParserLink getInstanceChainOfResponsability(MapContainer mc) {
		if (this.getHeadInstance() == null) {
			this.setHeadInstance(createInstanceChainOfResponsability(mc));
		}
		return this.getHeadInstance().strongClone();

	}

	/**
	 * Create the chain of responsability (by cloning each subparser) and
	 * associate a new MapContainer
	 * 
	 * @param mc
	 *            the container of the different map used for parsing
	 * @return the head of the chain of responsability
	 */
	public AmigoSParserLink createInstanceChainOfResponsability(MapContainer mc) {

		DebugPrinter.print(
			subparsers,
			"ParserFactory::createInstanceChainOfResponsability --> subparsers\n",
			DebugPrinter.INFO);
		DebugPrinter.print(locks,
			"ParserFactory::createInstanceChainOfResponsability --> locks\n",
			DebugPrinter.INFO);

		/*
		 * Create the chain of responsability (by cloning each subparser) and
		 * associate a new MapContainer
		 */
		try {

			// head of the chain of responsability
			AmigoSParserLink head = null;

			if (subparsers.size() > 0) {

				// variable declaration
				AmigoSParserLink previous = null;
				SharedLock lock = null;

				/*
				 * Walk through the list of subparsers and link them by updating
				 * the "next" field of each AmigoSParserLink instance.
				 */
				boolean firstEntry = true; // tag if it is the first time the
				// algotithm goes through the loop
				for (Entry<String, AmigoSParserLink> it : new ArrayList<Entry<String, AmigoSParserLink>>(
						subparsers.entrySet())) {

					if (firstEntry) {
						// initialise head
						previous = it.getValue().clone();
						this.updatePrefix(previous);
						previous.setContainer(mc);
						lock = locks.get(it.getKey());
						previous.setLock(lock.clone());
						head = previous;

						// the first Entry has been treated
						firstEntry = false;
					} else {

						// loop to create the chain of responsability

						AmigoSParserLink current = it.getValue().clone();
						this.updatePrefix(current);
						previous.setNext(current);
						current.setContainer(mc);
						lock = locks.get(it.getKey());
						current.setLock(lock.clone());
						previous = current;
					}

				}

				// conclude the chain by adding the ThreadCreatorParserLink
				// instance
				lock = new SharedLock();
				ThreadCreatorParserLink end = new ThreadCreatorParserLink(mc,
						lock.clone());
				previous.setNext(end);

			} else { // no subparser, so just display a message and add
				// ThreadCreatorParserLink instance
				DebugPrinter.print(
					"No key word will be recognize by your parser ! Please add some parser to the ParserFactory !",
					DebugPrinter.WARN);

				SharedLock lock = locks.get(ThreadCreatorParserLink.class.getName());

				head = new ThreadCreatorParserLink(mc, lock);
			}

			return head;
		} catch (CloneNotSupportedException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
			return null;
		}
	}

	/**
	 * Update the unknown prefiux to permit SubParser to indentify the needed
	 * tags.
	 * 
	 * @param link
	 *            the Parser to update
	 */
	private void updatePrefix(AmigoSParserLink link) {

		// get the list of concept to update
		String[] unknows = link.getURIOfBasesConcept();

		// Reminder : unknows is an array containing as pair value the name of
		// the prefix (as define by the developer of the subparser), and as
		// unpair value the ontologyURI that needs to be paired with the prefix
		// value cointained into the map.
		// For example:
		// String[]{"prefix1","http://www.daml.org/services/owl-s/1.1/Process.owl#"}

		// /Check the lengh of the array
		if (unknows.length % 2 != 0) {
			// the length is not pair => bad format ! => Display Error
			DebugPrinter.print(
				"The method getURIOfBasesConcept() of class "
						+ link.getClass()
						+ " returns an array with a incorrect length (must be pair) !",
				DebugPrinter.INFO);

		} else {

			// loop on it to get value from the map and reinject it into the
			// parser
			for (int i = 0; i < unknows.length; i = i + 2) {
				// get value from the map
				String prefix = this.prefixs.get(unknows[i + 1]);

				// if prefix not found, display an error message
				if (prefix == null) {
					DebugPrinter.print(
						"The prefix corresponding to "
								+ unknows[i + 1]
								+ " has not been found. \nPlease check that your description file contains an attribute for the tag "
								+ Parser.RDF
								+ " as xmlns:toto=\""
								+ unknows[i + 1]
								+ "\" where toto is the value of the required prefix.\n !!!!!!!!!! The Parser for tag "
								+ link.getMyKeyword()
								+ " has been desactivated !!!",
						DebugPrinter.WARN);

				} else {
					// reinject the value of the prefix to the parser
					link.setPrefixValue(unknows[i], prefix);
				}
			}
		}
	}

	// /////////////////////////////////////////
	// //// Method relative to "Singloton" /////
	// /////////////////////////////////////////

	/**
	 * Get the current instance of ParserFactory
	 * 
	 * @return the single instance of ParserFactory
	 */
	public static ParserFactory getInstanceOfFactory() {
		if (ParserFactory.instance == null) {
			ParserFactory.instance = new ParserFactory();
		}
		return ParserFactory.instance;
	}

	/**
	 * Initialise the ParserFactory. The local instance will be set to <b>null</b>,
	 * and a new instance of ParserFactory will be created when the method
	 * getInstance() will be called !
	 * 
	 */
	public static void init() {
		ParserFactory.instance = null;
	}

}
