/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser.specializedparsers;

import org.w3c.dom.Node;

import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.MapContainer;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.SharedLock;

/**
 * This class implement the parser. It use the Design Pattern "Chain of
 * Responsability". For main tags, a specialised parser is responsible of the
 * parsing of the corresponding part of XML File. The main advantage of "chain
 * of responsability" is that you can add easily some new Specific Handler.
 * 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 31, 2007 3:12:58 PM
 */
public abstract class AmigoSParserLink implements Cloneable {

	// ////////////////////////
	// /// Private Fields /////
	// ////////////////////////

	/** Container of the different map of object used for parsing */
	protected MapContainer container = null;

	/** the next element in the chain of responsibility */
	protected AmigoSParserLink next = null;

	/** the keyword of which the Parser is responsible */
	protected String myKeyword = null;

	/** the lock shared by all instance of a ParserLink */
	protected SharedLock lock = null;

	// //////////////////////
	// /// Constructors /////
	// //////////////////////

	/**
	 * Create a new Parser by giving the object container and its keyword
	 * 
	 * @param mp
	 *            the container of parsed object
	 * @param keyword
	 *            the keyword of which the Parser is responsible *
	 * @param lock
	 *            the shared lock
	 */
	protected AmigoSParserLink(MapContainer mp, String keyword, SharedLock lock) {
		this.container = mp;
		this.myKeyword = keyword;
		this.lock = lock;
	}

	/**
	 * Create a new Parser by giving the object container
	 * 
	 * @param mp
	 *            the container of parsed object *
	 * @param lock
	 *            the shared lock
	 */
	protected AmigoSParserLink(MapContainer mp, SharedLock lock) {
		this(mp, null, lock);
	}

	/**
	 * Create a new Parser
	 * 
	 * @param lock
	 *            the shared lock
	 */
	protected AmigoSParserLink(SharedLock lock) {
		this(null, null, lock);
	}

	// ///////////////////////
	// /// Abstract part /////
	// ///////////////////////

	/**
	 * Parse the part of XML code which root is given as parameter. This method
	 * just execute the part of code corresponding to the design pattern "Chain
	 * of Responsability", then give the hand to the method
	 * "parse_MethodContent".
	 * 
	 * @param rootNode
	 *            the root node of the part of code to parse
	 * @throws ParsingException
	 *             this exception is thrown when an error occurs during parsing
	 */
	public void parse(Node rootNode) throws ParsingException {

		// DebugPrinter.print("AmigoSParserLink::parse(..) Class="
		// + this.getClass().getSimpleName() + "\n rootNode =
		// "+rootNode.getNodeName(), DebugPrinter.INFO);

		// check if it my responsability
		if (rootNode.getNodeName().equals(this.getMyKeyword())) {

			// DebugPrinter.print("AmigoSParserLink::parse(..) Class="
			// + this.getClass().getSimpleName(), DebugPrinter.INFO);

			this.parse_MethodContent(rootNode);

		} else {// this is not my keyword so let responsability to the next
			// parser
			if (this.getNext() != null)
				this.getNext().parse(rootNode);
		}
	}

	/**
	 * Parse the part of XML code which root is given as parameter. This is the
	 * real method content, that mean that the part of code corresponding to
	 * design pattern "Chain of Responsability" has been already done ! So if
	 * enter this method, it means that the keyword is the keyword recognized by
	 * the class.
	 * 
	 * @param rootNode
	 *            the root node of the part of code to parse
	 * @throws ParsingException
	 *             this exception is thrown when an error occurs during parsing
	 */
	abstract protected void parse_MethodContent(Node rootNode)
			throws ParsingException;

	/**
	 * Parse the part of XML code which root is given as parameter and return
	 * the corresponding object. This method just execute the part of code
	 * corresponding to the design pattern "Chain of Responsability", then give
	 * the hand to the method "parseNextObject_MethodContent".
	 * 
	 * @param rootNode
	 *            the root node of the part of code to parse
	 * @return the object describe by the parsed part of code, <b>null</b> if
	 *         none object is associated with the part of code. *
	 * @throws ParsingException
	 *             this exception is thrown when an error occurs during parsing
	 */
	public Object parseNextObject(Node rootNode) throws ParsingException {
		if (rootNode.getNodeName().equals(this.getMyKeyword())) {
			return this.parseNextObject_MethodContent(rootNode);
		} else {
			// this is not a Output, so give responsability of parsing to the
			// next link !
			if (this.getNext() != null)
				return this.getNext().parseNextObject(rootNode);
			return null;
		}
	}

	/**
	 * Parse the part of XML code which root is given as parameter and return
	 * the corresponding object. This is the real method content, that mean that
	 * the part of code corresponding to design pattern "Chain of
	 * Responsability" has been already done ! So if enter this method, it means
	 * that the keyword is the keyword recognized by the class
	 * 
	 * @param rootNode
	 *            the root node of the part of code to parse
	 * @return the object describe by the parsed part of code, <b>null</b> if
	 *         none object is associated with the part of code. *
	 * @throws ParsingException
	 *             this exception is thrown when an error occurs during parsing
	 */
	abstract protected Object parseNextObject_MethodContent(Node rootNode)
			throws ParsingException;

	/**
	 * Get the list of URI of the concepts that the subparser needs to
	 * recognise. (ex: if the Amigo-S file contains
	 * <i>xmls:process="http://www.daml.org/services/owl-s/1.1/Process.owl#"</i>,
	 * then this method needs to return new
	 * String[]{"prefix1","http://www.daml.org/services/owl-s/1.1/Process.owl#"}.
	 * <br>
	 * This method will be called by the ParserFactory to update the prefix of
	 * the tag that need to be recongnise (ex: "process:Output"). Then the
	 * caller will invoke method setPrefix with parameters "prefix1" and
	 * "process". That means that the value of prefix1 is process.
	 * 
	 * @return the list of URI of ontology that need to be associated with a
	 *         prefix for impair value, and the associated prefix name as pair
	 *         value.
	 */
	abstract public String[] getURIOfBasesConcept();

	/**
	 * Associated the name of a prefix to a value. The call of this method
	 * should update the different tags that needs to be recognised by the
	 * parser.
	 * 
	 * @param prefixName
	 *            the name of the prefix
	 * @param prefixValue
	 *            the value of the prefix
	 * @see #getURIOfBasesConcept()
	 */
	public abstract void setPrefixValue(String prefixName, String prefixValue);

	/**
	 * Give a String representation of the ParserLink
	 * 
	 * @return the String that represent the current ParserLink
	 */
	protected abstract String myToString();

	/*
	 * Give a clone of the current object (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	public abstract AmigoSParserLink clone();

	/*
	 * Give a clone of the current object (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String result = this.myToString();
		if (this.getNext() != null) {
			result += this.getNext().toString();
		}
		return result;
	}

	/**
	 * Give a String containing the fields declared at AmigoSParserLink level
	 * and there associated values
	 * 
	 * @return the describe string
	 */
	protected String displayField() {
		String result = "------------------------------\n";
		Class c = this.getClass();
		result += " - Class = " + c.getName() + "\n";
		result += " - Next = "
				+ (this.getNext() != null ? this.getNext().getClass().getName()
						: "null") + "\n";
		result += " - MyKeyWord = " + this.getMyKeyword() + "\n";
		result += "------------------------------\n";
		return result;
	}

	/**
	 * Create a recursive clone of the chain of responsobility
	 * 
	 * @return a clone of this object and of the following (in chain of
	 *         Responsability) AmigoSParserLink
	 */
	public AmigoSParserLink strongClone() {
		AmigoSParserLink clone = this.clone();
		clone.setMyKeyword(this.getMyKeyword());
		clone.setLock(this.getLock());
		if (this.getNext() != null)
			clone.setNext(this.getNext().strongClone());
		return clone;
	}

	// ////////////////////////////
	// /// Getters \ Setters /////
	// ////////////////////////////

	/**
	 * Set the next specialized Parser
	 * 
	 * @param ap
	 *            the next Amigo parser
	 * @return the current object (this)
	 */
	public AmigoSParserLink setNext(AmigoSParserLink ap) {
		next = ap;
		return this;
	}

	/**
	 * @return the container
	 */
	public MapContainer getContainer() {
		return container;
	}

	/**
	 * @param container
	 *            the container to set
	 */
	public void setContainer(MapContainer container) {
		this.container = container;
	}

	/**
	 * @return the myKeyword
	 */
	public String getMyKeyword() {
		return myKeyword;
	}

	/**
	 * @param myKeyword
	 *            the myKeyword to set
	 */
	public void setMyKeyword(String myKeyword) {
		this.myKeyword = myKeyword;
	}

	/**
	 * @return the next
	 */
	public AmigoSParserLink getNext() {
		return next;
	}

	/**
	 * @return the lock
	 */
	public SharedLock getLock() {
		return lock;
	}

	/**
	 * @param lock
	 *            the lock to set
	 */
	public void setLock(SharedLock lock) {
		this.lock = lock;
	}

}
