package fr.inria.icocoa.servicemodel.domparser.specializedparsers;

import java.lang.reflect.Field;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.MapContainer;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.SharedLock;

/**
 * This parser is specialised for parsing inputs
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 3, 2007 3:09:15 PM
 */
public class InputParserLink extends AmigoSParserLink {

	/**
	 * The URI of the ontology containing the concept "Input"
	 */
	private static final String MYURI1 = "http://www.daml.org/services/owl-s/1.1/Process.owl#";

	/**
	 * The URI of the ontology containing the concept "ID"
	 */
	private static final String MYURI2 = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

	/** the prefix corresponding to the URI of the ontology MUURI1 */
	private String prefix1 = "";

	/** the prefix corresponding to the URI of the ontology MUURI2 */
	private String prefix2 = "";

	/**
	 * the string corresponding of the localName of the rootNode that this
	 * subparser is responsible
	 */
	private static final String PROCESS_HASINPUT = "hasInput";

	/**
	 * the string corresponding of the localName of the rootNode that this
	 * subparser is responsible
	 */
	private static final String PROCESS_INPUT = "Input";

	/** the string corresponding of the localName of the attribute giving an ID */
	private static final String RDF_ID = "ID";

	/**
	 * the string corresponding of the localName of the attribute giving an
	 * reference to an output declared somewhere else
	 */
	private static final String RDF_RESOURCE = "resource";

	/**
	 * the string corresponding of the localName of the node giving the
	 * parameter type of a Output
	 */
	private static final String PROCESS_PARAMETERTYPE = "parameterType";

	/**
	 * @param lock
	 *            the shared lock
	 */
	public InputParserLink(SharedLock lock) {
		super(lock);
		this.setMyKeyword(this.getCompleteTagName(InputParserLink.PROCESS_HASINPUT));
	}

	/**
	 * @param mp
	 * @param lock
	 *            the shared lock
	 */
	public InputParserLink(MapContainer mp, SharedLock lock) {
		super(mp, null, lock);
		this.setMyKeyword(this.getCompleteTagName(InputParserLink.PROCESS_HASINPUT));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.AmigoSParserLink#clone()
	 */
	@Override
	public AmigoSParserLink clone() {
		InputParserLink clone = new InputParserLink(this.getContainer(),
				this.getLock());
		clone.prefix1 = prefix1;
		clone.prefix2 = prefix2;
		return clone;
	}

	@Override
	public String[] getURIOfBasesConcept() {
		return new String[] { "prefix1", InputParserLink.MYURI1, "prefix2",
				InputParserLink.MYURI2 };
	}

	@Override
	public void setPrefixValue(String prefixName, String prefixValue) {
		try {
			Field fd = this.getClass().getDeclaredField(prefixName);
			fd.set(this, prefixValue);
		} catch (SecurityException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (NoSuchFieldException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		}
		this.setMyKeyword(this.getCompleteTagName(PROCESS_HASINPUT));
	}

	/**
	 * Get the string corresponding to the tag to recognise
	 * 
	 * @param concept
	 *            the name of the concept to recognise as tag
	 * @return the complete tag to recognise, <i>empty string</i> if an unknown
	 *         concept is given as parameter
	 */
	private String getCompleteTagName(String concept) {
		if (concept.equals(PROCESS_HASINPUT))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_INPUT))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_PARAMETERTYPE))
			return this.prefix1 + ":" + concept;
		if (concept.equals(RDF_ID))
			return this.prefix2 + ":" + concept;
		if (concept.equals(RDF_RESOURCE))
			return this.prefix2 + ":" + concept;
		return "";
	}

	/**
	 * This method with extract an Input Object from the given part of the XML
	 * file
	 * 
	 * @param startingNode
	 *            the node from which we have to search the information
	 * @return the Input object, <b>null</b> if an error occurs
	 */
	private Input extractInput(Node startingNode) {

		Node child = startingNode;

		// the information to gather to declare the output
		String localName = "";
		String type = "";

		loop1: while (child != null) {

			// get tag Output
			if (child.getNodeName().equals(
				this.getCompleteTagName(PROCESS_INPUT))) {
				// look for attributes toi find name
				NamedNodeMap mapOfAttributes = child.getAttributes();
				for (int i = 0; i < mapOfAttributes.getLength(); i++) {
					Node tmp = mapOfAttributes.item(i);

					if (tmp.getNodeName().equals(
						this.getCompleteTagName(RDF_ID))) {
						// get the name of the pointed Output
						localName = tmp.getNodeValue();
					}
				}

				// look for child to get type
				Node subchild = child.getFirstChild();
				while (subchild != null) {
					// we meet the next tag correct following the langage -> get
					// Type
					if (subchild.getNodeName().equals(
						this.getCompleteTagName(PROCESS_PARAMETERTYPE))) {
						type = child.getTextContent().trim();
						break loop1;
					}
					subchild = subchild.getNextSibling();
				}
			}
			child = child.getNextSibling();
		}

		return new Input(localName, type);
	}

	@Override
	protected Object parseNextObject_MethodContent(Node rootNode)
			throws ParsingException {
		NamedNodeMap mapOfAttributes = rootNode.getAttributes();

		// boolean to detect if the output is a declaration or just a
		// reference
		boolean isResource = false;

		// the information to gather to declare the output
		String localName = "";

		// read the attributes to detect the type of declaration
		for (int i = 0; i < mapOfAttributes.getLength(); i++) {
			Node tmp = mapOfAttributes.item(i);

			if (tmp.getNodeName().equals(this.getCompleteTagName(RDF_RESOURCE))) {
				isResource = true;
				// get the name of the pointed Output
				String ref = tmp.getNodeValue();

				// remove the "#" before the name
				localName = ref.substring(1);
			}
		}

		if (isResource) {
			return new Input(IO.UNKNOWN, IO.UNKNOWN, localName);
		} else {
			return this.extractInput(rootNode.getFirstChild());
		}
	}

	@Override
	protected void parse_MethodContent(Node rootNode) throws ParsingException {
		Object in_obj = this.parseNextObject(rootNode);

		if (in_obj != null) {
			if (in_obj instanceof Input) {
				Input in = (Input) in_obj;
				DebugPrinter.print("InputParserLink::parse(..) -> Input = "
						+ in, DebugPrinter.INFO);
				this.getContainer().getInputs().addElement(in.getLocalName(),
					in);
			} else {
				DebugPrinter.print("The node starting with " + rootNode
						+ " has not be recognised as an Input !",
					DebugPrinter.WARN);
			}
		} else {
			DebugPrinter.print("The node starting with " + rootNode
					+ " has not be recognised !", DebugPrinter.WARN);

		}

	}

	@Override
	protected String myToString() {
		return super.displayField();
	}

}
