/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser.specializedparsers;

import java.lang.reflect.Field;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.MapContainer;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.SharedLock;

/**
 * This parser is specialised for parsing output when they are part of a capability definition
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 3, 2007 3:09:15 PM
 */
public class OutputInCapabilityParserLink extends AmigoSParserLink {

	/**
	 * The URI of the ontology containing the concept "Output"
	 */
	private static final String MYURI1 = "http://www.hitech-projects.com/euprojects/amigo/Amigo-S.owl#";

	/**
	 * The URI of the ontology containing the concept "ID"
	 */
	private static final String MYURI2 = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

	/** the prefix corresponding to the URI of the ontology MUURI1 */
	private String prefix1 = "";

	/** the prefix corresponding to the URI of the ontology MUURI2 */
	private String prefix2 = "";

	/**
	 * the string corresponding of the localName of the rootNode that this
	 * subparser is responsible
	 */
	private static final String PROCESS_HASOUTPUT = "hasOutput";

	/**
	 * the string corresponding of the localName of the rootNode that this
	 * subparser is responsible
	 */
	private static final String PROCESS_OUTPUT = "Output";

	/** the string corresponding of the localName of the attribute giving an ID */
	private static final String RDF_ID = "ID";

	/**
	 * the string corresponding of the localName of the attribute giving an
	 * reference to an output declared somewhere else
	 */
	private static final String RDF_RESOURCE = "resource";

	/**
	 * the string corresponding of the localName of the node giving the
	 * parameter type of a Output
	 */
	private static final String PROCESS_PARAMETERTYPE = "parameterType";

	/**
	 * @param lock
	 *            the shared lock
	 */
	public OutputInCapabilityParserLink(SharedLock lock) {
		super(lock);
		this.setMyKeyword(this.getCompleteTagName(OutputInCapabilityParserLink.PROCESS_HASOUTPUT));
	}

	/**
	 * @param mp *
	 * @param lock
	 *            the shared lock
	 */
	public OutputInCapabilityParserLink(MapContainer mp, SharedLock lock) {
		super(mp, null, lock);
		this.setMyKeyword(this.getCompleteTagName(OutputInCapabilityParserLink.PROCESS_HASOUTPUT));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.AmigoSParserLink#clone()
	 */
	@Override
	public AmigoSParserLink clone() {
		OutputInCapabilityParserLink clone = new OutputInCapabilityParserLink(
				this.getContainer(), this.getLock());
		clone.prefix1 = prefix1;
		clone.prefix2 = prefix2;
		return clone;
	}

	@Override
	public String[] getURIOfBasesConcept() {
		return new String[] { "prefix1", OutputInCapabilityParserLink.MYURI1,
				"prefix2", OutputInCapabilityParserLink.MYURI2 };
	}

	@Override
	public void setPrefixValue(String prefixName, String prefixValue) {
		try {
			Field fd = this.getClass().getDeclaredField(prefixName);
			fd.set(this, prefixValue);
		} catch (SecurityException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (NoSuchFieldException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		}
		this.setMyKeyword(this.getCompleteTagName(PROCESS_HASOUTPUT));
	}

	/**
	 * Get the string corresponding to the tag to recognise
	 * 
	 * @param concept
	 *            the name of the concept to recognise as tag
	 * @return the complete tag to recognise, <i>empty string</i> if an unknown
	 *         concept is given as parameter
	 */
	private String getCompleteTagName(String concept) {
		if (concept.equals(PROCESS_HASOUTPUT))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_OUTPUT))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_PARAMETERTYPE))
			return this.prefix1 + ":" + concept;
		if (concept.equals(RDF_ID))
			return this.prefix2 + ":" + concept;
		if (concept.equals(RDF_RESOURCE))
			return this.prefix2 + ":" + concept;
		return "";
	}

	/**
	 * This method with extract an OutputObject from the given part of the XML
	 * file
	 * 
	 * @param startingNode
	 *            the node from which we have to search the information
	 * @return the Output object, <b>null</b> if an error occurs
	 */
	private Output extractOutput(Node startingNode) {

		Node child = startingNode;

		// the information to gather to declare the output
		String localName = "";
		String type = "";

		loop1: while (child != null) {

			// get tag Output
			if (child.getNodeName().equals(
				this.getCompleteTagName(PROCESS_OUTPUT))) {
				// look for attributes toi find name
				NamedNodeMap mapOfAttributes = child.getAttributes();
				for (int i = 0; i < mapOfAttributes.getLength(); i++) {
					Node tmp = mapOfAttributes.item(i);

					if (tmp.getNodeName().equals(
						this.getCompleteTagName(RDF_ID))) {
						// get the name of the pointed Output
						localName = tmp.getNodeValue();
					}
				}

				// look for child to get type
				Node subchild = child.getFirstChild();
				while (subchild != null) {
					// we meet the next tag correct following the langage -> get
					// Type
					if (subchild.getNodeName().equals(
						this.getCompleteTagName(PROCESS_PARAMETERTYPE))) {
						type = child.getTextContent().trim();
						break loop1;
					}
					subchild = subchild.getNextSibling();
				}
			}
			child = child.getNextSibling();
		}

		return new Output(localName, type);
	}

	@Override
	protected Object parseNextObject_MethodContent(Node rootNode)
			throws ParsingException {
		NamedNodeMap mapOfAttributes = rootNode.getAttributes();

		// boolean to detect if the output is a declaration or just a
		// reference
		boolean isResource = false;

		// the information to gather to declare the output
		String localName = "";

		// read the attributes to detect the type of declaration
		for (int i = 0; i < mapOfAttributes.getLength(); i++) {
			Node tmp = mapOfAttributes.item(i);

			if (tmp.getNodeName().equals(this.getCompleteTagName(RDF_RESOURCE))) {
				isResource = true;
				// get the name of the pointed Output
				String ref = tmp.getNodeValue();

				// remove the "#" before the name
				localName = ref.substring(1);
			}
		}

		if (isResource) {
			return new Output(IO.UNKNOWN, IO.UNKNOWN, localName);
		} else {
			return this.extractOutput(rootNode.getFirstChild());
		}
	}

	@Override
	protected void parse_MethodContent(Node rootNode) throws ParsingException {
		Object out_obj = this.parseNextObject(rootNode);

		if (out_obj != null) {
			if (out_obj instanceof Output) {
				Output out = (Output) out_obj;
				DebugPrinter.print(
					"OutputInCapabilityParserLink::parse(..) -> Output = "
							+ out, DebugPrinter.INFO);
				this.getContainer().getOutputs().addElement(out.getLocalName(),
					out);
			} else {
				DebugPrinter.print("The node starting with " + rootNode
						+ " has not be recognised as an Output !",
					DebugPrinter.WARN);
			}
		} else {
			DebugPrinter.print("The node starting with " + rootNode
					+ " has not be recognised !", DebugPrinter.WARN);

		}

	}

	@Override
	protected String myToString() {
		return super.displayField();
	}
}
