/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser.specializedparsers;

import java.lang.reflect.Field;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.QoSProperty;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.MapContainer;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.SharedLock;

/**
 * This parser is specialized to collect QoS Information
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 28, 2007 9:41:26 AM
 */
public class QoSParserLink extends AmigoSParserLink {

	/**
	 * the string corresponding of the localName of the rootNode that this
	 * subparser is responsible
	 */
	private static final String AMIGO_HASQOSPROFILE = "hasQoSProfile";

	/**
	 * The URI of the ontology containing the concept "ID"
	 */
	private static final String MYURI1 = "http://www.owl-ontologies.com/Amigo/Amigo.owl#";

	/** the prefix corresponding to the URI of the ontology MUURI1 */
	private String prefix1 = "";

	/** The string use as will be concatened to generate the correct indentation */
	private static final String INDENT = "   ";

	/**
	 * Create a new Parser by giving the object container and its keyword
	 * 
	 * @param mp
	 *            the container of parsed object
	 * @param keyword
	 *            the keyword of which the Parser is responsible *
	 * @param lock
	 *            the shared lock
	 */
	public QoSParserLink(MapContainer mp, String keyword, SharedLock lock) {
		super(mp, keyword, lock);
		this.setMyKeyword(this.getCompleteTagName(QoSParserLink.AMIGO_HASQOSPROFILE));
	}

	/**
	 * Create a new Parser by giving the object container
	 * 
	 * @param mp
	 *            the container of parsed object *
	 * @param lock
	 *            the shared lock
	 */
	public QoSParserLink(MapContainer mp, SharedLock lock) {
		this(mp, null, lock);
	}

	/**
	 * Create a new Parser for QoS *
	 * 
	 * @param lock
	 *            the shared lock
	 */
	public QoSParserLink(SharedLock lock) {
		this(null, null, lock);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.specializedparsers.AmigoSParserLink#clone()
	 */
	@Override
	public AmigoSParserLink clone() {
		QoSParserLink clone = new QoSParserLink(this.getContainer(),
				this.getLock());
		clone.prefix1 = this.prefix1;
		return clone;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.specializedparsers.AmigoSParserLink#getURIOfBasesConcept()
	 */
	@Override
	public String[] getURIOfBasesConcept() {
		return new String[] { "prefix1", QoSParserLink.MYURI1 };
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.specializedparsers.AmigoSParserLink#setPrefixValue(java.lang.String,
	 *      java.lang.String)
	 */
	@Override
	public void setPrefixValue(String prefixName, String prefixValue) {
		try {
			Field fd = this.getClass().getDeclaredField(prefixName);
			fd.set(this, prefixValue);
		} catch (SecurityException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (NoSuchFieldException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		}
		this.setMyKeyword(this.getCompleteTagName(AMIGO_HASQOSPROFILE));

	}

	/**
	 * Get the string corresponding to the tag to recognise
	 * 
	 * @param concept
	 *            the name of the concept to recognise as tag
	 * @return the complete tag to recognise, <i>empty string</i> if an unknown
	 *         concept is given as parameter
	 */
	private String getCompleteTagName(String concept) {
		if (concept.equals(AMIGO_HASQOSPROFILE))
			return this.prefix1 + ":" + concept;

		return "";
	}

	/**
	 * Gather all attributes of a node and add it in a String
	 * 
	 * @param node
	 *            the node from which the attribute will be extracted
	 * @return the String corresponding to the list of Attributes, <i>empty
	 *         string</i> if the node has no attributes.
	 */
	private String gatherAttributes(Node node) {
		String result = "";

		NamedNodeMap mapOfAttributes = node.getAttributes();

		// read the attributes
		for (int i = 0; i < mapOfAttributes.getLength(); i++) {
			Node tmp = mapOfAttributes.item(i);
			result += tmp.getNodeName() + "=\"" + tmp.getNodeValue() + "\" ";
		}

		return result;
	}

	/**
	 * Gather all the XML fragment contained into the children of the given node
	 * 
	 * @param node
	 *            the starting node of the gathering
	 * @param prefix
	 *            the string to put before each line to have a correct
	 *            indentation
	 * @return the String corresponding to the XML content of the node
	 */
	private String gatherText(Node node, String prefix) {
		String result = "";

		// if the current node has some child
		if (node.getFirstChild() != null) {

			result += prefix + "<" + node.getNodeName() + " "
					+ this.gatherAttributes(node) + ">\n";

			// list children and run gatherText recursively
			// look for child to get type
			Node child = node.getFirstChild();
			while (child != null) {
				result += this.gatherText(child, prefix + QoSParserLink.INDENT);
				child = child.getNextSibling();
			}

			result += prefix + "</" + node.getNodeName() + ">\n";

		} else {
			// the current node has no child so just gather attributes, value or
			// text...
			switch (node.getNodeType()) {
			case Node.ATTRIBUTE_NODE:
				break;
			case Node.CDATA_SECTION_NODE:
				result += prefix + "<![CDATA[" + node.getTextContent().trim()
						+ "]]>\n";
				break;
			case Node.COMMENT_NODE:
				result += prefix + "<!-- " + node.getTextContent().trim()
						+ "-->\n";
				break;
			case Node.DOCUMENT_FRAGMENT_NODE:
				break;
			case Node.DOCUMENT_NODE:
				break;
			case Node.DOCUMENT_TYPE_NODE:
				break;
			case Node.ELEMENT_NODE:
				result += prefix + "<" + node.getNodeName() + " "
						+ this.gatherAttributes(node) + "/>\n";
				break;
			case Node.ENTITY_NODE:
				break;
			case Node.ENTITY_REFERENCE_NODE:
				break;
			case Node.NOTATION_NODE:
				break;
			case Node.PROCESSING_INSTRUCTION_NODE:
				break;
			case Node.TEXT_NODE:
				if (!node.getTextContent().trim().equals("")
						&& !node.getTextContent().trim().equals("\n"))
					result += prefix + node.getTextContent().trim() + "\n";
				break;
			default:
				break;
			}
		}

		return result;
	}

	@Override
	protected Object parseNextObject_MethodContent(Node rootNode)
			throws ParsingException {

		DebugPrinter.print(
			"QosParserLink::parseNextObject() --> >>>>>>> QoS Detected <<<<<<<<",
			DebugPrinter.INFO);
		DebugPrinter.print("QosParserLink::parseNextObject() --> NodeText :\n"
				+ rootNode, DebugPrinter.INFO);

		return new QoSProperty(this.gatherText(rootNode, ""));

	}

	@Override
	protected void parse_MethodContent(Node rootNode) throws ParsingException {
		Object qos_obj = this.parseNextObject(rootNode);

		if (qos_obj != null) {
			if (qos_obj instanceof QoSProperty) {
				QoSProperty qos = (QoSProperty) qos_obj;
				DebugPrinter.print("QoSParserLink::parse(..) -> QOS = " + qos,
					DebugPrinter.INFO);

				this.getContainer().setQos(qos);
			} else {
				DebugPrinter.print("The node starting with " + rootNode
						+ " has not be recognised as a QoSProperty !",
					DebugPrinter.WARN);
			}
		} else {
			DebugPrinter.print("The node starting with " + rootNode
					+ " has not be recognised !", DebugPrinter.WARN);

		}

	}

	@Override
	protected String myToString() {
		return super.displayField();
	}

}
