/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser.specializedparsers;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.domparser.Parser;
import fr.inria.icocoa.servicemodel.domparser.ParserFactory;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.MapContainer;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.ParserThread;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.SharedLock;

/**
 * This parser is the master one. All subparser have been though to run a in dedicate thread (so as to be optimised for multi-core engine). This linbk of the chain is the "thread creator". It create new thread for each "high level" tag met, and start a parsing for this tag.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 2, 2007 4:52:14 PM
 */
public class ThreadCreatorParserLink extends AmigoSParserLink {

	/**
	 * Create a new instance by giving the
	 * 
	 * @param mp
	 *            the container of maps used during the parsing *
	 * @param lock
	 *            the shared lock
	 */
	public ThreadCreatorParserLink(MapContainer mp, SharedLock lock) {
		super(mp, lock);
	}

	/**
	 * Default constructor (initialize field to <b>null</b>) *
	 * 
	 * @param lock
	 *            the shared lock
	 * 
	 */
	public ThreadCreatorParserLink(SharedLock lock) {
		super(lock);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.AmigoSParserLink#clone()
	 */
	@Override
	public AmigoSParserLink clone() {

		AmigoSParserLink clone = new ThreadCreatorParserLink(
				this.getContainer(), this.getLock());
		return clone;
	}

	/*
	 * Launch a Thread on each subnode of given rootNode (non-Javadoc) *
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.AmigoSParserLink#parse(org.w3c.dom.Node)
	 */
	@Override
	public void parse(Node rootNode) {
		try {
			// check if the node is not a leaf !
			if (rootNode.hasChildNodes()) {
				// get list of children
				// NodeList listOfChildren = rootNode.getChildNodes();

				// get first child of Output
				Node nextFils = rootNode.getFirstChild();

				// create a list where all started Thread will be add
				List<Thread> listOfThreadToWait = new ArrayList<Thread>();

				/*
				 * for each child, which is an element, create a new Thread
				 * (associate a new parser) and launch it!
				 */
				while (nextFils != null) {

					// test if it is an element
					if (nextFils.getNodeType() == Node.ELEMENT_NODE
							|| nextFils.getNodeType() == Node.DOCUMENT_NODE) {
						// create a new Parser
						Parser p = ParserFactory.getInstanceOfFactory().getInstanceOfParser(
							this.getContainer());

						// DebugPrinter.print("ThreadCreatorParserLink::parse()
						// - parser = \n"+p,DebugPrinter.INFO);

						// create new thread conatining the parser and a ref on
						// the
						// child
						Thread t = new ParserThread(nextFils, p);

						// start the thread (<=> start of parsing the part of
						// XML
						// code)
						t.start();

						// add this thread to the list of thread to wait befor
						// terminaison
						listOfThreadToWait.add(t);

					}

					// update nextFils to continue the loop
					nextFils = nextFils.getNextSibling();
				}

				/* End the terminaison of all thread */
				for (Thread t : listOfThreadToWait)
					t.join();
			} else {
				// nothing to do !
			}
		} catch (InterruptedException e) {
			DebugPrinter.print(e.getStackTrace(), "A part of code ("
					+ rootNode.getNodeName()
					+ " - ) may not be parsed due to following exception : "
					+ e.getMessage(), DebugPrinter.WARN);
		}

	}

	/*
	 * If access this part of the chain of responsability, it means that
	 * recognised none object has been found so return null ! (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.AmigoSParserLink#parseNextObject(org.w3c.dom.Node)
	 */
	@Override
	public Object parseNextObject(Node rootNode) {
		return null;
	}

	@Override
	public String[] getURIOfBasesConcept() {
		// nothing to return, since no tag is recognise by this subparser
		return new String[] {};
	}

	@Override
	public void setPrefixValue(String prefixName, String prefixValue) {
		// nothing to do, since no tag is recognise by this subparser
	}

	@Override
	protected Object parseNextObject_MethodContent(Node rootNode)
			throws ParsingException {
		return null;
	}

	@Override
	protected void parse_MethodContent(Node rootNode) throws ParsingException {
		// nothing to do this method will be never called do to override of
		// methode "parse"
	}

	@Override
	protected String myToString() {
		return super.displayField();
	}

}
