/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser.toolobjects;

import java.util.ArrayList;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import fr.inria.icocoa.servicemodel.Transferable;

/**
 * This class provide an encapsulation of a ConcurrentMap that define some
 * synchronised methods (to create/modify the object contained into the map).
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 27, 2007 4:23:57 PM
 * @param K
 *            the type of data of the key of the map
 * @param V
 *            the type of data that will be contained into the map (value)
 */
public class ConcurrentDataMap<K, V extends Transferable<V>> {

	/** The map that will contains data */
	private ConcurrentHashMap<K, V> objectMap = new ConcurrentHashMap<K, V>();

	/** the object that will permit to lock the modification method */
	private Byte lock = new Byte("0");

	/**
	 * Add an element to the synchronised map
	 * 
	 * @param key
	 *            the key of the object to add
	 * @param value
	 *            the value associated to this key
	 */
	public void addElement(K key, V value) {
		if (objectMap.containsKey(key)) {
			V object = objectMap.get(key);
			// copie one object into the other
			synchronized (lock) {//
				this.compareAndTransfer(object, value);
			}
		} else {
			objectMap.put(key, value);
		}
	}

	/**
	 * Remove an element from the Map
	 * 
	 * @param key :
	 *            the element to remove
	 */
	public void removeElement(K key) {
		objectMap.remove(key);
	}

	@Override
	public String toString() {
		String result = "";
		int i = 0;
		for (Entry<K, V> e : new ArrayList<Entry<K, V>>(objectMap.entrySet())) {
			result += " - " + (i++) + ":[" + e.getKey().toString() + "] = "
					+ e.getValue().toString() + "\n";
		}
		return result;
	}

	/**
	 * Copie information from newValue to currentValue if currentValue contains
	 * less information than newValue
	 * 
	 * @param currentValue
	 *            the potential target object
	 * @param newValue
	 *            the potential source object
	 */
	private void compareAndTransfer(V currentValue, V newValue) {
		// compare the "filling" of object
		int difference = currentValue.containsMoreInformation(newValue);

		// if the currentValue has less information that the previous then copie
		// the one with the more information in the other
		if (difference < 0)
			currentValue.transfertInformation(newValue);
	}

	/**
	 * Get the number of elements contained into the map
	 * 
	 * @return the size of the map
	 */
	public int size() {
		return this.objectMap.size();
	}

	/**
	 * Try to find an object in the map
	 * 
	 * @param valueToFind
	 *            the object to find
	 * @return <i>true</i> if the given object is contained into the map,
	 *         <i>false</i> otherwise.
	 */
	public boolean containsElement(V valueToFind) {
		return this.objectMap.contains(valueToFind);
	}

}
