/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser.toolobjects;

import org.w3c.dom.Node;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.domparser.Parser;

/**
 * This class permit to run a new parser from a node ! This allows a
 * multithreaded parsing;
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 3, 2007 1:08:29 PM
 */
public class ParserThread extends Thread {

	/** The global number of ParserThread */
	private static long threadID = 0;

	/** The global number of active ParserThread */
	private static long activeThreadNumber = 0;

	/** The node from which the parser will start */
	private Node rootNode;

	/** The parser run by this thread */
	private Parser parser;

	/** my ID */
	private final long id;

	/**
	 * the object that will be used as lock to avoid fault when counting the
	 * number of active thread
	 */
	private static Byte lockActiveTheadCounter = new Byte("0");

	/**
	 * the object that will be used as lock to avoid fault when updating the ID
	 * of each thread
	 */
	private static Byte lockTheadID = new Byte("0");

	/**
	 * Initialise a new Thread with the parser and the starting node
	 * 
	 * @param rootNode
	 *            The node from which the parser will start
	 * @param parser
	 *            The parser run by this thread
	 */
	public ParserThread(Node rootNode, Parser parser) {
		/* Update the ID of the thread */
		synchronized (lockTheadID) {
			id = threadID++;
		}

		/* Update field */
		this.rootNode = rootNode;
		this.parser = parser;

		/*
		 * DebugPrinter.print("the ParserThread with ID=" + this.id + " has been
		 * created (active=" + ParserThread.activeThreadNumber + ")" + "\n\t-
		 * Node Name = " + rootNode.getNodeName(), DebugPrinter.INFO);
		 */
	}

	/**
	 * @return the parser
	 */
	public Parser getParser() {
		return parser;
	}

	/**
	 * @param parser
	 *            the parser to set
	 */
	public void setParser(Parser parser) {
		this.parser = parser;
	}

	/**
	 * @return the rootNode
	 */
	public Node getRootNode() {
		return rootNode;
	}

	/**
	 * @param rootNode
	 *            the rootNode to set
	 */
	public void setRootNode(Node rootNode) {
		this.rootNode = rootNode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Thread#run()
	 */
	@Override
	public void run() {
		super.run();

		/* update number of active thread */
		synchronized (lockActiveTheadCounter) {
			ParserThread.activeThreadNumber++;
		}

		/*
		 * DebugPrinter.print("the ParserThread with ID=" + this.id + " has been
		 * started (active=" + ParserThread.activeThreadNumber + ")",
		 * DebugPrinter.INFO);
		 */

		/* execute parsing */
		try {
			this.parser.parse(this.rootNode);
		} catch (ParsingException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.ERROR);
		}

		/* update number of active thread */
		synchronized (lockActiveTheadCounter) {
			ParserThread.activeThreadNumber--;
		}
	}

	/**
	 * @return the activeThreadNumber
	 */
	public static long getActiveThreadNumber() {
		return activeThreadNumber;
	}

	/**
	 * @return the threadID
	 */
	public static long getThreadID() {
		return threadID;
	}

	/**
	 * @return the id
	 */
	@Override
	public long getId() {
		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		super.finalize();
		/*
		 * DebugPrinter.print("the ParserThread with ID=" + this.id + " has been
		 * deleted (active=" + ParserThread.activeThreadNumber + ")",
		 * DebugPrinter.INFO);
		 */
	}

}
