/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.flow;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.ParameterBinding;
import fr.inria.icocoa.servicemodel.Service;

/**
 * 
 * This class permit to store some information about the flow of operation
 * needed to perform the required task
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 26, 2007 2:45:51 PM
 */
public abstract class CapabilityFlow implements Cloneable, Serializable {

	// ////////////////////
	// // Class fields ////
	// ////////////////////

	/**
	 * The list of input data binding corresponding to the Capability
	 */
	protected Map<String, ParameterBinding> inputsBinding = null;

	/**
	 * The list of output data binding corresponding to the Capability
	 */
	protected Map<String, ParameterBinding> outputsBinding = null;

	// /////////////////////
	// // Abstract part ////
	// /////////////////////

	/**
	 * Indicate if the current CapabilityFlow is composite or no
	 * 
	 * @return <i>true</i> if the Flow is composed of only one element,
	 *         <i>false</i> otherwise.
	 */
	public abstract boolean isAtomic();

	/**
	 * This method generate the BPEL script fragment corresponding to the
	 * current Flow
	 * 
	 * @param prefix
	 *            the prefix to write before each generated line (to have a
	 *            correct indented xml-file)
	 * @param service_name
	 *            the map serviceURI=serviceName
	 * @param service_portType
	 *            the map serviceURI=portType
	 * @param service_nameSpace
	 *            the map serviceURI=namespace
	 * @return the XML fragment corresponding to flow
	 */
	public abstract String generateBPELFragment(String prefix,
			Map<String, String> service_name,
			Map<String, String> service_portType,
			Map<String, String> service_nameSpace);

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public abstract String toString();

	/**
	 * Get string representation of the object by putting the prefix string at
	 * each beginning of line
	 * 
	 * @param prefix
	 *            the prefix to put at each beginning of lines
	 * @return the string representation of the object
	 */
	public abstract String toString(String prefix);

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	public abstract CapabilityFlow clone() throws CloneNotSupportedException;

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public abstract boolean equals(Object obj);

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public abstract int hashCode();

	// /////////////////
	// // Real part ////
	// /////////////////

	/**
	 * Default constructor to initialise bindings map
	 * 
	 */
	protected CapabilityFlow() {
		this.inputsBinding = new HashMap<String, ParameterBinding>();
		this.outputsBinding = new HashMap<String, ParameterBinding>();
	}

	/**
	 * @return the inputsBinding
	 */
	public Map<String, ParameterBinding> getInputsBinding() {
		return inputsBinding;
	}

	/**
	 * @param inputsBinding
	 *            the inputsBinding to set
	 */
	public void setInputsBinding(Map<String, ParameterBinding> inputsBinding) {
		this.inputsBinding = inputsBinding;
	}

	/**
	 * @return the outputsBinding
	 */
	public Map<String, ParameterBinding> getOutputsBinding() {
		return outputsBinding;
	}

	/**
	 * @param outputsBinding
	 *            the outputsBinding to set
	 */
	public void setOutputsBinding(Map<String, ParameterBinding> outputsBinding) {
		this.outputsBinding = outputsBinding;
	}

	/**
	 * Add a entry in the future InputBindings Map
	 * 
	 * @param key
	 *            the key associated to the value to add in the map
	 * @param value
	 *            the object to add in the map
	 * @return the previous value associated with specified key, or <i>null</i>
	 *         if there was no mapping for key.
	 */
	public ParameterBinding addInputsBinding(String key, ParameterBinding value) {
		return this.getInputsBinding().put(key, value);
	}

	/**
	 * Add a entry in the future OutputBindings Map
	 * 
	 * @param key
	 *            the key associated to the value to add in the map
	 * @param value
	 *            the object to add in the map
	 * @return the previous value associated with specified key, or <i>null</i>
	 *         if there was no mapping for key.
	 */
	public ParameterBinding addOutputsBinding(String key, ParameterBinding value) {
		return this.getOutputsBinding().put(key, value);
	}

	/**
	 * Update list of Input and Output bindings of the clone, by cloning those
	 * parameter from this object to the clone object
	 * 
	 * @param clone
	 *            the CapabilityFlow instance that will receive the copy of
	 *            parameter binding
	 * @throws CloneNotSupportedException
	 *             if an error occurs during cloning ParameterBinding object
	 */
	protected void copyInputAndOutputBinding(CapabilityFlow clone)
			throws CloneNotSupportedException {

		/* Copy input bindings */
		for (Entry<String, ParameterBinding> inputBinding : this.getInputsBinding().entrySet()) {
			clone.addInputsBinding(inputBinding.getKey(),
				inputBinding.getValue().clone());
		}

		/* Copy output bindings */
		for (Entry<String, ParameterBinding> outputBinding : this.getOutputsBinding().entrySet()) {
			clone.addOutputsBinding(outputBinding.getKey(),
				outputBinding.getValue().clone());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.inputsBinding.clear();
		this.inputsBinding = null;

		this.outputsBinding.clear();
		this.outputsBinding = null;
	}

	/**
	 * Search among Capabilities of a service to check that flow doesn't
	 * contains any FuturCapabilityFlow object. If a Futur capability is found,
	 * then replace it by the updated value. Then restart the process.
	 * 
	 * <p>
	 * The process invariant is : the update of a FuturCapabilityFlow doesn't
	 * contains any FuturCapabilityFlow<br>
	 * The algorithm variant is the number of remaining FuturCapabilityFlow
	 * objects
	 * </p>
	 * 
	 * @param s
	 *            the service object to check.
	 * @throws ParsingException
	 *             if an error occurs during flow construction
	 */
	public static void removeFuturObject(Service s) throws ParsingException {

		/*
		 * initialize the variable that will indicate if any change has been
		 * done during the loop, or need to be done
		 */
		// if we update a FuturCapabilityFlow
		boolean oneChangeDone = false;
		// if it remains some FuturCapabilityFlow
		boolean oneChangeHasToBeDone = false;

		// count the number of iteration
		int numberOfIteration = 0;

		// check capabilities flows
		do {
			// reinitialize the final condition
			oneChangeDone = false;

			// count the number of iteration
			numberOfIteration++;

			// check all sub-capabilities of the service
			for (Capability capa : s.getCapabilities()) {
				try {
					// update the futur object and the result value
					boolean[] result = CapabilityFlow.treatCapability(capa, s,
						oneChangeHasToBeDone);
					oneChangeDone = result[0];
					oneChangeHasToBeDone = result[1];

					// go faster out of the loop (only one change per loop)
					if (oneChangeDone)
						break;
				} catch (CloneNotSupportedException e) {
					DebugPrinter.print(e, DebugPrinter.ERROR);
					throw new ParsingException(e);
				}
			}

			// check only if all sub-capabilities have updated there
			// FuturCapabilityFlow
			if (!oneChangeDone) {
				// check all provided capabilities of the service

				for (Capability capa : s.getProvidedCapabilities()) {
					try {
						// update the futur object and the result value
						boolean[] result = CapabilityFlow.treatCapability(capa,
							s, oneChangeHasToBeDone);
						oneChangeDone = result[0];
						oneChangeHasToBeDone = result[1];

						// go faster out of the loop (only one change per loop)
						if (oneChangeDone)
							break;
					} catch (CloneNotSupportedException e) {
						DebugPrinter.print(e, DebugPrinter.ERROR);
						throw new ParsingException(e);
					}
				}

				// check only if all sub-capabilities and provided capabilities
				// have updated there FuturCapabilityFlow
				if (!oneChangeDone) {
					// check all required capabilities of the service
					for (Capability capa : s.getRequiredCapabilities()) {
						try {

							// update the futur object and the result value
							boolean[] result = CapabilityFlow.treatCapability(
								capa, s, oneChangeHasToBeDone);
							oneChangeDone = result[0];
							oneChangeHasToBeDone = result[1];

							// go faster out of the loop (only one change per
							// loop)
							if (oneChangeDone)
								break;
						} catch (CloneNotSupportedException e) {
							DebugPrinter.print(e, DebugPrinter.ERROR);
							throw new ParsingException(e);
						}
					}
				}

			}

			/* Avoid infinite loops */
			if (numberOfIteration > Integer.MAX_VALUE)
				throw new ParsingException(
						"An infinite loop start during the resolution of FuturObject");
		} while (oneChangeDone || oneChangeHasToBeDone);

	}

	/**
	 * Check if the flow of the capa contains some FuturCapabilityFlow object.
	 * If some are contains, then update them by the up-to-date value.
	 * 
	 * @param capa
	 *            the capability to check
	 * @param s
	 *            the service containing thsi capability
	 * @param _oneChangeHasToBeDone
	 *            indicate if one change has to be done (it remains some
	 *            FuturCapabilityFlow object) (<i>true</i> in this case,
	 *            <i>false</i> otherwise)
	 * @return a table of two boolean :
	 *         <ul>
	 *         <li> <b>return[0]</b> - <i>true</i> if one change has been
	 *         done, <i>false</i> otherwise</li>
	 *         <li> <b>return[1]</b> - <i>true</i> if one change has to be
	 *         done (it remains some FuturCapabilityFlow object), <i>false</i>
	 *         otherwise</li>
	 *         </ul>
	 * @throws ParsingException
	 * @throws CloneNotSupportedException
	 */
	private static boolean[] treatCapability(Capability capa, Service s,
			boolean _oneChangeHasToBeDone) throws ParsingException,
			CloneNotSupportedException {
		CapabilityFlow flow = capa.getFlow();

		// if we update a FuturCapabilityFlow
		boolean oneChangeDone = false;
		// if it remains some FuturCapabilityFlow
		boolean oneChangeHasToBeDone = _oneChangeHasToBeDone;

		// check if there is some FuturCapabilityObject
		if (!flow.isAtomic()) {
			if (flow instanceof FuturCapabilityFlow) {
				// get future object
				FuturCapabilityFlow futur = (FuturCapabilityFlow) flow;

				// update the futur object and the result value
				boolean[] result = CapabilityFlow.updateFuturObject(futur, s,
					_oneChangeHasToBeDone);
				oneChangeDone |= result[0];
				oneChangeHasToBeDone = result[1];

			} else {

				if (flow instanceof CompositeCapabilityFlow) {
					// get composite flow object
					CompositeCapabilityFlow composite = (CompositeCapabilityFlow) flow;

					// check if the composite object contains some
					// FuturCapabilityFlow object
					FuturCapabilityFlow futur = composite.containsFuturCapabilityFlow();

					// if the composite contains such object, then update them
					if (futur != null) {
						// update the futur object and the result value
						boolean[] result = CapabilityFlow.updateFuturObject(
							futur, s, _oneChangeHasToBeDone);
						oneChangeDone |= result[0];
						oneChangeHasToBeDone = result[1];
					}

				}

			}
		}
		return new boolean[] { oneChangeDone, oneChangeHasToBeDone };
	}

	/**
	 * Update the FuturCapabilityFlow object (find the required flow and replace
	 * it)
	 * 
	 * @param futur
	 *            the FuturCapabilityFlow object to replace
	 * @param s
	 *            the service in which the FuturCapabilityFlow object has been
	 *            found
	 * @param _oneChangeHasToBeDone
	 *            indicate if one change has to be done (it remains some
	 *            FuturCapabilityFlow object) (<i>true</i> in this case,
	 *            <i>false</i> otherwise)
	 * @return a table of two boolean :
	 *         <ul>
	 *         <li> <b>return[0]</b> - <i>true</i> if one change has been
	 *         done, <i>false</i> otherwise</li>
	 *         <li> <b>return[1]</b> - <i>true</i> if one change has to be
	 *         done (it remains some FuturCapabilityFlow object), <i>false</i>
	 *         otherwise</li>
	 *         </ul>
	 * @throws ParsingException
	 * @throws CloneNotSupportedException
	 */
	private static boolean[] updateFuturObject(FuturCapabilityFlow futur,
			Service s, boolean _oneChangeHasToBeDone) throws ParsingException,
			CloneNotSupportedException {

		// get required capability
		Capability required = s.getCapabilityFromLocalName(Tools.getSecondPart(futur.getFuturLocalName()));

		if (required == null)
			throw new ParsingException(
					"The required capability (parsed in a conversation) "
							+ futur.getFuturLocalName() + " has not been found");

		// clone the flow
		CapabilityFlow replacer = required.getFlow().clone();

		/*
		 * Check the type of the replacer ! If the replacer is also a Futur
		 * object, don't replace it now ! Wait that the previous Futur has been
		 * replaced.
		 * 
		 */
		if (replacer instanceof AtomicCapabilityFlow)
			// replace the future object
			return new boolean[] {
					futur.replaceByUpToDateCapabilityFlow(replacer),
					(false || _oneChangeHasToBeDone) };

		if (replacer instanceof CompositeCapabilityFlow) {

			// check if the replacer contains some FuturCapabilityFlow ! if it
			// does, we don't make any change because the algotithm invariant is
			// :
			// the update of a FuturCapabilityFlow doesn't contains any
			// FuturCapabilityFlow
			FuturCapabilityFlow remainingFutur = ((CompositeCapabilityFlow) replacer).containsFuturCapabilityFlow();
			if (remainingFutur != null)
				// no change has been done, but some change have to be done
				return new boolean[] { false, true };
			else
				return new boolean[] {
						futur.replaceByUpToDateCapabilityFlow(replacer),
						(false || _oneChangeHasToBeDone) };

		}
		// replacer is a FuturCapabilityFlow
		else
			return new boolean[] { false, true }; // no change has been done,
		// but some change have to
		// be done

	}

}
