/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.flow;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.servicemodel.ControlConstruct;
import fr.inria.icocoa.servicemodel.ParameterBinding;

/**
 * This class implements action that are composed with many other action. Theses
 * other action could be composed themselves.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 26, 2007 3:26:48 PM
 */
public abstract class CompositeCapabilityFlow extends CapabilityFlow {

	/**
	 * The control construct relative to the object that compose this composite
	 */
	private ControlConstruct cc = null;

	/**
	 * Construct a new CompositeCapabilityFlow object by indicating the
	 * ControlConstruct that will manage this part of Flow
	 * 
	 * @param cc
	 *            the ControlConstruct object
	 */
	public CompositeCapabilityFlow(ControlConstruct cc) {
		super();
		this.cc = cc;
	}

	/**
	 * @return the cc
	 */
	public ControlConstruct getCc() {
		return cc;
	}

	/**
	 * @param cc
	 *            the cc to set
	 */
	public void setCc(ControlConstruct cc) {
		this.cc = cc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#isAtomic()
	 */
	@Override
	public boolean isAtomic() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#toString()
	 */
	@Override
	public String toString() {
		return this.toString("");
	}

	// /////////////////////////////////////////
	// //// Composite Design Pattern Part //////
	// /////////////////////////////////////////

	/** List of CapabilityFlow object that compose this composite */
	private List<CapabilityFlow> children = new ArrayList<CapabilityFlow>();

	/**
	 * Add a new component to this composite object
	 * 
	 * @param child
	 *            the CapabilityFlow object to add
	 */
	public void addChild(CapabilityFlow child) {
		this.children.add(child);
	}

	/**
	 * Remove a new component to this composite object
	 * 
	 * @param child
	 *            the CapabilityFlow object to remove
	 */
	public void removeChild(CapabilityFlow child) {
		this.children.remove(child);
	}

	/**
	 * @return the children
	 */
	public List<CapabilityFlow> getChildren() {
		return children;
	}

	/**
	 * @param children
	 *            the children to set
	 */
	public void setChildren(List<CapabilityFlow> children) {
		this.children = children;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.cc = null;
		this.children.clear();
		this.children = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cc == null) ? 0 : cc.hashCode());
		result = prime * result
				+ ((children == null) ? 0 : children.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#addInputsBinding(java.lang.String,
	 *      fr.inria.icocoa.servicemodel.ParameterBinding)
	 */
	@Override
	public ParameterBinding addInputsBinding(String key, ParameterBinding value) {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#addOutputsBinding(java.lang.String,
	 *      fr.inria.icocoa.servicemodel.ParameterBinding)
	 */
	@Override
	public ParameterBinding addOutputsBinding(String key, ParameterBinding value) {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#clone()
	 */
	@Override
	public CapabilityFlow clone() throws CloneNotSupportedException {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#generateBPELFragment(java.lang.String,
	 *      java.util.Map, java.util.Map, java.util.Map)
	 */
	@Override
	public String generateBPELFragment(String prefix,
			Map<String, String> service_name,
			Map<String, String> service_portType,
			Map<String, String> service_nameSpace) {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#toString(java.lang.String)
	 */
	@Override
	public String toString(String prefix) {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof CompositeCapabilityFlow))
			return false;
		final CompositeCapabilityFlow other = (CompositeCapabilityFlow) obj;
		if (cc == null) {
			if (other.cc != null)
				return false;
		} else if (!cc.equals(other.cc))
			return false;
		if (children == null) {
			if (other.children != null)
				return false;
		} else if (!Tools.compareList(children, other.children))
			return false;
		return true;
	}

	/**
	 * Check each children to know if it is a FuturCapabilityFlow Object
	 * 
	 * @return a <i>FuturCapabilityFlow</i> object if the list of children
	 *         contains at least one FuturCapabilityFlow object (return the last
	 *         FuturCapabilityFlow object found) , <i>null</i> otherwise.
	 */
	public FuturCapabilityFlow containsFuturCapabilityFlow() {

		FuturCapabilityFlow lastFound = null;
		for (CapabilityFlow child : this.getChildren()) {
			if (child instanceof FuturCapabilityFlow)
				lastFound = (FuturCapabilityFlow) child;
			if (child instanceof CompositeCapabilityFlow) {
				FuturCapabilityFlow recursiveResult = ((CompositeCapabilityFlow) child).containsFuturCapabilityFlow();
				// return the last FuturCapabilityObject found
				if (recursiveResult != null)
					lastFound = recursiveResult;
			}

		}

		return lastFound;
	}
}
