/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.flow;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import fr.inria.icocoa.common.DebugPrinter;

/**
 * This class could create instances of all subclasses of
 * CompositeCapabilityFlow according the name of the ControlConstruct
 * 
 * <font color="#FF0000">WARNING :</font> if you want to add some new
 * CompositeCapabilityFlow children, you need to update the static part of this
 * class
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Oct 23, 2007 1:18:46 PM
 */
public class CompositeCapabilityFlowFactory {

	/** The Amigo-S keyword for sequence */
	public static String SEQUENCE = "Sequence";

	/** The Amigo-S keyword for choice */
	public static String CHOICE = "choice";

	/** The Amigo-S keyword for if_then_else */
	public static String IF_THEN_ELSE = "if_then_else";

	/** The Amigo-S keyword for split */
	public static String SPLIT = "split";

	/** The Amigo-S keyword for split join */
	public static String SPLIT_JOIN = "split_join";

	/** The Amigo-S keyword for ant order */
	public static String ANY_ORDER = "any_order";

	/** The Amigo-S keyword for repeat while */
	public static String REPEAT_WHILE = "repeat_while";

	/** The Amigo-S keyword for sequence */
	public static String REPEAT_UNTIL = "repeat_until";

	/** The Amigo-S keyword for iterate */
	public static String ITERATE = "iterate";

	/**
	 * Store the association keyword/Classname
	 */
	private static Map<String, Class<? extends CompositeCapabilityFlow>> keywordToClass;

	static {
		keywordToClass = new HashMap<String, Class<? extends CompositeCapabilityFlow>>();

		// add all knowed subclasses of CompositeCapabilityFlow
		keywordToClass.put(CompositeCapabilityFlowFactory.SEQUENCE,
			SequenceAction.class);

	}

	/**
	 * get an instance of CompositeCapabilityFlow, according the passed keyword
	 * 
	 * @param keyword
	 *            the name of the ControlConstruct
	 * @param params
	 *            the list of objects corresponding to parameter that have to be
	 *            used to build the instance of CompositeCapabilityFlow
	 * @return the corresponding CompositeCapabilityFlow, <i>null</i> if an
	 *         error occurs !
	 */
	@SuppressWarnings("unchecked")
	public static CompositeCapabilityFlow getInstance(String keyword,
			Object[] params) {
		// get the class associated to the keyword
		Class<? extends CompositeCapabilityFlow> compositeCapabilityClass = keywordToClass.get(keyword);

		// check if a class is associated to the given keyword
		if (compositeCapabilityClass == null)
			return null;

		// create list of parameter type for the constructor
		Class[] parameterTypes = new Class[params.length];
		for (int i = 0; i < parameterTypes.length; i++) {
			parameterTypes[i] = params[i].getClass();
		}

		try {
			// get the corresponding constructor
			Constructor<? extends CompositeCapabilityFlow> constructor = compositeCapabilityClass.getConstructor(parameterTypes);

			// DebugPrinter.print("constructor = "+constructor,
			// DebugPrinter.INFO);

			// return an instance of corresponding object
			return constructor.newInstance(params);

		} catch (SecurityException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
			return null;
		} catch (NoSuchMethodException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
			return null;
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
			return null;
		} catch (InstantiationException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
			return null;
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
			return null;
		} catch (InvocationTargetException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
			return null;
		}
	}

}
