/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.flow;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import fr.inria.icocoa.common.Tools;

/**
 * This class represents temporary object. Indeed, The workflow is created during the parsing of service. Sometime, workflow reference some other workflow elements that hve not been created yet (that refer some capability not parsed yet). In that case, the missing workflow element is replace by an FuturCapabilityFlow object. At the end of the Service parsing, an algorithm is started to remove all FuturCapabilityFlow object and replace them by the real peace of workflow. this algorithm is multipass so as to be sur to remove all undefined references.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Nov 23, 2007 9:47:13 AM
 */
public class FuturCapabilityFlow extends CapabilityFlow {

	/**
	 * serial version ID
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The list of CapabilityFlow that have a pointer on this object and so that
	 * have to be updated.
	 */
	private List<CompositeCapabilityFlow> referenceOnI = new ArrayList<CompositeCapabilityFlow>();

	/**
	 * The name of the capability that will be replace
	 */
	private String futurLocalName = null;

	/**
	 * Create a new FuturCapabilityFlow and add a capa as new Capability that
	 * point on this CapabilityFlow
	 * 
	 * @param capaLocalName
	 *            the localname of the capability that will replace this Futur
	 *            Object*
	 * @param capaFlow
	 *            a CapabilityFlow object that point on the current object
	 *            (reference will be updated)
	 */
	public FuturCapabilityFlow(String capaLocalName,
			CompositeCapabilityFlow capaFlow) {
		this.futurLocalName = capaLocalName;
		this.addNewReferenceOnI(capaFlow);
	}

	/**
	 * Create a new FuturCapabilityFlow and add a capa as new CapabilityFlow
	 * that point on this CapabilityFlow
	 * 
	 * @param capaLocalName
	 *            the localname of the capability that will replace this Futur
	 *            Object*
	 */
	public FuturCapabilityFlow(String capaLocalName) {
		this(capaLocalName, null);
	}

	/**
	 * Add a capability flow as new Capability that point on this CapabilityFlow
	 * 
	 * @param capaFlow
	 *            a CapabilityFlow object that point on the current object
	 *            (reference will be updated)
	 */
	public void addNewReferenceOnI(CompositeCapabilityFlow capaFlow) {
		if (capaFlow != null) {
			this.referenceOnI.add(capaFlow);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#clone()
	 */
	@Override
	public CapabilityFlow clone() throws CloneNotSupportedException {
		FuturCapabilityFlow clone = new FuturCapabilityFlow(
				this.getFuturLocalName());

		/* update list of reference */
		for (CompositeCapabilityFlow capaFlow : this.getReferenceOnI()) {
			clone.addNewReferenceOnI(capaFlow);
		}

		/* Update list of Parameters Bindings */
		this.copyInputAndOutputBinding(clone);

		return clone;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#generateBPELFragment(java.lang.String,
	 *      java.util.Map, java.util.Map, java.util.Map)
	 */
	@Override
	public String generateBPELFragment(String prefix,
			Map<String, String> service_name,
			Map<String, String> service_portType,
			Map<String, String> service_nameSpace) {

		return "An error occurs during workflow generation since no FuturCapabilityFlow instance have to remain during BPEL generation part.";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#isAtomic()
	 */
	@Override
	public boolean isAtomic() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#toString()
	 */
	@Override
	public String toString() {
		return this.toString("");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#toString(java.lang.String)
	 */
	@Override
	public String toString(String prefix) {
		String result = prefix
				+ "An error occurs during workflow generation since no FuturCapabilityFlow instance have to remain during BPEL generation part.\n";
		result += prefix + "\tRequired: " + this.getFuturLocalName() + "\n";
		result += prefix + "\tThere are " + this.getReferenceOnI().size()
				+ " references on the object.\n";
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((futurLocalName == null) ? 0 : futurLocalName.hashCode());
		result = prime * result
				+ ((referenceOnI == null) ? 0 : referenceOnI.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof FuturCapabilityFlow))
			return false;
		final FuturCapabilityFlow other = (FuturCapabilityFlow) obj;
		if (futurLocalName == null) {
			if (other.futurLocalName != null)
				return false;
		} else if (!futurLocalName.equals(other.futurLocalName))
			return false;
		if (referenceOnI == null) {
			if (other.referenceOnI != null)
				return false;
		} else if (!Tools.compareList(referenceOnI, other.referenceOnI))
			return false;
		return true;
	}

	/**
	 * @return the futurLocalName
	 */
	public String getFuturLocalName() {
		return futurLocalName;
	}

	/**
	 * @param futurLocalName
	 *            the futurLocalName to set
	 */
	public void setFuturLocalName(String futurLocalName) {
		this.futurLocalName = futurLocalName;
	}

	/**
	 * @return the referenceOnI
	 */
	public List<CompositeCapabilityFlow> getReferenceOnI() {
		return referenceOnI;
	}

	/**
	 * @param referenceOnI
	 *            the referenceOnI to set
	 */
	public void setReferenceOnI(List<CompositeCapabilityFlow> referenceOnI) {
		this.referenceOnI = referenceOnI;
	}

	/**
	 * Replace this object by an up-to-date one (<=> reference has been solve,
	 * so replace this object by the correct one) Copy data binding from this
	 * object to the up-to-date one
	 * 
	 * @param capaFlow
	 *            the capaFlow object that will replace the current one
	 * @return <i>true</i> if one change has been done, <i>false</i> otherwise
	 * @throws CloneNotSupportedException
	 *             if an error occurs when creating clone of ParameterBinding
	 *             objects
	 */
	public boolean replaceByUpToDateCapabilityFlow(CapabilityFlow capaFlow)
			throws CloneNotSupportedException {
		// Clone the data bindings parameters and associate them to capaFlow
		// object
		this.copyInputAndOutputBinding(capaFlow);

		// if we update a FuturCapabilityFlow
		boolean oneChangeDone = false;

		// get all object that have reference on it and replace reference on
		// this object by reference to capaFlow object
		for (CompositeCapabilityFlow ccFlow : this.getReferenceOnI()) {
			// get list of children of the composite flow that refer the current
			// object
			// -> This list of object contains a ref to the current object !
			// -> We need to get the index of the current object in the list and
			// overwrite it
			List<CapabilityFlow> children = ccFlow.getChildren();

			// get the index of the current object
			int index = children.indexOf(this);

			// if no Futur Object is found in the list of futur, it means that
			// the Futur object has been replaced
			if (index != -1) {

				// add the new Capaflow at index+1
				children.add(index + 1, capaFlow);

				// delete the useless element
				children.remove(index);

				// return that one change has been done
				oneChangeDone = true;
			}
		}

		return oneChangeDone;
	}

}
