/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.flow;

import java.util.Map;
import java.util.Map.Entry;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.ParameterBinding;

/**
 * 
 * This class implement CapabilityFlow that is composoed of only one Capability
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 26, 2007 2:51:34 PM
 */
public class InvokeCapabilityAction extends AtomicCapabilityFlow {

	/**
	 * serial version ID
	 */
	private static final long serialVersionUID = 1L;

	/** the associated capability */
	private Capability capability = null;

	/**
	 * Create the new object
	 * 
	 * @param capa
	 *            the Capability associated to this node.
	 */
	public InvokeCapabilityAction(Capability capa) {
		this.capability = capa;
	}

	/**
	 * @return the capability
	 */
	public Capability getCapability() {
		return capability;
	}

	/**
	 * @param capability
	 *            the capability to set
	 */
	public void setCapability(Capability capability) {
		this.capability = capability;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((capability == null) ? 0 : capability.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final InvokeCapabilityAction other = (InvokeCapabilityAction) obj;
		if (capability == null) {
			if (other.capability != null)
				return false;
		} else if (!capability.equals(other.capability))
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#toString(java.lang.String)
	 */
	@Override
	public String toString(String prefix) {
		String result = prefix + "- [";

		// inputs bindings
		for (Entry<String, ParameterBinding> input : this.getInputsBinding().entrySet()) {
			// DebugPrinter.print("ParameterBinding =
			// "+input.getValue(),DebugPrinter.INFO);
			result += input.getValue().getSourceVariable() + "->"
					+ input.getValue().getTargetVariable() + ",";
		}

		// capability UTI
		result += "] " + this.getCapability().getCapabilityURI() + " [";

		// outputs bindings
		for (Entry<String, ParameterBinding> output : this.getOutputsBinding().entrySet()) {
			// DebugPrinter.print("ParameterBinding =
			// "+output.getValue(),DebugPrinter.INFO);
			result += output.getValue().getSourceVariable() + "->"
					+ output.getValue().getTargetVariable() + ",";
		}

		result += "]\n";

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#clone()
	 */
	@Override
	public InvokeCapabilityAction clone() throws CloneNotSupportedException {
		InvokeCapabilityAction clone = new InvokeCapabilityAction(
				this.getCapability());

		// clone InputsBindings
		for (Entry<String, ParameterBinding> entry : this.getInputsBinding().entrySet())
			clone.getInputsBinding().put(entry.getKey(),
				entry.getValue().clone());

		// clone OutputsBindings
		for (Entry<String, ParameterBinding> entry : this.getOutputsBinding().entrySet())
			clone.getOutputsBinding().put(entry.getKey(),
				entry.getValue().clone());

		return clone;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#generateBPELFragment(java.lang.String,
	 *      java.util.Map, java.util.Map, java.util.Map)
	 */
	@Override
	public String generateBPELFragment(String prefix,
			Map<String, String> service_name,
			Map<String, String> service_portType,
			Map<String, String> service_nameSpace) {
		String bpel_str = "";

		String serviceBase = this.getCapability().getService().getBase();

		/*
		 * Invoke the service
		 */
		bpel_str += prefix + "<!-- Invoke the service "
				+ this.getCapability().getService().getLocalName() + " -->\n";
		bpel_str += prefix + "<invoke \n" + prefix + "\tpartnerLink=\""
				+ service_name.get(serviceBase) + "\"\n";
		bpel_str += prefix + "\tportType=\""
				+ service_nameSpace.get(serviceBase) + ":"
				+ service_portType.get(serviceBase) + "\"\n";
		bpel_str += prefix
				+ "\toperation=\""
				+ Tools.getSecondPart(this.getCapability().getGrounding().getOperationName())
				+ "\"\n";

		// the input variable
		bpel_str += prefix
				+ "\tinputVariable=\""
				+ Tools.getSecondPart(this.getCapability().getGrounding().getWsdlInputMessage())
				+ "_" + service_nameSpace.get(serviceBase) + "_Variable\"\n";

		// the output variable
		bpel_str += prefix
				+ "\toutputVariable=\""
				+ Tools.getSecondPart(this.getCapability().getGrounding().getWsdlOutputMessage())
				+ "_" + service_nameSpace.get(serviceBase) + "_Variable\">\n";

		bpel_str += prefix + "</invoke>\n\n";

		return bpel_str;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.capability = null;
	}

}
