/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.matching;

import java.util.ArrayList;
import java.util.List;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.servicemodel.Capability;

/**
 * This class describes a mapping from an abstract, required capability, to a
 * concrete, provided capability.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * @author <a href="mailto:Graham.Thomson@inria.fr">Graham Thomson</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : May 24, 2007 10:59:46 AM
 */
public class CapabilityMatch {

	/**
	 * the capability of the user task
	 */
	private Capability taskCapability = null;

	/**
	 * the equivalent Capability (relative to a Service)
	 */
	private Capability serviceCapability = null;

	/**
	 * The mapping list of inputs
	 */
	private List<IOMatch> inputsMapping = new ArrayList<IOMatch>();

	/**
	 * The mapping list of inputs
	 */
	private List<IOMatch> outputsMapping = new ArrayList<IOMatch>();

	/** The type of matching between the two concept */
	private short typeOfMatching = MatchingType.FAIL;

	/**
	 * Construct an CapablityMatching object
	 * 
	 * @param taskCapability
	 *            the Capability in the UserTask
	 * @param serviceCapability
	 *            the corresponding Capability (relative to a service)
	 * @param matchingType
	 *            the type of the match between the two concept
	 */
	public CapabilityMatch(Capability taskCapability,
			Capability serviceCapability, short matchingType) {
		this.serviceCapability = serviceCapability;
		this.taskCapability = taskCapability;
		this.typeOfMatching = matchingType;
	}

	/**
	 * Add an io matching to the list of matching io (separate input and output)
	 * 
	 * @param io :
	 *            the IOMatch object to add
	 * @return <i>true</i> if no error occurs, <i>false</i> otherwise
	 */
	public boolean addIOMatching(IOMatch io) {
		switch (io.getTypeOfIO()) {
		case IOMatch.INPUT:
			return this.inputsMapping.add(io);
		case IOMatch.OUTPUT:
			return this.outputsMapping.add(io);
		default:
			return false;
		}
	}

	/**
	 * 
	 * @return the type of the IO
	 */
	public String getType() {
		return this.getOntology() + "#" + this.getTypeName();
	}

	/**
	 * @return the inputsMapping
	 */
	public List<IOMatch> getInputsMapping() {
		return inputsMapping;
	}

	/**
	 * @return the ontology
	 */
	public String getOntology() {
		return this.getTaskCapability().getOntology();
	}

	/**
	 * @return the outputsMapping
	 */
	public List<IOMatch> getOutputsMapping() {
		return outputsMapping;
	}

	/**
	 * @return the serviceLocalName
	 */
	public String getServiceCapabilityLocalName() {
		return this.getServiceCapability().getLocalName();
	}

	/**
	 * @return the taskLocalName
	 */
	public String getTaskCapabilityLocalName() {
		return this.getTaskCapability().getLocalName();
	}

	/**
	 * @return the typeName
	 */
	public String getTypeName() {
		return this.getTaskCapability().getTypeName();
	}

	/**
	 * @return the typeOfMatching
	 */
	public short getTypeOfMatching() {
		return typeOfMatching;
	}

	/**
	 * @param typeOfMatching
	 *            the typeOfMatching to set
	 */
	public void setTypeOfMatching(short typeOfMatching) {
		this.typeOfMatching = typeOfMatching;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME
				* result
				+ ((this.getInputsMapping() == null) ? 0 : this
						.getInputsMapping().hashCode());
		result = PRIME
				* result
				+ ((this.getOntology() == null) ? 0 : this.getOntology()
						.hashCode());
		result = PRIME
				* result
				+ ((this.getOutputsMapping() == null) ? 0 : this
						.getOutputsMapping().hashCode());
		result = PRIME
				* result
				+ ((this.getServiceCapabilityLocalName() == null) ? 0 : this
						.getServiceCapabilityLocalName().hashCode());
		result = PRIME
				* result
				+ ((this.getTaskCapabilityLocalName() == null) ? 0 : this
						.getTaskCapabilityLocalName().hashCode());
		result = PRIME
				* result
				+ ((this.getTypeName() == null) ? 0 : this.getTypeName()
						.hashCode());
		result = PRIME * result + this.getTypeOfMatching();
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final CapabilityMatch other = (CapabilityMatch) obj;
		if (this.getInputsMapping() == null) {
			if (other.getInputsMapping() != null)
				return false;
		} else if (!Tools.compareList(this.getInputsMapping(), other
				.getInputsMapping()))
			return false;
		if (this.getOntology() == null) {
			if (other.getOntology() != null)
				return false;
		} else if (!this.getOntology().equals(other.getOntology()))
			return false;
		if (this.getOutputsMapping() == null) {
			if (other.getOutputsMapping() != null)
				return false;
		} else if (!Tools.compareList(this.getOutputsMapping(), other
				.getOutputsMapping()))
			return false;
		if (this.getServiceCapabilityLocalName() == null) {
			if (other.getServiceCapabilityLocalName() != null)
				return false;
		} else if (!this.getServiceCapabilityLocalName().equals(
				other.getServiceCapabilityLocalName()))
			return false;
		if (this.getTaskCapabilityLocalName() == null) {
			if (other.getTaskCapabilityLocalName() != null)
				return false;
		} else if (!this.getTaskCapabilityLocalName().equals(
				other.getTaskCapabilityLocalName()))
			return false;
		if (this.getTypeName() == null) {
			if (other.getTypeName() != null)
				return false;
		} else if (!this.getTypeName().equals(other.getTypeName()))
			return false;
		if (this.getTypeOfMatching() != other.getTypeOfMatching())
			return false;
		return true;
	}

	@Override
	public String toString() {
		String result = "\t\tFunctionalityMatching : ";
		switch (this.getTypeOfMatching()) {
		case MatchingType.FAIL:
			result += " [FAIL]";
			break;
		case MatchingType.EXACT:
			result += " [EXACT]";
			break;
		case MatchingType.SUBSUMES:
			result += " [SUBSUMES]";
			break;
		case MatchingType.PLUGIN:
			result += " [PLUGIN]";
			break;
		default:
			result += " [FAIL]";
			break;
		}
		result += " \n\t\t\tType = " + this.getType()
				+ "\n\t\t\tService LocalName = "
				+ this.getServiceCapabilityLocalName()
				+ "\n\t\t\tTask LocalName = "
				+ this.getTaskCapabilityLocalName() + "\n\t\t\tInputs = ";

		for (int i = 0; i < this.getInputsMapping().size(); i++)
			result += this.getInputsMapping().get(i).toString();

		result += "\n\t\t\tOutputs = ";
		for (int i = 0; i < this.getOutputsMapping().size(); i++)
			result += this.getOutputsMapping().get(i).toString();

		return result + "\n";
	}

	/**
	 * Check if the cm object is equivalent to the current one, without
	 * considerint Input and Outpur
	 * 
	 * @param other
	 *            the object to compare with
	 * @return <i>true</i> if both objects are equivalent, <i>false</i>
	 *         otherwise
	 */
	public boolean lightEquals(CapabilityMatch other) {

		if (this.getOntology() == null) {
			if (other.getOntology() != null)
				return false;
		} else if (!this.getOntology().equals(other.getOntology()))
			return false;

		if (this.getServiceCapabilityLocalName() == null) {
			if (other.getServiceCapabilityLocalName() != null)
				return false;
		} else if (!this.getServiceCapabilityLocalName().equals(
				other.getServiceCapabilityLocalName()))
			return false;
		if (this.getTaskCapabilityLocalName() == null) {
			if (other.getTaskCapabilityLocalName() != null)
				return false;
		} else if (!this.getTaskCapabilityLocalName().equals(
				other.getTaskCapabilityLocalName()))
			return false;
		if (this.getTypeName() == null) {
			if (other.getTypeName() != null)
				return false;
		} else if (!this.getTypeName().equals(other.getTypeName()))
			return false;
		if (this.getTypeOfMatching() != other.getTypeOfMatching())
			return false;
		return true;
	}

	/**
	 * @return the serviceCapability
	 */
	public Capability getServiceCapability() {
		return serviceCapability;
	}

	/**
	 * @return the taskCapability
	 */
	public Capability getTaskCapability() {
		return taskCapability;
	}

	/**
	 * Check if the match is exact (that means that all IOMatch are exact and
	 * the capabilityMatch is exact)
	 * 
	 * @return <i>true</i> if the match is exact, <i>false</i> otherwise
	 */
	public boolean isExact() {
		// check capability level
		if (this.getTypeOfMatching() == MatchingType.EXACT) {
			// check input level
			for (IOMatch inputmatch : this.getInputsMapping()) {
				if (!inputmatch.isExact())
					return false;
			}

			// check output level
			for (IOMatch outputmatch : this.getOutputsMapping()) {
				if (!outputmatch.isExact())
					return false;
			}

			// all io matching are exact
			return true;

		} else
			return false;
	}

	/**
	 * Check if the match is inclusive (that means that if the IOMatch is for
	 * input, the match is exact or subsum ; and exact or plugin for output ; and the capabilityMatch is exact or subsum)
	 * 
	 * @return <i>true</i> if the match is inclusive, <i>false</i> otherwise
	 */
	public boolean isInclusive() {
		short typeCapa = this.getTypeOfMatching();
		// check capability level
		if (typeCapa == MatchingType.EXACT || typeCapa == MatchingType.SUBSUMES) {
			// check input level
			for (IOMatch inputmatch : this.getInputsMapping()) {
				if (!inputmatch.isInclusive())
					return false;
			}

			// check output level
			for (IOMatch outputmatch : this.getOutputsMapping()) {
				if (!outputmatch.isInclusive())
					return false;
			}

			// all io matching are exact
			return true;

		} else
			return false;
	}

}
