/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.matching;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.semantictype.Code;
import fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;
import fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.EncodedOntologiesManager;

/**
 * This matcher use the encoded ontology and the dedicated method isExact
 * isSubsum and isPlugin define in class
 * {@link fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType}.
 * 
 * Be carefull by using the method
 * {@link #ioMatch(java.lang.String, java.lang.String)}. This method get the
 * encoded value of the concept. This encoded value refer to the last file
 * parsed (service description or task description) if the required ontology has
 * not been already encoded. <br>
 * This matcher always use the first code of the list of codes of SemanticType
 * to return result.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 15, 2008 3:17:48 PM
 */
public class GenericEncodedOntologyMatcher extends Matcher {

	/**
	 * default constructor
	 * 
	 */
	public GenericEncodedOntologyMatcher() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#capabilityMatch(fr.inria.icocoa.servicemodel.Capability,
	 *      fr.inria.icocoa.servicemodel.Capability)
	 */
	@Override
	public CapabilityMatch capabilityMatch(Capability serviceCapa,
			Capability taskCapa) throws SemanticTypeNotKnowException {
		// increase the counter of Capabilities matched
		stats[1]++;

		// get Type
		EncodedSemanticType serviceCapaSemanticType = (EncodedSemanticType) serviceCapa
				.getSemanticType();
		EncodedSemanticType taskCapaSemanticType = (EncodedSemanticType) taskCapa
				.getSemanticType();

		if (serviceCapaSemanticType != null && taskCapaSemanticType != null) {

			// test relations
			if (serviceCapaSemanticType.isEquals(taskCapaSemanticType))
				return this.generateMatching(serviceCapa, taskCapa,
						MatchingType.EXACT);
			if (serviceCapaSemanticType.isSubsum(taskCapaSemanticType))
				return this.generateMatching(serviceCapa, taskCapa,
						MatchingType.PLUGIN);
			if (serviceCapaSemanticType.isPlugin(taskCapaSemanticType))
				return this.generateMatching(serviceCapa, taskCapa,
						MatchingType.SUBSUMES);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#ioMatch(java.lang.String,
	 *      java.lang.String)
	 */
	@Override
	public int ioMatch(String serviceIOSemanticType, String taskIOsemanticType)
			throws SemanticTypeNotKnowException {
		// increase the counter of IO matched
		stats[2]++;

		EncodedOntologiesManager manager = EncodedOntologiesManager
				.getInstance();
		List<Code> serviceIOCodes = manager.getEncryptedObject(Tools
				.getSecondPart(serviceIOSemanticType), Tools
				.getFirstPart(serviceIOSemanticType));
		List<Code> taskIOCodes = manager.getEncryptedObject(Tools
				.getSecondPart(taskIOsemanticType), Tools
				.getFirstPart(taskIOsemanticType));

		if (serviceIOCodes != null && taskIOCodes != null) {
			if (!serviceIOCodes.isEmpty() && !taskIOCodes.isEmpty()) {
				Code serviceIOCode = serviceIOCodes.get(0);
				Code taskIOCode = taskIOCodes.get(0);
				if (serviceIOCode.isExact(taskIOCode))
					return MatchingType.EXACT;
				if (serviceIOCode.isPlugin(taskIOCode))
					return MatchingType.SUBSUMES;
				if (serviceIOCode.isSubsum(taskIOCode))
					return MatchingType.PLUGIN;
			}
		}
		return MatchingType.FAIL;
	}
	
	
	
	

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#typeMatch(fr.inria.icocoa.servicemodel.semantictype.SemanticType, fr.inria.icocoa.servicemodel.semantictype.SemanticType)
	 */
	@Override
	public short typeMatch(SemanticType serviceType, SemanticType taskType) throws SemanticTypeNotKnowException {
		EncodedOntologiesManager manager = EncodedOntologiesManager
		.getInstance();
List<Code> serviceIOCodes = manager.getEncryptedObject(serviceType);
List<Code> taskIOCodes = manager.getEncryptedObject(taskType);

if (serviceIOCodes != null && taskIOCodes != null) {
	if (!serviceIOCodes.isEmpty() && !taskIOCodes.isEmpty()) {
		Code serviceIOCode = serviceIOCodes.get(0);
		Code taskIOCode = taskIOCodes.get(0);
		if (serviceIOCode.isExact(taskIOCode))
			return MatchingType.EXACT;
		if (serviceIOCode.isPlugin(taskIOCode))
			return MatchingType.SUBSUMES;
		if (serviceIOCode.isSubsum(taskIOCode))
			return MatchingType.PLUGIN;
	}
}
return MatchingType.FAIL;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#ioMatch(fr.inria.icocoa.servicemodel.IO,
	 *      fr.inria.icocoa.servicemodel.IO)
	 */
	@Override
	public IOMatch ioMatch(IO serviceIO, IO taskIO)
			throws SemanticTypeNotKnowException {
		// increase the counter of IO matched
		stats[2]++;

		EncodedSemanticType serviceIOType = (EncodedSemanticType) serviceIO
				.getSemanticType();
		EncodedSemanticType taskIOType = (EncodedSemanticType) taskIO
				.getSemanticType();

		if (serviceIO.getClass().equals(taskIO.getClass())) {
			if (serviceIOType != null && taskIOType != null) {
				if (serviceIOType.isEquals(taskIOType))
					return new IOMatch(taskIO, serviceIO, MatchingType.EXACT);
				if (serviceIOType.isPlugin(taskIOType))
					return new IOMatch(taskIO, serviceIO, MatchingType.SUBSUMES);
				if (serviceIOType.isSubsum(taskIOType))
					return new IOMatch(taskIO, serviceIO, MatchingType.PLUGIN);
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.matching.Matcher#serviceMatch(fr.inria.icocoa.servicemodel.Service,
	 *      fr.inria.icocoa.servicemodel.UserTask)
	 */
	@Override
	public ServiceMatch serviceMatch(Service service, UserTask userTask)
			throws SemanticTypeNotKnowException {
		// increase the counter of Services matched
		stats[0]++;

		ServiceMatch result = this.partialExactMatch(service, userTask);

		// if no matching
		if (result == null)
			return null;

		// test the number of Functionality matching (all required need to me
		// matched)
		if (result.getCapabilityMatches().size() < (userTask
				.getRequired_Capabilities().size() + userTask
				.getCapabilitiesMap().size()))
			return null;

		return result;
	}

	/**
	 * Generate the complete FunctionalityMatching object corresponding to the
	 * matching between this Capability and the one give as parameter
	 * 
	 * @param serviceCapa
	 *            the current Capability
	 * @param taskCapa
	 *            the capability to match with.
	 * @param matchingType
	 *            the type of matching (cf MatchingType)
	 * @return a MatchingCapabibility Object containing all information, if the
	 *         the matching is possible, <i>null</i> otherwise.
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	private CapabilityMatch generateMatching(Capability serviceCapa,
			Capability taskCapa, short matchingType)
			throws SemanticTypeNotKnowException {
		
		//DebugPrinter.print("match "+serviceCapa+"\n-- with --\n"+taskCapa, DebugPrinter.INFO);

		// generate IOMatching
		List<IOMatch> iomatching = this.ioMatch(serviceCapa, taskCapa);

		// if no io matching
		if (iomatching == null)
			return null;

		// create the matching at Capability level
		CapabilityMatch result = new CapabilityMatch(taskCapa, serviceCapa,
				matchingType);

		// add matching at io level
		for (Iterator<IOMatch> it = iomatching.iterator(); it.hasNext();)
			result.addIOMatching(it.next());

		return result;
	}

	/**
	 * Try to match the IO of the given capabilities with this (used as UserTask
	 * capability)
	 * 
	 * @param serviceCapa :
	 *            the current Capability
	 * @param taskCapa :
	 *            the capability to match with.
	 * @return the List of matching (at IO level), <i>null</i> if we can't
	 *         match (not the the same type or not the same number)
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	private List<IOMatch> ioMatch(Capability serviceCapa, Capability taskCapa)
			throws SemanticTypeNotKnowException {
		// Map<IO, IO> matchs = new HashMap<IO, IO>();

		List<IOMatch> listOfMatchs = new ArrayList<IOMatch>();

		/*
		 * Match Inputs (the capa (service capa) could have less inputs the
		 * Usertask)
		 */
		if (taskCapa.getInputs().size() <= serviceCapa.getInputs().size()) {

			/* For each input of selected service capability... */
			for (Iterator<Input> it = taskCapa.getInputs().values().iterator(); it
					.hasNext();) {
				Input toFind = it.next();

				// String type = toFind.getType();
				boolean found = false;

				/*
				 * ...Look if there is an corresponding input for task
				 * capability
				 */
				loopServiceInput: for (Iterator<Input> it2 = serviceCapa
						.getInputs().values().iterator(); !found
						&& it2.hasNext();) {
					Input current = it2.next();

					// test matching between task IO "toFind" and service IO
					// "current"
					IOMatch tmpMatch = this.ioMatch(current, toFind);
					if (tmpMatch != null) {
						listOfMatchs.add(tmpMatch);

						// stop the loop to speed up the process
						found = true;
						break loopServiceInput;
					}

				}
				if (!found) {
					DebugPrinter.print("The input "+toFind.getLocalName()+" ("+toFind.getSemanticType()+") has not been found !",DebugPrinter.WARN);
					return null;
				}
			}
		} else {
			return null;
		}

		/*
		 * Match output (the capa (service capa) could have more outputs the
		 * Usertask)
		 */
		if (taskCapa.getOutputs().size() >= serviceCapa.getOutputs().size()) {

			/* For each output of usertask capability... */
			for (Iterator<Output> it = serviceCapa.getOutputs().values()
					.iterator(); it.hasNext();) {
				Output toFind = it.next();

				// String type = toFind.getType();
				boolean found = false;
				/*
				 * ...Look if there is an corresponding input for selected
				 * service capability
				 */
				loopTaskOutput: for (Iterator<Output> it2 = taskCapa
						.getOutputs().values().iterator(); it2.hasNext()
						&& !found;) {
					Output current = it2.next();

					// test matching between service IO "toFind" and task IO
					// "current"
					IOMatch tmpMatch = this.ioMatch(toFind, current);
					if (tmpMatch != null) {
						listOfMatchs.add(tmpMatch);

						// stop the loop to speed up the process
						found = true;
						break loopTaskOutput;
					}

				}
				if (!found) {
					DebugPrinter.print("The output "+toFind.getLocalName()+" ("+toFind.getSemanticType()+") has not been found !",DebugPrinter.WARN);
					return null;
				}
			}
		} else {
			return null;
		}
		// return matchs;
		return listOfMatchs;
	}

	/**
	 * Get a partial matching between the task and the current service
	 * 
	 * @param service :
	 *            the current service
	 * @param userTask :
	 *            the task to match with
	 * @return a ServiceMatch object containing the matching information; if
	 *         some capabilities could be matched, <i>null</i> if no matching
	 *         is possible
	 * @throws SemanticTypeNotKnowException
	 *             if a type is not knowed
	 */
	private ServiceMatch partialExactMatch(Service service, UserTask userTask)
			throws SemanticTypeNotKnowException {

		ServiceMatch result = new ServiceMatch(userTask, service,
				MatchingType.EXACT);

		/* Get the required capabilities */
		List<Capability> required = userTask.getRequiredCapabilities();

		// check all required capability
		for (Iterator<Capability> it = required.iterator(); it.hasNext();) {
			Capability required_capa = it.next();

			List<String> equivalents = this.listEquivalentConcept(required_capa
					.getType());

			for (String equivalent : equivalents) {
				// match with all capability with the same/equivalent type
				List<Capability> sameType = service
						.getCapabilityFromTypeName(equivalent);
				/*DebugPrinter.print(sameType,
						"same Type - required // equivalent = " + equivalent,
						DebugPrinter.INFO);*/

				// try to match at CapabilityLevel (<=> test io)
				for (Iterator<Capability> it2 = sameType.iterator(); it2
						.hasNext();) {
					Capability service_capa = it2.next();
					CapabilityMatch cm = service_capa
							.match(required_capa, this);
					if (cm != null)
						result.addCapabilityMatch(cm);
				}
			}
		}

		/* Get the other capabilities */
		List<Capability> subcapabilties = userTask.getCapabilities();

		// check other capability
		for (Iterator<Capability> it = subcapabilties.iterator(); it.hasNext();) {
			Capability subcapa = it.next();

			List<String> equivalents = this.listEquivalentConcept(subcapa
					.getType());

			for (String equivalent : equivalents) {
				// match with all capability with the same/equivalent type
				List<Capability> sameType = service
						.getCapabilityFromTypeName(equivalent);
				DebugPrinter.print(sameType,
						"same Type - other // equivalent = " + equivalent,
						DebugPrinter.INFO);

				// try to match at CapabilityLevel (<=> test io)
				for (Iterator<Capability> it2 = sameType.iterator(); it2
						.hasNext();) {
					Capability service_capa = it2.next();
					CapabilityMatch cm = service_capa.match(subcapa, this);
					if (cm != null)
						result.addCapabilityMatch(cm);
				}
			}
		}

		// if no matching
		if (result.getCapabilityMatches().size() == 0)
			return null;

		return result;
	}

	/**
	 * List the equivalent concept of the one given as parameter. This one is
	 * add at the beginning of the list
	 * 
	 * @param type
	 *            the complete type of the concept that we which an equivalent
	 *            (ontology#concept)
	 * @return the list of equibalent type, starting with the one given as
	 *         parameter
	 * @throws SemanticTypeNotKnowException
	 *             if the type has not been found in the ontology
	 */
	private List<String> listEquivalentConcept(String type)
			throws SemanticTypeNotKnowException {
		// get Ontology class
		EncodedOntologiesManager manager = EncodedOntologiesManager
				.getInstance();
		return manager.listEquivalentConcept(type);
	}

}
