/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.matching;

import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;

/**
 * 
 * This class represents a mapping between the IO of a service and the IO of a
 * task.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * @author <a href="mailto:Graham.Thomson@inria.fr">Graham Thomson</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : May 24, 2007 10:41:18 AM
 */
public class IOMatch {

	/**
	 * the type of IO (0 - Input,1 - Output, 1 - Error)
	 */
	private short typeOfIO;

	/**
	 * the IO in the task
	 */
	private IO taskIO;

	/**
	 * the corresponding IO in the service
	 */
	private IO serviceIO;

	/** The type of matching between the two concept */
	private short typeOfMatching = MatchingType.FAIL;

	/**
	 * the value of typeOfIO if it is an Input
	 */
	public static final short INPUT = 0;

	/**
	 * the value of typeOfIO if it is an Output
	 */
	public static final short OUTPUT = 1;

	/**
	 * the value of typeOfIO if the match is not valid (one Input and one
	 * Output)
	 */
	private static final short ERROR = 2;

	/**
	 * Construct an IOMatch object
	 * 
	 * @param type
	 *            the rdf type of the param
	 * @param serviceLocalName
	 *            its localname in the Service description
	 * @param taskLocalName
	 *            its localname in the taskdescription
	 * @param typeOfIO
	 *            indicate the type of IO (Input, Output) *
	 * @param matchingType
	 *            the type of the match between the two concept
	 */
	/*
	 * public IOMatch(String type, String serviceLocalName, String
	 * taskLocalName, int typeOfIO, short matchingType) { this.serviceLocalName =
	 * serviceLocalName; this.taskLocalName = taskLocalName; this.typeOfIO =
	 * typeOfIO; this.typeName = Tools.getSecondPart(type); this.ontology =
	 * Tools.getFirstPart(type); this.typeOfMatching = matchingType; }
	 */

	/**
	 * Construct an IOMatch object
	 * 
	 * @param userTaskIO
	 *            the concerning io in the UserTask
	 * @param serviceIO
	 *            the corresponding io in the Service
	 * @param matchingType
	 *            the type of the match between the two concept
	 */
	public IOMatch(IO userTaskIO, IO serviceIO, short matchingType) {

		// collect data
		this.taskIO = userTaskIO;
		this.serviceIO = serviceIO;
		this.typeOfMatching = matchingType;

		// fill type of IO
		if ((userTaskIO instanceof Input) && (serviceIO instanceof Input))
			this.typeOfIO = IOMatch.INPUT;
		else if ((userTaskIO instanceof Output)
				&& (serviceIO instanceof Output))
			this.typeOfIO = IOMatch.OUTPUT;
		else
			this.typeOfIO = IOMatch.ERROR;

	}

	/**
	 * @return the ontology
	 */
	public String getOntology() {
		return this.getServiceIO().getOntology();
	}

	/**
	 * @return the serviceLocalName
	 */
	public String getServiceLocalName() {
		return this.getServiceIO().getLocalName();
	}

	/**
	 * @return the taskLocalName
	 */
	public String getTaskLocalName() {
		return this.getTaskIO().getLocalName();
	}

	/**
	 * @return the typeName
	 */
	public String getTypeName() {
		return this.getTaskIO().getType();
	}

	/**
	 * @return the typeOfIO
	 */
	public short getTypeOfIO() {
		return typeOfIO;
	}

	/**
	 * @param typeOfIO
	 *            the typeOfIO to set
	 */
	@SuppressWarnings("unused")
	private void setTypeOfIO(short typeOfIO) {
		this.typeOfIO = typeOfIO;
	}

	/**
	 * 
	 * @return the type of the IO
	 */
	public String getType() {
		return this.getOntology() + "#" + this.getTypeName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME
				* result
				+ ((this.getOntology() == null) ? 0 : this.getOntology()
						.hashCode());
		result = PRIME
				* result
				+ ((this.getServiceLocalName() == null) ? 0 : this
						.getServiceLocalName().hashCode());
		result = PRIME
				* result
				+ ((this.getTaskLocalName() == null) ? 0 : this
						.getTaskLocalName().hashCode());
		result = PRIME
				* result
				+ ((this.getTypeName() == null) ? 0 : this.getTypeName()
						.hashCode());
		result = PRIME * result + this.getTypeOfIO();
		result = PRIME * result + this.getTypeOfMatching();
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final IOMatch other = (IOMatch) obj;
		if (this.getOntology() == null) {
			if (other.getOntology() != null)
				return false;
		} else if (!this.getOntology().equals(other.getOntology()))
			return false;
		if (this.getServiceLocalName() == null) {
			if (other.getServiceLocalName() != null)
				return false;
		} else if (!this.getServiceLocalName().equals(
				other.getServiceLocalName()))
			return false;
		if (this.getTaskLocalName() == null) {
			if (other.getTaskLocalName() != null)
				return false;
		} else if (!this.getTaskLocalName().equals(other.getTaskLocalName()))
			return false;
		if (this.getTypeName() == null) {
			if (other.getTypeName() != null)
				return false;
		} else if (!this.getTypeName().equals(other.getTypeName()))
			return false;
		if (this.getTypeOfIO() != other.getTypeOfIO())
			return false;
		if (getTypeOfMatching() != other.getTypeOfMatching())
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String result = "\n\t\t\t\t";
		switch (this.getTypeOfIO()) {
		case IOMatch.INPUT:
			result += "{INPUT}";
			break;
		case IOMatch.OUTPUT:
			result += "{OUTPUT}";
			break;
		default:
			result += "{ERROR}";
		}
		result += " " + this.getType() + ": (S->T) "
				+ this.getServiceLocalName() + "->" + this.getTaskLocalName();
		switch (this.getTypeOfMatching()) {
		case MatchingType.FAIL:
			result += " [FAIL]";
			break;
		case MatchingType.EXACT:
			result += " [EXACT]";
			break;
		case MatchingType.SUBSUMES:
			result += " [SUBSUMES]";
			break;
		case MatchingType.PLUGIN:
			result += " [PLUGIN]";
			break;
		default:
			result += " [FAIL]";
			break;
		}
		return result;
	}

	/**
	 * @return the typeOfMatching
	 */
	public short getTypeOfMatching() {
		return typeOfMatching;
	}

	/**
	 * @return the serviceIO
	 */
	public IO getServiceIO() {
		return serviceIO;
	}

	/**
	 * @return the taskIO
	 */
	public IO getTaskIO() {
		return taskIO;
	}

	/**
	 * Check if the match is exact
	 * 
	 * @return <i>true</i> if the match is exact, <i>false</i> otherwise
	 */
	public boolean isExact() {
		return this.getTypeOfMatching() == MatchingType.EXACT;
	}

	/**
	 * Check if the match is inclusive (that means that if the IOMatch is for
	 * input, the match is exact or subsum ; and exact or plugin for output)
	 * 
	 * @return <i>true</i> if the match is inclusive, <i>false</i> otherwise
	 */
	public boolean isInclusive() {
		short typeCapa = this.getTypeOfMatching();
		if (this.typeOfIO == IOMatch.INPUT) {
			if (typeCapa == MatchingType.EXACT
					|| typeCapa == MatchingType.SUBSUMES)
				return true;
			else
				return false;
		} else if (this.typeOfIO == IOMatch.OUTPUT) {
			if (typeCapa == MatchingType.EXACT
					|| typeCapa == MatchingType.PLUGIN)
				return true;
			else
				return false;
		} else
			return false;
	}

}
