/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.matching;

import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;

/**
 * 
 * A Matcher is the object containing the algorithm of matching between two
 * Service object. This interface expose the operation that have to be
 * implemented to compare each level of the model (Service, Operation,
 * Input/Output)
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 13, 2007 3:17:02 PM
 */
public abstract class Matcher {

	/**
	 * Counter of the number of match This array of int corresponds to
	 * <p>
	 * 0 - The number of service matched<br>
	 * 1 - The number of capability matched<br>
	 * 3 - The number of io matched
	 * </p>
	 */
	protected int[] stats = new int[] { 0, 0, 0 };
	
	/**
	 * Get a matching between the two concepts (the task Concept comparing to the service one)
	 * @param serviceType the reference concept
	 * @param taskType the concept to compare with
	 * @return <ul><li>MatchingType.EXACT if the two concept are equals or equivalent</li><li>Matching.SUBSUMES if task concept subsums the service one </li><li>Matching.PLUGIN if service concept subsums the task one </li><li>Matching.FAIL if there is no link between the two concept</li></ul>
	 * @throws SemanticTypeNotKnowException if a type is not known
	 */
	public abstract short typeMatch(SemanticType serviceType, SemanticType taskType) throws SemanticTypeNotKnowException;

	/**
	 * Get a full matching between the task and the current service
	 * 
	 * @param service
	 *            the current service
	 * @param userTask
	 *            the task to match with
	 * @return a ServiceMatch object containing the matching information; if
	 *         some capabilities could be matched, <i>null</i> if no matching
	 *         is possible or if the matching is not complete
	 * @throws SemanticTypeNotKnowException
	 *             if a type is not known
	 */
	public abstract ServiceMatch serviceMatch(Service service, UserTask userTask)
			throws SemanticTypeNotKnowException;

	/**
	 * Check if the IO match with the given semanticType<br>
	 * This method return
	 * <ul>
	 * <li><b>MatchingType.EXACT</b> if the match is exact</li>,
	 * <li><b>MatchingType.PLUGIN</b> if the serviceIOSemantictype subsum the
	 * taskIOSemanticType,</li>
	 * <li><b>MatchingType.SUBSUM</b> if the taskIOSemantictype subsum the
	 * serviceIOSemanticType,</li>
	 * <li><b>MatchingType.FAIL</b> if there is no match</li>
	 * </ul>
	 * 
	 * <br>
	 * <br>
	 * <b><font color="#FF0000">WARNING</font> :</b> each Matcher are
	 * specialized, so some matcher could response only MatchingType.EXACT or
	 * MatchingType.FAIL
	 * 
	 * @param serviceIOSemanticType
	 *            the semanticType of the current io
	 * @param taskIOsemanticType
	 *            the semanticType we want to check the matching with (the one
	 *            of the corresponding io in the task).
	 * @return an integer corresponding to a specified MatchingType value
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	public abstract int ioMatch(String serviceIOSemanticType,
			String taskIOsemanticType) throws SemanticTypeNotKnowException;

	/**
	 * Check if the two Capability match (depending the alorithm implemented)
	 * 
	 * @param serviceCapa
	 *            the current Capability (the one of the service)
	 * @param taskCapa
	 *            the capability to match with.(the one of the task)
	 * @return a CapabilityMatch object, containing all matching information if
	 *         the match is ok, <i>null</i> otherwise
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	public abstract CapabilityMatch capabilityMatch(Capability serviceCapa,
			Capability taskCapa) throws SemanticTypeNotKnowException;

	/**
	 * Check if the two IOmatch (depending the alorithm implemented)
	 * 
	 * @param serviceIO
	 *            the current IO (the one of the service)
	 * @param taskIO
	 *            the capability IO to match with.(the one of the task)
	 * @return a IOMatch object, containing all matching information if the
	 *         match is ok, <i>null</i> otherwise
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	public abstract IOMatch ioMatch(IO serviceIO, IO taskIO)
			throws SemanticTypeNotKnowException;

	/**
	 * Try to match 2 capabilities with followinf properties :<br>
	 * ExactCapabilityMatch(Adv,Req) = <br>
	 * for all inputs in of Req.In, and all inputs in0 of Adv.In
	 * ConceptMatch(in0, in) = exact AND <br>
	 * for all outputs out in Req.Out, and all outputs out0 in Adv.Out
	 * ConceptMatch(out0,out) = exact AND <br>
	 * ConceptMatch(Adv.Category, Req.Category) = exact <br>
	 * 
	 * @param serviceCapa
	 *            the proposed capability (Adv)
	 * @param taskCapa
	 *            the required capability (Req)
	 * @return <i>true</i> if the both capabilities match with given
	 *         properties, <i>false</i> otherwise
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	public boolean exactCapabilityMatch(Capability serviceCapa,
			Capability taskCapa) throws SemanticTypeNotKnowException {
		CapabilityMatch match = this.capabilityMatch(serviceCapa, taskCapa);

		if (match != null)
			return match.isExact();
		return false;
		
	}

	/**
	 * Try to match 2 capabilities with followinf properties :<br>
	 * InclusiveCapabilityMatch(Adv,Req)= <br>
	 * for all inputs in of Req.In, and all inputs in0 of Adv.In
	 * ConceptMatch(in0, in) = exact|plugin AND<br>
	 * for all outputs out in Req.Out, and all outputs out0 in Adv.Out
	 * ConceptMatch(out0,out) = exact|plugin AND<br>
	 * ConceptMatch(Adv.Category, Req.Category) = exact|plugin<br>
	 * 
	 * @param serviceCapa
	 *            the proposed capability (Adv)
	 * @param taskCapa
	 *            the required capability (Req)
	 * @return <i>true</i> if the both capabilities match with given
	 *         properties, <i>false</i> otherwise
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	public boolean inclusiveCapabilityMatch(Capability serviceCapa,
			Capability taskCapa) throws SemanticTypeNotKnowException {
		CapabilityMatch match = this.capabilityMatch(serviceCapa, taskCapa);

		/*DebugPrinter.print("match (" + serviceCapa.getLocalName() + ","
				+ taskCapa.getLocalName() + "):\n" + match, DebugPrinter.INFO);*/

		if (match != null)
			return match.isInclusive();
		return false;
	}

	/**
	 * Try to match 2 capabilities with followinf properties :<br>
	 * WeakCapabilityMatch(Adv,Req) =<br>
	 * for all inputs in of Req.In, and all inputs in0 of Adv.In
	 * ConceptMatch(in0, in) != fail AND<br>
	 * for all outputs out in Req.Out, and all outputs out0 in Adv.Out
	 * ConceptMatch(out0,out) != fail AND<br>
	 * ConceptMatch(Adv.Category, Req.Category) != fail<br>
	 * 
	 * @param serviceCapa
	 *            the proposed capability (Adv)
	 * @param taskCapa
	 *            the required capability (Req)
	 * @return <i>true</i> if the both capabilities match with given
	 *         properties, <i>false</i> otherwise
	 * @throws SemanticTypeNotKnowException
	 *             if one type is not found
	 */
	public boolean weakCapabilityMatch(Capability serviceCapa,
			Capability taskCapa) throws SemanticTypeNotKnowException {
		CapabilityMatch match = this.capabilityMatch(serviceCapa, taskCapa);
		return match != null;
	}

	/**
	 * get the number of match done during the all discovery process
	 * 
	 * @return a array of int corresponding to
	 *         <p>
	 *         0 - The number of service matched<br>
	 *         1 - The number of capability matched<br>
	 *         3 - The number of io matched
	 *         </p>
	 */
	public int[] getStat() {
		return this.stats;
	}

	/**
	 * Reset all counter for number of matching
	 * 
	 */
	public void reInitStat() {
		for (int i = 0; i < this.stats.length; i++)
			stats[i] = 0;
	}
}
