/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.matching;

import java.util.ArrayList;
import java.util.List;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;

/**
 * 
 * This class represents a mapping between a service and a task.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * @author <a href="mailto:Graham.Thomson@inria.fr">Graham Thomson</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : May 25, 2007 10:44:06 AM
 */
public class ServiceMatch {

	/**
	 * The user task object used as reference for the match
	 */
	private UserTask task;

	/**
	 * The service object used for the match
	 */
	private Service service;

	/**
	 * List of the match between Service capabilities and UserTask capabilities
	 */
	private List<CapabilityMatch> capabilityMatches = new ArrayList<CapabilityMatch>();

	/** The type of matching between the two concept */
	private short typeOfMatching = MatchingType.FAIL;

	/**
	 * Constructor for the ServiceMatch object
	 * 
	 * @param task
	 *            the task that are required by the User
	 * @param service
	 *            the service object that has been compared with the task
	 * @param matchingType
	 *            the type of the match between the two concept
	 */
	public ServiceMatch(UserTask task, Service service, short matchingType) {
		this.task = task;
		this.service = service;
		this.typeOfMatching = matchingType;
	}

	/**
	 * Add an capability matching to the list of matching capability
	 * 
	 * @param cm :
	 *            the CapabilityMatch object to add
	 * @return <i>true</i> if no error occurs, <i>false</i> otherwise
	 */
	public boolean addCapabilityMatch(CapabilityMatch cm) {
		return this.capabilityMatches.add(cm);
	}

	/**
	 * @return the capabilityMatches
	 */
	public List<CapabilityMatch> getCapabilityMatches() {
		// TODO update this to use a map, to enable efficient lookup in
		// CapabilityInvoker.
		return capabilityMatches;
	}

	/**
	 * @return the service
	 */
	public Service getService() {
		return service;
	}

	/**
	 * @return the task
	 */
	public UserTask getTask() {
		return task;
	}

	/**
	 * @return the typeOfMatching
	 */
	public short getTypeOfMatching() {
		return typeOfMatching;
	}

	@Override
	public String toString() {
		String result = "ServiceMatch:\n\tTask = " + task + "\n\tService = "
				+ service + "\n\tMatching = ";
		for (int i = 0; i < this.getCapabilityMatches().size(); i++)
			result += "\n" + this.getCapabilityMatches().get(i).toString();

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME
				* result
				+ ((capabilityMatches == null) ? 0
						: capabilityMatches.hashCode());
		result = PRIME * result + ((service == null) ? 0 : service.hashCode());
		result = PRIME * result + ((task == null) ? 0 : task.hashCode());
		result = PRIME * result + typeOfMatching;
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final ServiceMatch other = (ServiceMatch) obj;
		if (capabilityMatches == null) {
			if (other.capabilityMatches != null)
				return false;
		} else if (!Tools.compareList(capabilityMatches,
			other.capabilityMatches))
			return false;
		if (service == null) {
			if (other.service != null)
				return false;
		} else if (!service.equalsWithoutCapabilities(other.service))
			return false;
		if (task == null) {
			if (other.task != null)
				return false;
		} else if (!task.equalsWithoutCapabilities(other.task))
			return false;
		if (typeOfMatching != other.typeOfMatching)
			return false;
		return true;
	}

}
