package fr.inria.icocoa.servicemodel.parsers;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParsingException;

/**
 * This object is used to get the most appropriate parser for the service
 * description. <b>There is no default value for the parser.</b> <br>
 * <br>
 * The class that has to be used as parser can be set by using the System
 * property "serviceparser.classname". To set it, please run jvm with option
 * <i>-Dserviceparser.classname="YourCompleteClassName"</i>. If the classname
 * is false, then default semantic type will be used (StringSemanticType) ! This
 * will be indicate in log traces.
 * 
 * <br><br>Exemple : -Dserviceparser.classname="fr.inria.icocoa.servicemodel.parsers.jenaparser.SDL2ServiceParser"
 * 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 6, 2008 4:59:01 PM
 */
public class ServiceParserFactory {

	/**
	 * the System property value to look at to set the class that need to be
	 * used during execution
	 */
	private final static String PROPERTYNAME = "serviceparser.classname";

	/**
	 * the single instance of ServiceParserFactory
	 */
	private static ServiceParserFactory instance = null;

	/**
	 * The constructor that will be used to generate object without any
	 * parameters
	 */
	private Constructor<ServiceParser> defaultConstructor = null;

	/**
	 * Check property "serviceparser.classname" and check that corresponding
	 * value is a subclass of ServiceParser.
	 * 
	 * @throws ParsingException
	 *             if no property "serviceparser.classname" has been set or if
	 *             the value doesn't correspond to a valid ServiceParser
	 */
	private ServiceParserFactory() throws ParsingException {
		// get System property
		String _ClassName = System
				.getProperty(ServiceParserFactory.PROPERTYNAME);

		if (_ClassName != null) {
			// load the class
			try {
				Class _ServiceParserClass = Class.forName(_ClassName);

				// check constructors
				Constructor<ServiceParser> _defaultConstructor = _ServiceParserClass
						.getConstructor();

				if (_defaultConstructor != null) {
					this.defaultConstructor = _defaultConstructor;
					DebugPrinter.print("using "+_ServiceParserClass+" as Service Parser", DebugPrinter.INFO);
				} else {
					DebugPrinter
							.print(
									"The property "
											+ ServiceParserFactory.PROPERTYNAME
											+ " has been set to "
											+ _ClassName
											+ " but this class has no default constructor => No parser to use ! ",
									DebugPrinter.ERROR);
					throw new ParsingException(
							"The property "
									+ ServiceParserFactory.PROPERTYNAME
									+ " has been set to "
									+ _ClassName
									+ " but this class has no default constructor => No parser to use ! ");
				}

			} catch (ClassNotFoundException e) {
				DebugPrinter
						.print(
								"The property "
										+ ServiceParserFactory.PROPERTYNAME
										+ " has been set to "
										+ _ClassName
										+ " but this class has not been found => No parser to use ! ",
								DebugPrinter.ERROR);
				throw new ParsingException(e);
			} catch (SecurityException e) {
				DebugPrinter
						.print(
								"The property "
										+ ServiceParserFactory.PROPERTYNAME
										+ " has been set to "
										+ _ClassName
										+ " but this class cannot be acceded => No parser to use ! ",
								DebugPrinter.ERROR);
				throw new ParsingException(e);
			} catch (NoSuchMethodException e) {
				DebugPrinter
						.print(
								"The property "
										+ ServiceParserFactory.PROPERTYNAME
										+ " has been set to "
										+ _ClassName
										+ " but this class has no default constructor => No parser to use ! ",
								DebugPrinter.ERROR);
				throw new ParsingException(e);
			}
		} else {
			DebugPrinter.print("The property "
					+ ServiceParserFactory.PROPERTYNAME
					+ " has not been set... No parser to use ! ",
					DebugPrinter.ERROR);
			throw new ParsingException("The property "
					+ ServiceParserFactory.PROPERTYNAME
					+ " has not been set... No parser to use ! ");
		}

	}

	/**
	 * Get the single instance of ServiceParserFactory
	 * 
	 * @return the ServiceParserFactory object
	 * @throws ParsingException
	 *             if no property "serviceparser.classname" has been set or if
	 *             the value doesn't correspond to a valid ServiceParser
	 */
	public static ServiceParserFactory getInstance() throws ParsingException {
		if (ServiceParserFactory.instance == null)
			ServiceParserFactory.instance = new ServiceParserFactory();

		return ServiceParserFactory.instance;
	}
	
	/**
	 * Create an instance of ServiceParser using the selected class to represent
	 * SemanticType, using default constructor
	 * 
	 * @return a ServiceParser, <b>null</b> if an error occurs.
	 */
	public ServiceParser getParser(){
		try {
			return defaultConstructor.newInstance();
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (InstantiationException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (InvocationTargetException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		}
		return null;
	}

}
