/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype;

import java.math.BigInteger;
import java.util.List;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.BigIntegerCode;
import fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.PrimeNumberEncoder;

/**
 * Implementation of EncodedSemanticType by using Prime number to encode SemanticType 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 6, 2008 1:48:54 PM
 */
public class PrimeNumberEncodedSemanticType extends EncodedSemanticType {

	/*
	 * Done before any creation of object of this type
	 */
	static {
		// associate the encoder
		EncodedSemanticType.associatedEncoder = PrimeNumberEncoder.class;

		// associate the class used as Code
		EncodedSemanticType.associatedCodeClass = BigIntegerCode.class;
	}

	/**
	 * serial ID of the class
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * 
	 */
	public PrimeNumberEncodedSemanticType() {
		super();
	}

	/**
	 * @param typeName
	 * @param ontology
	 */
	public PrimeNumberEncodedSemanticType(String typeName, String ontology) {
		super(typeName, ontology);
		this.setType(ontology + "#" + typeName);
	}

	/**
	 * 
	 * @param typeURI
	 * @param encodedValue
	 */
	public PrimeNumberEncodedSemanticType(String typeURI,
			BigInteger encodedValue) {
		super();
		super.setType(typeURI);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType#isEquals(fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType)
	 */
	@Override
	public boolean isEquals(EncodedSemanticType other) {
		if (this.getOntology().equals(other.getOntology())) {
			if (!this.getListOfCodes().isEmpty()
					&& !other.getListOfCodes().isEmpty()) {
				BigIntegerCode myCode = (BigIntegerCode) this.getListOfCodes()
						.get(0);
				BigIntegerCode otherCode = (BigIntegerCode) other
						.getListOfCodes().get(0);
				if (myCode != null && otherCode != null)
					return myCode.isExact(otherCode);
				else
					return false;
			} else
				return false;
		} else
			return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType#isPlugin(fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType)
	 */
	@Override
	public boolean isPlugin(EncodedSemanticType other) {
		if (this.getOntology().equals(other.getOntology())) {
			if (!this.getListOfCodes().isEmpty()
					&& !other.getListOfCodes().isEmpty()) {
				BigIntegerCode myCode = (BigIntegerCode) this.getListOfCodes()
						.get(0);
				BigIntegerCode otherCode = (BigIntegerCode) other
						.getListOfCodes().get(0);
				if (myCode != null && otherCode != null)
					return myCode.isPlugin(otherCode);
				else
					return false;
			} else
				return false;
		} else
			return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType#isSubsum(fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType)
	 */
	@Override
	public boolean isSubsum(EncodedSemanticType other) {
		if (this.getOntology().equals(other.getOntology())) {
			if (!this.getListOfCodes().isEmpty()
					&& !other.getListOfCodes().isEmpty()) {
				BigIntegerCode myCode = (BigIntegerCode) this.getListOfCodes()
						.get(0);
				BigIntegerCode otherCode = (BigIntegerCode) other
						.getListOfCodes().get(0);
				if (myCode != null && otherCode != null)
					return myCode.isSubsum(otherCode);
				else
					return false;
			} else
				return false;
		} else
			return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = super.hashCode();
		result = PRIME
				* result
				+ ((this.getListOfCodes() == null) ? 0 : this.getListOfCodes()
						.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		final PrimeNumberEncodedSemanticType other = (PrimeNumberEncodedSemanticType) obj;
		if (this.getListOfCodes() == null) {
			if (other.getListOfCodes() != null)
				return false;
			else
				return true;
		} else {
			if (other.getListOfCodes() == null)
				return false;
			else
				return Tools.compareList(this.getListOfCodes(), other
						.getListOfCodes());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return super.toString() + " $" + this.getListOfCodes().toString() + "$";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType#setType(java.lang.String)
	 */
	@Override
	public void setType(String newType) {
		super.setType(newType);
		if (!this.getTypeName().equals(UNKNOWN)
				&& !this.getOntology().equals(UNKNOWN)) {
			// get the corresponding Encodedvalue from the ontManager that store
			// all map "list of ontologies"<->"encodedvalue"
			
			try {
				List<Code> listOfCodes = EncodedSemanticType.ontManager
						.getEncryptedObject(this.getTypeName(), this.getOntology());
				if (listOfCodes != null) {
					try {
						this.getListOfCodes().addAll(listOfCodes);
					} catch (ClassCastException e) {
						DebugPrinter.print(e, DebugPrinter.INFO);
					}
				}
				
			} catch (SemanticTypeNotKnowException e1) {
				DebugPrinter.print(e1,DebugPrinter.WARN);
			}			
		}
		// DebugPrinter.print("result = "+this,DebugPrinter.INFO);
	}

}
