/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;

/**
 * This class generate SemanticType objects taking care the name of the class
 * that need to be used as SemanticType. Be carefull that this class is a
 * subclass of SemanticType.<br>
 * <br>
 * 
 * This indication could be done by using the System property
 * "semantictype.classname". To set it, please run jvm with option
 * <i>-Dsemantictype.classname="YourCompleteClassName"</i>. If the classname is
 * false, then default semantic type will be used (StringSemanticType) ! This
 * will be indicate in log traces.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 4, 2008 1:52:26 PM
 */
public class SemanticTypeFactory {
	
	//call before all code
	static{
		//initialise config manager so as to be sur that good system properties are loaded (required for semantictype.classname)
		ConfigManager.getInstance();
	}

	/**
	 * the System property value to look at to set the class that need to be
	 * used during execution
	 */
	private final static String PROPERTYNAME = "semantictype.classname";

	/**
	 * The default class that will be load in case of error
	 */
	private final static Class DEFAULTCLASS = StringSemanticType.class;

	/**
	 * Single instance of SemanticTypeFactory
	 */
	private static SemanticTypeFactory instance = null;

	/**
	 * The constructor that will be used to generate object without any
	 * parameters
	 */
	private Constructor<SemanticType> defaultConstructor = null;

	/**
	 * The constructor that will be used to generate object with two parameters
	 * (Strings)
	 */
	private Constructor<SemanticType> twoStringsConstructor = null;

	/**
	 * the class object that will be used to generate instance of SemanticType
	 */
	private Class semanticTypeClass = null;

	/**
	 * Check property "semantictype.classname" and check that corresponding
	 * value is a subclass of SemanticType, and contains all required
	 * constructor.
	 * 
	 */
	@SuppressWarnings("unchecked")
	private SemanticTypeFactory() {
		// get System property
		String _ClassName = System
				.getProperty(SemanticTypeFactory.PROPERTYNAME);

		try {
			// set class to default class... If no error occurs, this class will
			// be updated
			this.semanticTypeClass = SemanticTypeFactory.DEFAULTCLASS;

			if (_ClassName != null) {
				// load the class
				Class _SemanticTypeClass = Class.forName(_ClassName);

				// check constructors
				Constructor<SemanticType> _defaultConstructor = _SemanticTypeClass
						.getConstructor();
				Constructor<SemanticType> _twoStringsConstructor = _SemanticTypeClass
						.getConstructor(new Class[] { "".getClass(),
								"".getClass() });

				if (_defaultConstructor != null
						&& _twoStringsConstructor != null) {
					this.semanticTypeClass = _SemanticTypeClass;
					DebugPrinter.print("using " + _SemanticTypeClass
							+ " as Semantic Type", DebugPrinter.INFO);
				} else {
					DebugPrinter
							.print(
									"The class "
											+ _ClassName
											+ " return null object when trying to get one of the required constructor. The class "
											+ SemanticTypeFactory.DEFAULTCLASS
													.getName()
											+ " will be used as SemanticType",
									DebugPrinter.INFO);
					this.semanticTypeClass = SemanticTypeFactory.DEFAULTCLASS;
				}
			} else {
				DebugPrinter.print("No system property "
						+ SemanticTypeFactory.PROPERTYNAME
						+ " has been declared. The class "
						+ SemanticTypeFactory.DEFAULTCLASS.getName()
						+ " will be used as SemanticType", DebugPrinter.INFO);
			}

		} catch (ClassNotFoundException e) {
			DebugPrinter.print("The class " + _ClassName
					+ " has not been found. The class "
					+ SemanticTypeFactory.DEFAULTCLASS.getName()
					+ " will be used as SemanticType", DebugPrinter.INFO);
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (SecurityException e) {
			DebugPrinter.print("The class " + _ClassName
					+ " cannot be acceded (or just a constructor). The class "
					+ SemanticTypeFactory.DEFAULTCLASS.getName()
					+ " will be used as SemanticType", DebugPrinter.INFO);
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (NoSuchMethodException e) {
			DebugPrinter
					.print(
							"The class "
									+ _ClassName
									+ " miss one of the required constructor (cf SemanticType javadoc). The class "
									+ SemanticTypeFactory.DEFAULTCLASS
											.getName()
									+ " will be used as SemanticType",
							DebugPrinter.INFO);
			DebugPrinter.print(e, DebugPrinter.INFO);
		} finally {
			try {
				// update constructors
				this.defaultConstructor = this.semanticTypeClass
						.getConstructor(new Class[] {});
				this.twoStringsConstructor = this.semanticTypeClass
						.getConstructor(new Class[] { "".getClass(),
								"".getClass() });
			} catch (SecurityException e) {
				DebugPrinter.print(e, DebugPrinter.ERROR);
			} catch (NoSuchMethodException e) {
				DebugPrinter.print(e, DebugPrinter.ERROR);
			}

		}
	}

	/**
	 * Create a new instance of the singloton if none exist, or return the
	 * current instance.
	 * 
	 * @return an instance of SemanticTypeFactory.
	 */
	public static SemanticTypeFactory getInstance() {
		if (SemanticTypeFactory.instance == null)
			SemanticTypeFactory.instance = new SemanticTypeFactory();

		return SemanticTypeFactory.instance;
	}

	/**
	 * delete the current instance of the singloton
	 * 
	 */
	public static void clearInstance() {
		SemanticTypeFactory.instance = null;
	}

	/**
	 * Create an instance of semanticType using the selected class to represent
	 * SemanticType, and using the given parameters to initialize it
	 * 
	 * @param typeName
	 *            The typeName (ontology class) of the input (without the
	 *            ontology extension)
	 * @param ontology
	 *            The URL of the ontology used
	 * @return a SemanticObject, <b>null</b> if an error occurs.
	 */
	public SemanticType getObject(String typeName, String ontology) {
		try {
			return twoStringsConstructor.newInstance(typeName, ontology);
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (InstantiationException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (InvocationTargetException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		}
		return null;
	}

	/**
	 * Create an instance of semanticType using the selected class to represent
	 * SemanticType, using default constructor
	 * 
	 * @return a SemanticObject, <b>null</b> if an error occurs.
	 */
	public SemanticType getObject() {
		try {
			return defaultConstructor.newInstance();
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (InstantiationException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		} catch (InvocationTargetException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
		}
		return null;
	}

	/**
	 * @return the semanticTypeClass
	 */
	public Class getSemanticTypeClass() {
		return semanticTypeClass;
	}
	
	
	
}
