/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.UUID;
import java.util.Map.Entry;

import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.parsers.ServiceParser;
import fr.inria.icocoa.servicemodel.semantictype.Code;
import fr.inria.icocoa.servicemodel.semantictype.EncodedSemanticType;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;
import fr.inria.icocoa.servicemodel.semantictype.SemanticTypeFactory;

/**
 * This object manage the stack of the ontologies that have already be encoded.<br>
 * <br>
 * When a new service is parsed, the manager get a list of the ontologies used
 * to describe the service. Then it checks if those ontologies have been already
 * encoded (also check the date if the last modification so as to be sur that
 * the ontologie has not evolved). <br>
 * <br>
 * If the encoded ontology If the encoded ontology exist, then the Manager will
 * load it. Otherwise, the encodement start, and the result is stored in a File.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 6, 2008 2:56:09 PM
 */
public class EncodedOntologiesManager implements Serializable {

	/**
	 * the serial ID of the class
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The separator that will be use to generate the concatenation of the list
	 * of imported ontologies
	 */
	public final transient static String IMPORTED_ONTOLOGIES_HASHCODE_SEPARATOR = ";";

	/**
	 * 
	 * This class gather the importHash code and the name of the class used for
	 * Encryption so as to be used as key of a map "Encoded ontology"<->"Map of
	 * encoded values"
	 * 
	 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> -
	 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
	 * 
	 * Creation Date : Mar 27, 2008 1:23:21 PM
	 */
	private class EncodedOntologiesKey implements Serializable {

		/**
		 * serial ID of the class
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * The hascode corresponding to the list of imports of the last parsed
		 * file
		 */
		private String importHashCode = null;

		/**
		 * The name of the class used to encode the semantic type
		 */
		private String encryptionClassName = null;

		/**
		 * @param importHashCode
		 * @param encryptionClassName
		 */
		public EncodedOntologiesKey(String importHashCode,
				String encryptionClassName) {
			this.importHashCode = importHashCode;
			this.encryptionClassName = encryptionClassName;
		}

		/**
		 * @return the encryptionClassName
		 */
		public String getEncryptionClassName() {
			return encryptionClassName;
		}

		/**
		 * @return the importHashCode
		 */
		public String getImportHashCode() {
			return importHashCode;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Object#hashCode()
		 */
		@Override
		public int hashCode() {
			final int PRIME = 31;
			int result = 1;
			result = PRIME
					* result
					+ ((encryptionClassName == null) ? 0 : encryptionClassName
							.hashCode());
			result = PRIME
					* result
					+ ((importHashCode == null) ? 0 : importHashCode.hashCode());
			return result;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Object#equals(java.lang.Object)
		 */
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			final EncodedOntologiesKey other = (EncodedOntologiesKey) obj;
			if (encryptionClassName == null) {
				if (other.encryptionClassName != null)
					return false;
			} else if (!encryptionClassName.equals(other.encryptionClassName))
				return false;
			if (importHashCode == null) {
				if (other.importHashCode != null)
					return false;
			} else if (!importHashCode.equals(other.importHashCode))
				return false;
			return true;
		}
	}

	/**
	 * 
	 * This class gather the map storing the encoded value of concept and the
	 * name of the file where those result are stored.
	 * 
	 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> -
	 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
	 * 
	 * Creation Date : Mar 27, 2008 1:28:42 PM
	 */
	private class EncodedOntologiesValue implements Serializable {

		/**
		 * serial ID of the class
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * The Map containing the values of encoded concept
		 */
		private Map<String, List<Code>> mapOfConcept = new HashMap<String, List<Code>>();

		/**
		 * the path of the file used to store those informations
		 */
		private String filePath = "";

		/**
		 * @param mapOfConcept
		 * @param filePath
		 */
		public EncodedOntologiesValue(Map<String, List<Code>> mapOfConcept,
				String filePath) {
			this.mapOfConcept = mapOfConcept;
			this.filePath = filePath;
		}

		/**
		 * @return the filePath
		 */
		public String getFilePath() {
			return filePath;
		}

		/**
		 * @return the mapOfConcept
		 */
		public Map<String, List<Code>> getMapOfConcept() {
			return mapOfConcept;
		}

	}

	/**
	 * The single instance of the manager that exist
	 */
	private transient static EncodedOntologiesManager instance = null;

	/**
	 * The class currently used to encrypt ontologies
	 */
	private transient Class<? extends EncodedSemanticType> encryptionClass = null;

	/**
	 * the map stroring the assocation "Encoded ontology"<->"Map of encoded
	 * values"
	 */
	private Map<EncodedOntologiesKey, EncodedOntologiesValue> mapOfEncodedOntologies = new HashMap<EncodedOntologiesKey, EncodedOntologiesValue>();

	/**
	 * Default constructor, load the map containing filepath to the already
	 * encoded ontologies
	 * 
	 */
	@SuppressWarnings("unchecked")
	private EncodedOntologiesManager() {

		// get EncryptionClass
		this.encryptionClass = SemanticTypeFactory.getInstance()
				.getSemanticTypeClass();
		// DebugPrinter.print("encryptionClass = " +
		// encryptionClass,DebugPrinter.INFO);

		// load the map of ontologies
		this.loadTheMapOfOntologies();
	}

	/**
	 * Get the single instance of the EncodedOntologiesManager
	 * 
	 * @return the EncodedOntologiesManager object
	 */
	public static EncodedOntologiesManager getInstance() {
		if (instance == null)
			EncodedOntologiesManager.instance = new EncodedOntologiesManager();

		return EncodedOntologiesManager.instance;
	}

	/**
	 * delete current instance of EncodedOntologiesManager
	 * 
	 */
	public void clearInstance() {
		EncodedOntologiesManager.instance = null;
	}

	/**
	 * Get the encoded object corresponding to the concept
	 * 
	 * @param typeName
	 *            the name of the concept
	 * @param ontology
	 *            th ontology the concept depends on
	 * @return the object corresponding to this encrypted object if the concept
	 *         exist and has been encoded, <b>null</b> otherwise
	 * @throws SemanticTypeNotKnowException 
	 */
	public List<Code> getEncryptedObject(String typeName, String ontology) throws SemanticTypeNotKnowException {

		EncodedOntologiesKey key = new EncodedOntologiesKey(ontology,
				this.encryptionClass.getName());

		EncodedOntologiesValue value = this.mapOfEncodedOntologies.get(key);

		if (value == null) {

			// get the hashCodeImports
			try {
				List<String> importHashCode = ServiceParser.getCurrentParser()
						.getListOfImports();

				for (String importedOntology : importHashCode) {
					// create the key and check if this key is contained into
					// the map
					EncodedOntologiesKey key2 = new EncodedOntologiesKey(
							importedOntology, this.encryptionClass.getName());
					if (!this.mapOfEncodedOntologies.containsKey(key2)) {
						this.encodeTheCurrentOntologies(importedOntology);
					}
				}

				// get the value corresponding to the key
				value = this.mapOfEncodedOntologies.get(key);

			} catch (NullPointerException e) {
				DebugPrinter
						.print(
								"No associated parser has been found => No code will be associated with the concept",
								DebugPrinter.INFO);
				// a problem probably occurs and the ontology has not been
				// encoded and
				// not added into the map
				return null;
			}

		}

		if (value != null) {
			Map<String, List<Code>> encodedOntology = value.getMapOfConcept();
			if (encodedOntology != null) {
				// DebugPrinter.print(encodedOntology,"encodedOntology",DebugPrinter.INFO);
				// DebugPrinter.print("searched concept = "+ontology + "#" +
				// typeName, DebugPrinter.INFO);
				List<Code> result =  encodedOntology.get(ontology + "#" + typeName);
				if(result == null)
					throw new SemanticTypeNotKnowException("the concept "+ontology + "#" + typeName+" has no associated code");
				
				return result;
			}
		} 
		return null;

	}

	/**
	 * This method request the encryption of the ontologies.<br>
	 * Once done the result is store in a file, in the map ; and the map itself
	 * is saved.
	 * 
	 * @param importHashCode
	 *            the hash code corresponding to the ontologies used by the
	 *            service that is cuurrently parsed
	 */
	private void encodeTheCurrentOntologies(String importHashCode) {
		DebugPrinter.print("Encode the current Ontology " + importHashCode,
				DebugPrinter.INFO);

		try {
			/*
			 * get the encoder associated to the current class used to encode
			 */
			List<String> listOfImports = new ArrayList<String>();
			String[] importedOntologies = importHashCode
					.split(EncodedOntologiesManager.IMPORTED_ONTOLOGIES_HASHCODE_SEPARATOR);
			for (String importedOntology : importedOntologies) {
				listOfImports.add(importedOntology);
			}

			Map<String, List<Code>> encryptionResult = ((EncodedSemanticType) this.encryptionClass
					.newInstance()).getAssociatedEncoder().newInstance()
					.encode(listOfImports);

			EncodedOntologiesValue value = new EncodedOntologiesValue(
					encryptionResult, this.saveTheEncodedOntology(
							encryptionResult, importHashCode));

			// store result into the map
			this.mapOfEncodedOntologies.put(new EncodedOntologiesKey(
					importHashCode, this.encryptionClass.getName()), value);

			// save the map
			this.saveTheMapOfontologies();

		} catch (InstantiationException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
		}
	}

	/**
	 * Save the current object in a file in a text format
	 * 
	 * @param encryptionResult
	 *            the Map to store un a file
	 * @param associatedHashCode
	 *            the hashcode of ontologies corresponding to theses encoded
	 *            ontologies
	 * @return the path of the file
	 */
	private String saveTheEncodedOntology(
			Map<String, List<Code>> encryptionResult, String associatedHashCode) {
		String filepath = "";
		File fileToWrite = null;

		// check if the file exist, anyway find another unuse name
		do {
			filepath = ConfigManager.getInstance().getRepositoryFolder()+ConfigManager.FILE_SEPARATOR
					+ UUID.randomUUID().toString();
			fileToWrite = new File(filepath);
		} while (fileToWrite.exists());

		// create the file
		try {
			fileToWrite.createNewFile();

			// create writer stream
			BufferedWriter writer = new BufferedWriter(new FileWriter(
					fileToWrite));

			// Write Introduction => associatedHashCode
			String[] ontologies = associatedHashCode.split(";");
			writer
					.write("##########################################################");
			writer.newLine();
			writer
					.write("################### Encoded Ontologies ###################");
			writer.newLine();
			for (int i = 0; i < ontologies.length; i++) {
				writer.write("## " + i + " : " + ontologies[i]);
				writer.newLine();
			}
			writer
					.write("##########################################################");
			writer.newLine();

			writer
					.write("#################### Encoded Concepts ####################");
			writer.newLine();

			// sort Data by using a TreeMap
			Map<String, List<Code>> sortedEncryptionResult = new TreeMap<String, List<Code>>(
					encryptionResult);

			writer.write("##### Nbr of concept = "
					+ sortedEncryptionResult.size());
			writer.newLine();

			// write data
			Iterator<Entry<String, List<Code>>> it = sortedEncryptionResult
					.entrySet().iterator();
			while (it.hasNext()) {
				Entry<String, List<Code>> entry = it.next();
				for (Code code : entry.getValue()) {
					writer.write(entry.getKey() + " = " + code.toString());
					writer.newLine();
				}
			}

			// close stream
			writer.flush();
			writer.close();
		} catch (IOException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
		}

		return filepath;
	}

	private final String FILEPATH = ConfigManager.getInstance().getRepositoryFolder()+ConfigManager.FILE_SEPARATOR
			+ "encodedOntologies.db";

	/**
	 * Save the map of ontologies in a file
	 * 
	 */
	private void saveTheMapOfontologies() {

		DebugPrinter.print("save the map of ontologies", DebugPrinter.INFO);

		File f = new File(FILEPATH);

		try {
			// delete existing file to overwrite the data
			if (f.exists())
				f.delete();

			// create the file
			boolean b = f.createNewFile();

			// create the stream
			ObjectOutputStream writer = new ObjectOutputStream(
					new FileOutputStream(f));

			// write the object
			writer.writeObject(this.mapOfEncodedOntologies);

			// close rge stream
			writer.flush();
			writer.close();
		} catch (IOException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		}
	}

	@SuppressWarnings("unchecked")
	private void loadTheMapOfOntologies() {
		File f = new File(FILEPATH);

		try {
			// check tha file exist
			if (f.exists()) {

				// create the stream
				ObjectInput objIn = new ObjectInputStream(
						new FileInputStream(f));
				this.mapOfEncodedOntologies = (Map<EncodedOntologiesKey, EncodedOntologiesValue>) objIn
						.readObject();
				objIn.close();

			} else {// f doesn't exist
				DebugPrinter
						.print(
								"the file "
										+ f
										+ " has not been found. The list of already encoded ontologies cannot be restaured.",
								DebugPrinter.INFO);
			}

		} catch (IOException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		} catch (ClassNotFoundException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		}

	}

	/**
	 * @return the encryptionClass
	 */
	public Class<? extends EncodedSemanticType> getEncryptionClass() {
		return encryptionClass;
	}
	
	/**
	 * Get all equivalent concept of the given one (it means that they have the same list of codes)
	 * @param type the concept class that we want an equivalent
	 * @return the list of concept that are equivalent to the given one. This list contains at least the given parameter.
	 */
	public List<String> listEquivalentConcept(String type) {
		
		List<String> result = new ArrayList<String>();
		result.add(Tools.getSecondPart(type));
		
		//check if a corresponding ontology is store into the repository
		String ontology = Tools.getFirstPart(type);
		EncodedOntologiesKey key = new EncodedOntologiesKey(ontology,this.getEncryptionClass().getName());
		
		if(this.mapOfEncodedOntologies.containsKey(key)){
			EncodedOntologiesValue value = this.mapOfEncodedOntologies.get(key);
			if(value != null){
								
				//Get the Map of encoded value
				Map<String, List<Code>> mapOfCodes = value.getMapOfConcept();
				if(mapOfCodes!=null){
					
					//get the code corresponding to the given type
					List<Code> encodedType = mapOfCodes.get(type);
					
					/*
					 * Check all othe concept of the ontology to get thoses that have the same code
					 */
					for(Entry<String, List<Code>> entry : mapOfCodes.entrySet()){
						if(entry.getKey()!=type){
							if(entry.getValue()!=null){
								//compare the list of code to see if they are equals without concidering order of elements
								if(Tools.compareList(entry.getValue(), encodedType)){
									String toAdd = Tools.getSecondPart(entry.getKey());
									if(!result.contains(toAdd))
										result.add(toAdd);
								}
							}
						}
					}
					
					//return list of result, if they are at least one equivalent
					if(!result.isEmpty())
						return result;
				}
				
			}
			
		}		
		return null;
	}

	/**
	 * Get the encoded object corresponding to the concept
	 * 
	 * @param semanticType the concept we want to obtain code
	 * @return the object corresponding to this encrypted object if the concept
	 *         exist and has been encoded, <b>null</b> otherwise
	 * @throws SemanticTypeNotKnowException 
	 */
	public List<Code> getEncryptedObject(SemanticType semanticType) throws SemanticTypeNotKnowException {
		return this.getEncryptedObject(semanticType.getTypeName(), semanticType.getOntology());
	}

}
