/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import fr.inria.icocoa.common.DebugPrinter;

/**
 * This class is a factory so as to obtain OntologyGraphGenerator object. <br>
 * This factory get the system property "graphgenerator.classname", so as to dynamically load the classes that has to be used as GraphGenerator. If this propserty is not set, the default value <i>fr.inria.icocoa.servicemodel.semantictype.encoder.jenaencoder.JenaOntologiesGraphGenerator</i> will be used.  
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 31, 2008 4:06:12 PM
 */
public class OntologiesGraphGeneratorFactory {

	/**
	 * Single instance of the Factory
	 */
	private static OntologiesGraphGeneratorFactory instance = null;
	
	/**
	 * The constructor that will be used to return instance of OntologieGraphGenerator
	 */
	private Constructor<OntologiesGraphGenerator> noParametersConstructor = null;
	
	/**
	 * The name of the class to load as Graph Generator
	 */
	private String classToLoadName = DEFAULT_LOADED_CLASS;
	
	/**
	 * The name of the system property
	 */
	private static final String SYSTEM_PROPERTY_NAME = "graphgenerator.classname";
	
	/**
	 * the name of the class that will be load if no property "graphgenerator.classname" is set
	 */
	private static final String DEFAULT_LOADED_CLASS = "fr.inria.icocoa.servicemodel.semantictype.encoder.jenaencoder.JenaOntologiesGraphGenerator";
	
	/**
	 * Default constructor of the class. Initialise the private field <i>noParametersConstructor</i>
	 * thanks to system property <b>graphgenerator.classname</b>
	 */
	@SuppressWarnings("unchecked")
	private OntologiesGraphGeneratorFactory(){
		String classToLoad = System.getProperty(SYSTEM_PROPERTY_NAME);
		
		//check if property has been defined
		if(classToLoad == null){
			DebugPrinter.print("The system property "+SYSTEM_PROPERTY_NAME+" has not been set. The class "+DEFAULT_LOADED_CLASS+" will be used has OntologyGraphGenerator", DebugPrinter.WARN);
			classToLoad = DEFAULT_LOADED_CLASS;
		}else{
			this.classToLoadName = classToLoad;
		}
				
		try {
			//load the class
			Class<OntologiesGraphGenerator> loadedClass = (Class<OntologiesGraphGenerator>) Class.forName(classToLoad);
			
			//initialise constructor (default constructor)
			this.noParametersConstructor = loadedClass.getConstructor(new Class[]{});
		} catch (ClassNotFoundException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		} catch (SecurityException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		} catch (NoSuchMethodException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		}
		DebugPrinter.print("end of intialisation => this ="+this,DebugPrinter.INFO);
	}
	
	/**
	 * Get the single instance of the Factory
	 * @return the OntologiesGraphGeneratorFactory instance
	 */
	public static OntologiesGraphGeneratorFactory getInstance(){
		if(instance==null)
			instance = new OntologiesGraphGeneratorFactory();
		return instance;
	}

	/**
	 * Get a Graph generator instance 
	 * @return a graph generator instance, <b>null</b> if an error occurs
	 */
	public OntologiesGraphGenerator getGraphGenerator(){
		DebugPrinter.print("start getGraphGenerator => this = "+this,DebugPrinter.ERROR);
		try {
			if(noParametersConstructor!=null){
				DebugPrinter.print("getGraphGenerator => create new Instance",DebugPrinter.ERROR);
				return noParametersConstructor.newInstance();
			}else{
				DebugPrinter.print("noParametersConstructor is null ! (for "+this.classToLoadName+")",DebugPrinter.ERROR);
			}
		
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e,DebugPrinter.ERROR);
		} catch (InstantiationException e) {
			DebugPrinter.print(e,DebugPrinter.ERROR);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e,DebugPrinter.ERROR);
		} catch (InvocationTargetException e) {
			DebugPrinter.print(e,DebugPrinter.ERROR);
		} catch (Exception e){
			DebugPrinter.print(e,DebugPrinter.ERROR);
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return super.toString()+"\n\t - classToLoadName = "+this.classToLoadName + " \n\t - noParametersConstructor = "+noParametersConstructor;
	}
	
	
}
