/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.servicemodel.semantictype.Code;
import fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.toolobjects.Node;
import fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.toolobjects.PrimeNumberGenerator;

/**
 * 
 * Class that implement the Encoder associated to the class
 * PrimeNumberEncodedSemanticType
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 8, 2008 10:03:23 AM
 */
public class PrimeNumberEncoder implements Encoder {

	/**
	 * Default constructor
	 * 
	 */
	public PrimeNumberEncoder() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.Encoder#encode(java.util.List)
	 */
	public Map<String, List<Code>> encode(List<String> listOfOntologies) {

		// get the graph of ontologies
		Node rootNode = OntologiesGraphGeneratorFactory.getInstance()
				.getGraphGenerator().generateOntologiesGraph(listOfOntologies);
				
		// Recursively encodes subtrees (associate a prime to each node)
		this.encodeSubtreeFirstPass(rootNode, 0);
			
		//DebugPrinter.print("result = " + rootNode.displayNode(""),DebugPrinter.INFO);

		//compute all codes
		this.encodeSubtreeSecondPass(rootNode);

		// collect the result of encryption
		Map<String, List<Code>> result = new HashMap<String, List<Code>>();
		rootNode.collectEncodedSemanticType(result);

		// delete graph
		rootNode = null;
		return result;
	}

	/**
	 * First pass of encryption of all children of the rootNode (recursive
	 * method). During this pass, a prime number is associated to each Node.
	 * 
	 * @param parentNode
	 *            the node (encoded) which children has to be encoded
	 * @param currentIndex
	 *            the index of the next prime to use
	 * @return the index of the last prime number used
	 */
	private int encodeSubtreeFirstPass(Node parentNode, int currentIndex) {
		// DebugPrinter.print("currentIndex = "+currentIndex +" list of parents
		// of "+parentNode.getUriOfConcept()+"= "+parentNode.getParents(),
		// DebugPrinter.INFO);
		BigInteger myBigInt  = new BigInteger(PrimeNumberGenerator.getPrime(currentIndex).toString());
		parentNode.setMyOwnCode(new BigIntegerCode(myBigInt));
		
		//if this node has equivalent, the same First need to be associated to each equivalent node
		if(parentNode.getEquivalents().size()>0){
			List<BigInteger> equivalentsValues = new ArrayList<BigInteger>();
			equivalentsValues.add(myBigInt);
			for(Node equivalent : parentNode.getEquivalents()){
				BigIntegerCode equivalentOwnCode = (BigIntegerCode)equivalent.getMyOwnCode();
				if(equivalentOwnCode!=null)
					equivalentsValues.add(((BigIntegerCode)equivalentOwnCode).getValue());				
			}
			
			//get the min of own code
			BigInteger min = Collections.min(equivalentsValues);
			
			//update value of equivalent (and I)
			parentNode.setMyOwnCode(new BigIntegerCode(min));
			for(Node equivalent : parentNode.getEquivalents()){
				equivalent.setMyOwnCode(new BigIntegerCode(min));		
			}			
		}

		// check that the node has child
		if (parentNode.isLeaf()) {
			// nothing more to do
			return currentIndex + 1;
		} else {
			int lastIndex = currentIndex + 1;
			for (Node child : parentNode.getChildren()) {
				lastIndex = this.encodeSubtreeFirstPass(child, lastIndex);
			}
			return lastIndex;
		}
	}

	private void encodeSubtreeSecondPass(Node rootNode) {

		// list of node to treat + initialisation
		LinkedList<Node> toTreat = new LinkedList<Node>();
		rootNode.addCodes(rootNode.getMyOwnCode());
		toTreat.addAll(rootNode.getChildren());

		/*
		 * Treat all Node in the FIFO
		 */
		while (!toTreat.isEmpty()) {
			Node current = toTreat.poll();
			
			//DebugPrinter.print(current.getParents(),"list of parents of "+current,DebugPrinter.INFO);
			
			// check if no code has been already calculated.
			if (current.getCodes().size() == 0) {
				if (current.getParents().size() == 0) {// the root of the tree
					current.addCodes(current.getMyOwnCode());

					// put all children in toTreat
					toTreat.addAll(current.getChildren());
				} else {
					// get the own prime corresponding to the current ont
					BigInteger codeValue = ((BigIntegerCode) current
							.getMyOwnCode()).getValue();

					// get list of code (multiplication of primes) corresponding
					// to
					// the parents
					LinkedList<BigInteger> parentValues = new LinkedList<BigInteger>();
					boolean allParentsFound = true;
					for (Node parent : current.getParents()) {
						if (parent.getCodes().size() != 0) {
							BigIntegerCode toAdd = (BigIntegerCode) parent
									.getCodes().get(0);
							if (toAdd != null){
								parentValues.add(toAdd.getValue());
							}else{
								allParentsFound = false;
								toTreat.add(current);
								break;
							}
						} else {
							// DebugPrinter.print(parent + " has no
							// codes",DebugPrinter.INFO);
							allParentsFound = false;
							toTreat.add(current);
							break;
						}
					}

					//DebugPrinter.print(parentValues, "parentValues",DebugPrinter.INFO);

					if (allParentsFound) {
						if(parentValues.size()>1)
							this.cleanList(parentValues);
						

						//DebugPrinter.print(parentValues,"parentValues (after washing)",DebugPrinter.INFO);

						// compute the code of the current node by multiplying
						// the
						// list of correct integer with the own of the Node
						for (BigInteger value : parentValues) {
							codeValue = codeValue.multiply(value);
						}

						// set the code to the current Node
						current.addCodes(new BigIntegerCode(codeValue));

						// put all children in toTreat
						toTreat.addAll(current.getChildren());

					}
				}

			}
		}

	}

	/**
	 * Remove BigInteger that are multiple of another one in the list of  BigInteger given as parameter
	 * @param listOfBigInt the list of BigInteger to clean
	 */
	private void cleanList(List<BigInteger> listOfBigInt) {
		//sort the list of bigInteger
		Collections.sort(listOfBigInt);
		
		//remove multiple
		int i = 0;
		while(listOfBigInt.size()>0 && i<(listOfBigInt.size()-1)){//we need at least 2 values in the list 
			BigInteger small = listOfBigInt.get(i);
			boolean doRemove = false;//indicate if we have remove a value
			for(int j=i+1; j<listOfBigInt.size();j++){				
				BigInteger big = listOfBigInt.get(j);
				if((big.remainder(small)).intValue()==0){//big is a multiple of small
					listOfBigInt.remove(i);
					doRemove = true;
					break;
				}
			}
			if(!doRemove)
				i++;			
		}		
	}

}
