package fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.toolobjects;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.servicemodel.semantictype.Code;

/**
 * 
 * This class represent a node of the Graph that represent the Ontology
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 31, 2008 10:17:23 AM
 */
public class Node {

	/**
	 * The list of code corresponding to the concept
	 */
	private List<Code> codes = null;

	/**
	 * The list of children (node)
	 */
	private List<Node> children = null;

	/**
	 * The list of equivalents concept (node)
	 */
	private List<Node> equivalents = null;

	/**
	 * The concept associated with this Node
	 */
	private String uriOfConcept = null;
	
	/**
	 * The list parent Node of the current one 
	 */
	private List<Node> parents = null;
	
	/**
	 * A representative code for the node
	 */
	private Code myOwnCode = null;
	
	/**
	 * Default constructor (do nothing)
	 *
	 */
	public Node(){
		super();
	}
	
	/**
	 * Create the node with its associated concept name
	 * 
	 * @param uriOfConcept
	 *            the URI of the conept associated with this node
	 */
	public Node(String uriOfConcept) {
		super();
		this.uriOfConcept = uriOfConcept;
	}

	/**
	 * Get the list of children
	 * 
	 * @return a list of Node corresponding to subclasses of the concept
	 *         associated with the current node, <i>empty list</i> if there is
	 *         no child
	 */
	public List<Node> getChildren() {
		if (this.children != null)
			return children;
		else
			return new LinkedList<Node>();
	}

	/**
	 * Get the list of equivalents
	 * 
	 * @return a list of Node corresponding to equivalents class of the concept
	 *         associated with the current node, <i>empty list</i> if there is
	 *         no equivalents
	 */
	public List<Node> getEquivalents() {
		if (this.equivalents != null)
			return equivalents;
		else
			return new LinkedList<Node>();
	}

	/**
	 * Get the list of codes associated with the current node
	 * 
	 * @return the list of codes, <i>empty list</i> if no codes
	 */
	public List<Code> getCodes() {
		if (codes != null)
			return codes;
		else
			return new LinkedList<Code>();
	}

	/**
	 * @return the uriOfConcept
	 */
	public String getUriOfConcept() {
		return uriOfConcept;
	}
	

	/**
	 * get the list of parents node
	 * @return the list of parents, <i>empty list</i> if no parent
	 */
	public List<Node> getParents() {		
		if(parents != null)
			return parents;
		else return new LinkedList<Node>();
	}

	/**
	 * @return the myOwnCode
	 */
	public Code getMyOwnCode() {
		return myOwnCode;
	}

	/**
	 * @param myOwnCode the myOwnCode to set
	 */
	public void setMyOwnCode(Code myOwnCode) {
		this.myOwnCode = myOwnCode;
	}

	/**
	 * Add a new child node
	 * 
	 * @param child
	 *            the node to add as child
	 */
	public void addChild(Node child) {
		// If it is the first child, instantiates the Vector
		if (this.children == null)
			children = new LinkedList<Node>();

		//if the child is the same as me, or one of my equivalents, don't add as a child
		if(!(this.equals(child)) && !(this.getEquivalents().contains(child)) && !(this.children.contains(child))){
			this.children.add(child);
			child.addParent(this);
		}
	}
	
	/**
	 * Add a new parent node
	 * 
	 * @param parent
	 *            the node to add as parent
	 */
	public void addParent(Node parent) {
		// If it is the first child, instantiates the Vector
		if (this.parents == null)
			parents = new LinkedList<Node>();

		if(!this.parents.contains(parent)){
			this.parents.add(parent);
		}
	}


	/**
	 * Add a new equivalents node
	 * 
	 * @param equivalent
	 *            the node to add as equivalents
	 */
	public void addEquivalent(Node equivalent) {
		// If it is the first child, instantiates the Vector
		if (this.equivalents == null)
			this.equivalents = new LinkedList<Node>();

		//add as equivalents, if the equivalents is not the same as me
		if(!this.equals(equivalent) && !this.equivalents.contains(equivalent)){
			this.equivalents.add(equivalent);
		}
	}

	/**
	 * Add a new Code for the associated concept
	 * 
	 * @param code
	 *            a code to associate with this node
	 */
	public void addCodes(Code code) {
		// If it is the first child, instantiates the Vector
		if (this.codes == null)
			codes = new LinkedList<Code>();
		this.codes.add(code);
	}

	/**
	 * Print on "System.out" the content of the current Node and children node
	 * (recursive method)
	 * 
	 * @param prefix
	 *            the prefix to set before each new line
	 */
	public String displayNode(String prefix) {

		String result = "";
		result += prefix + this.getUriOfConcept() + " ("
				+ this.getCodes().size() + ") {#"+this.getMyOwnCode()+"#} = ";

		/*
		 * Display list of code
		 */
		Iterator<Code> it1 = this.getCodes().iterator();
		while (it1.hasNext()) {
			result += "$" + it1.next().toString() + "$";
		}

		result += "\n";

		/*
		 * Display equivalents Node
		 */
		if (this.getEquivalents() != null) {
			Iterator<Node> it = this.getEquivalents().iterator();
			while (it.hasNext())
				result += prefix + "=" + it.next().getUriOfConcept() + "\n";
		}

		/*
		 * Recursivly launch display of children
		 */
		if (this.getChildren() != null) {
			Iterator<Node> it = this.getChildren().iterator();
			while (it.hasNext())
				result += it.next().displayNode(prefix + ">");
		}

		return result;
	}

	/**
	 * test if the node is a leave of the tree or not
	 * 
	 * @return <b>true</b> if the node is a leaf (=has no child), <b>false</b>
	 *         otherwise
	 */
	public boolean isLeaf() {
		if (this.getChildren() == null)
			return true;
		else
			return this.getChildren().isEmpty();
	}

	/**
	 * Collect all the leave of the graph (recursive method). This method remove
	 * doublon
	 * 
	 * @return the list of Node that have no children, <i>empty list</i> if the
	 *         subgraph starting with the current node has no leaf
	 */
	public List<Node> getAllLeave() {
		List<Node> result = new ArrayList<Node>();
		if (this.isLeaf()) {
			result.add(this);
		} else {
			for (Node child : this.getChildren()) {
				List<Node> subchildren = child.getAllLeave();
				for (Node subchild : subchildren) {
					if (!result.contains(subchild)) {
						result.add(subchild);
					}
				}
			}
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result
				+ ((uriOfConcept == null) ? 0 : uriOfConcept.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Node other = (Node) obj;
		if (uriOfConcept == null) {
			if (other.uriOfConcept != null)
				return false;
			else return true;
		} else return uriOfConcept.equals(other.uriOfConcept);
	}

	/**
	 * This method walk trough the graph so as to collect the data for map
	 * uriOfConcept<->EncodedSemanticType (recursive method)
	 * 
	 * @param result
	 *            the Map where all result will be stored
	 */
	public void collectEncodedSemanticType(
			Map<String, List<Code>> result) {
		//DebugPrinter.print("result.size - "+result.size() + " ; me = " + this.getUriOfConcept(),	DebugPrinter.INFO);

		/*
		 * treatment for current Node
		 */
		if (!result.containsKey(this.getUriOfConcept())) {
			result.put(this.getUriOfConcept(), this.getCodes());
		} else {
			// if some code is already associated with this URI, complete the
			// list of codes
			result.get(this.getUriOfConcept()).addAll(this.getCodes());
		}

		/*
		 * recursive call
		 */
		// stop condition
		if (!this.isLeaf()) {
			// the Node has some children
			for (Node child : this.getChildren()) {
				if(!result.containsKey(child.getUriOfConcept())){
					child.collectEncodedSemanticType(result);
				}
			}
		}

	}

	/**
	 * test if the Node is equals to another one. This means that the two nodes
	 * have the same URI of Concept, the same equivalents and the same list of
	 * children (that need to be the same (&lt;=&gt;recursive call)).<br>
	 * This method don't care about the order of children or equivalents in
	 * there lists.
	 * 
	 * @param other
	 *            the Node to compare with
	 * @return <i>true</i> if both node are "equals", <i>false</i> otherwise
	 */
	public boolean deepEquals(Node other) {
		//DebugPrinter.print("mine = "+this+" , other = "+other, DebugPrinter.INFO);
		
		if (other != null) {
			// check the URI of concept
			if (this.getUriOfConcept()
					.equalsIgnoreCase(other.getUriOfConcept())) {

				/*
				 * Check equivalents
				 */
				// check the size of list
				if (this.getEquivalents().size() == other.getEquivalents().size()) {

					// check pair of children
					boolean isTheSame1 = true;
					Iterator<Node> nodeIterator1 = this.getEquivalents()
							.iterator();
					while (nodeIterator1.hasNext() && isTheSame1) {
						Node myChild = nodeIterator1.next();
						// check that the other children contains the same Node
						if (!other.getEquivalents().contains(myChild)) {
							DebugPrinter.print("The other has not " + myChild
									+ " has equivalents !", DebugPrinter.INFO);
							isTheSame1 = false;
						}
					}
					if (!isTheSame1)
						return false;

					/*
					 * check children
					 */
					// check the size of list
					if (this.getChildren().size() == other.getChildren().size()) {

						// check pair of children
						boolean isTheSame = true;
						Iterator<Node> nodeIterator = this.getChildren()
								.iterator();
						while (nodeIterator.hasNext() && isTheSame) {
							Node myChild = nodeIterator.next();
							int index = -1;
							// check that the other children contains the same
							// Node
							if ((index = other.getChildren().indexOf(myChild)) != -1) {
								// deepCompare of children
								isTheSame = myChild.deepEquals(other
										.getChildren().get(index));
							} else {
								DebugPrinter.print("The other ("+other.getUriOfConcept()+")  has not "
										+ myChild + " has child !",
										DebugPrinter.INFO);
								isTheSame = false;
							}
						}
						return isTheSame;
					} else {
						DebugPrinter.print(
								"The other children list has not the size than mine ("+this+") ! \nmine ["+this.getChildren().size()+"]="
										+ this.getChildren() + " \nother ["+other.getChildren().size()+"]="
										+ other.getChildren() + ")",
								DebugPrinter.INFO);
						return false;
					}
				} else {
					DebugPrinter.print(
							"The other equivalents list has not the same size than mine ("+this+") ! \nmine ["+this.getEquivalents().size()+"]="
									+ this.getEquivalents() + "\nother ["+other.getEquivalents().size()+"]="
									+ other.getEquivalents() + ")",
							DebugPrinter.INFO);
					return false;
				}
			} else {
				DebugPrinter.print(
						"The other has not the same URI of concept ! \nmine ="
								+ this.getUriOfConcept() + "\nother ="
								+ other.getUriOfConcept() + ")",
						DebugPrinter.INFO);
				return false;
			}
		} else {
			DebugPrinter.print("The other object is null ! (current = " + this
					+ ")", DebugPrinter.INFO);
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {		
		return this.getUriOfConcept();
	}
	
	/**
	 * Compare my code with those of the other node
	 * @param other the Node we have to compare codes
	 * @return <b>true</b> if the list of code are equals (without considering order), <b>false</b> otherwise.
	 */
	public boolean compareCodes(Node other){
		return Tools.compareList(this.getCodes(),other.getCodes());
		
	}
	
	/**
	 * Update both node sa as to they have the same list of codes
	 * @param other the Node to synchronise the list of codes with
	 */
	public void synchroniseCodes(Node other){
		//one way sync
		for(Code code : this.getCodes()){
			if(!other.getCodes().contains(code)){
				other.getCodes().add(code);
			}
		}
		
		//other way sync
		for(Code code : other.getCodes()){
			if(!this.getCodes().contains(code)){
				this.getCodes().add(code);
			}
		}
	}
	
	

}
