/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.toolobjects;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Vector;

import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.MeasurementTool;

/**
 * This class is used to generate a list of first prime number. Has the calculation is long, a list is generate once and store in a file. If this file is not found anymore, the process is done another time. <br><br>
 * To be sur that the processis done once, in worst case, this generator implements Singleton design pattern.  
 * 
 * <br>You can set the number of required primes by setting the System property "primes.requirednumber"
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 1, 2008 9:34:38 AM
 */
public class PrimeNumberGenerator {
	
	/**
	 * The single instance of the generator
	 */
	private static PrimeNumberGenerator instance = null;
	
	/**
	 * the path of the file contining the list of prime number
	 */
	private static final String PRIMENUMBERS_FILEPATH = ConfigManager.getInstance().getRepositoryFolder()+ConfigManager.FILE_SEPARATOR+"prime_numbers_list.txt";
	
	/**
	 * The name of the property to look for to set the number of primes to generate
	 */
	private final static String PRIMENUMBER_PROPERTY = "primes.requirednumber";
	
	/**
	 * the number max of prime to generate
	 */
	private static int primes_number_max = 100000;
	
	/**
	 * The list of first prime number < Integer.MAXVALUE
	 */
	private List<Integer> listOfPrimes = new Vector<Integer>();
	
	/**
	 * Static initialisation of the class. This is done once, only when the class is loaded for the first time
	 */
	static{

		/*
		 *check the system propoerty to set the number of primes to generate 
		 */
		String nb_primes = System.getProperty(PRIMENUMBER_PROPERTY);
		if(nb_primes!=null){
			primes_number_max = Integer.parseInt(nb_primes);
		}
		
		//create instance og generator
		PrimeNumberGenerator.instance = new PrimeNumberGenerator();
		
	}
	
	
	/**
	 * @return the instance
	 */
	private static PrimeNumberGenerator getInstance() {
		return PrimeNumberGenerator.instance;
	}

	/**
	 * Default constructor of the class.<br>
	 * Load the file containing the list of first prime numbers. If this file doesn't exist, then the generation is done. <br>
	 * <br><u><b>WARNING :</b></u> The generation process is extremly long.<br>
	 *
	 */
	private PrimeNumberGenerator(){
		//Try to load the file
		this.load();
		
		//Check that some prime have been loaded, otherwise, generate the list of primes
		//if the required number of primes is not reached, continue generation
		if(this.listOfPrimes.isEmpty()){
			this.generateListOfPrimes();
		}else if(this.listOfPrimes.size()<primes_number_max){
			this.completeListOfPrimes();
		}
	}

	/**
	 * This method generate the list of all 100 000 first primes,
	 * store it in the list of primes.<br>
	 * At the end of the process, the list is store in the file<br>
	 * <br><u><b>WARNING :</b></u> The generation process is extremly long.<br>
	 *
	 */
	private void generateListOfPrimes() {
		DebugPrinter.print("Start primes generation. Please Wait !\n"+PrimeNumberGenerator.primes_number_max+" prime number required", DebugPrinter.INFO);
		
		long start = System.currentTimeMillis();
		// First prime number
		this.listOfPrimes.add(new Integer(2));
			
		//continue generation
		this.completeListOfPrimes();
		MeasurementTool.displayTime("Prime Generation : ", System.currentTimeMillis()-start);
		
	}
	
	/**
	 * Get the last value in the map, and continue to generate 
	 *
	 */
	private void completeListOfPrimes(){
		//Primality tests
		//initialize loop with the last value contained into the list 
		for(int p=this.listOfPrimes.get(this.listOfPrimes.size()-1)+1; p<Integer.MAX_VALUE && this.listOfPrimes.size()<primes_number_max; p++)
		{
			
						
			boolean isPrime=true;
			for(Integer currentInt : this.listOfPrimes)
			{
				if((p%currentInt.intValue())==0)
				{
					isPrime=false;
					break;
				}
			}
			
			if(isPrime)
				this.listOfPrimes.add(new Integer(p));
			
			//every 2000 loops diplay a message to show the progress
			if(p%4001==0){
				double pourcentDone = ((100.0*this.listOfPrimes.size())/((double)primes_number_max));
				DebugPrinter.print("List of prime numbers generation : done = "+pourcentDone+"% - currentIndex = "+p+" - Number of generated primes = "+this.listOfPrimes.size(), DebugPrinter.INFO);
			}
		}
		
		//save the list of primes
		this.save();
	}
	
	/**
	 * Load the list of prime from a file
	 *
	 */
	private void load(){
		File primeNumbersFile = new File(PrimeNumberGenerator.PRIMENUMBERS_FILEPATH);
		if(primeNumbersFile.exists() && primeNumbersFile.canRead()){
			//create input stream
			try {
				BufferedReader reader = new BufferedReader(new FileReader(primeNumbersFile));
				String line = null;
				//the file contains a prime number on each line 
				while((line = reader.readLine())!=null){
					try{
						//add the read prime in list
						this.listOfPrimes.add(new Integer(line.trim()));
					}catch(NumberFormatException e){
						DebugPrinter.print(e,DebugPrinter.INFO);
					}					
				}
				reader.close();
			} catch (FileNotFoundException e) {
				DebugPrinter.print(e,DebugPrinter.WARN);
			} catch (IOException e) {
				DebugPrinter.print(e,DebugPrinter.WARN);
			}
		}
	}

	/**
	 * Save the list of primes in a file
	 *
	 */
	private void save() {
		//get the file object from filepath
		File primeNumbersFile = new File(PrimeNumberGenerator.PRIMENUMBERS_FILEPATH);
		if(primeNumbersFile.exists())
			primeNumbersFile.delete();		
		
		try {
			//create the file
			primeNumbersFile.createNewFile();
			
			//create the output stream
			BufferedWriter writer = new BufferedWriter(new FileWriter(primeNumbersFile));
			
			//write data into the file
			for(Integer currentInt : this.listOfPrimes){
				writer.write(currentInt.toString());
				writer.newLine();
			}
			
			//close the stream
			writer.flush();
			writer.close();
			
			
		} catch (IOException e) {
			DebugPrinter.print(e,DebugPrinter.WARN);
		}
	}

	/**
	 * @return the listOfPrimes
	 */
	private List<Integer> getListOfPrimes() {
		return this.listOfPrimes;
	}
	
	/**
	 * Get the list of first prime numbers.<br>
	 * Load the file containing the list of first prime numbers. If this file doesn't exist, then the generation is done. <br>
	 * <br><u><b>WARNING :</b></u> The generation process is extremly long.<br>
	 * @return the list of primes numbers 
	 */
	public static List<Integer> getListOfPrimeNumbers() {
		return PrimeNumberGenerator.getInstance().getListOfPrimes();
	}
	
	/**
	 * Get a prime number (natural growing order)
	 * @param index the index of the required prime in the list of primes
	 * @return the "index"-th first prime number
	 */
	public static Integer getPrime(int index){
		return PrimeNumberGenerator.getListOfPrimeNumbers().get(index);
	}
	



}
